# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
#
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------


"""
Here is the main frame and the main loop of MRS Viewer software.
If you want to add some GUI items, you should write them in a *.interface
module.
"""

import os
import os.path

from utils.preferences import PreferencesDialog
from utils.constants import *
import quest.interface as Qst
import mrsignal.interface as Sig
import mrsignal.procgui as Proc

import wx
from wx.adv import NotificationMessage

# =======================================================================

class MainFrame(wx.Frame):
    """
    The first frame that the user will see. Basicelly, it's just a
    SignalPanel that allow to display a MR signal into several
    ways. See signal.interface.SignalPanel for more details.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        screen_size = wx.DisplaySize()
        super().__init__(*args, **kwargs, size=screen_size)
        self.SetBackgroundColour(BACKGROUND_COLOR)
        self.SetForegroundColour(FOREGROUND_COLOR)

        self.spanel = Sig.SignalPanel(self.logStatus,
                                      self, size=screen_size)
        self.statusbar = StatusBarTime(self)
        self.SetStatusBar(self.statusbar)
        self.initMenu()

        self.Bind(wx.EVT_CLOSE, self.onClose)
        self.Bind(Proc.EVT_END_PREPROC, self.onEndPreproc)

        self.Show()
        self.Center()

        try:
            pref = wx.GetApp().pref
            pref.load()
        except FileNotFoundError:
            self.logStatus("Preferences file was not found. A default \
configuration has been loaded.", 3, 7)
            pref.setDefaultConfig()
            pref.save()

    # -------------------------------------------------------------------
    def initMenu(self)->None:
        """
        Initialize the menubar of the viewer.
        """
        menubar = wx.MenuBar()

        # File menu
        file_menu = wx.Menu()
        new_signal = file_menu.Append(wx.ID_NEW, "Open signal ...", 'help')
        qfolder = file_menu.Append(wx.ID_ANY, "Open QUEST folder...")
        file_menu.AppendSeparator()
        quit_item = file_menu.Append(wx.ID_ANY, "&Quit\tCtrl+Q")

        # Preprocessing submenu
        preproc = wx.Menu()
        scaling = preproc.Append(wx.ID_ANY, 'Scaling')
        phasing = preproc.Append(wx.ID_ANY, 'Phasing')
        shifting = preproc.Append(wx.ID_ANY, 'Shifting')
        damping = preproc.Append(wx.ID_ANY, 'Damping')

        # Edit menu
        edit_menu = wx.Menu()
        edit_menu.Append(wx.ID_ANY, '&Pre-Processing', preproc)
        process_quest = edit_menu.Append(wx.ID_ANY, "Process Quest")
        edit_menu.AppendSeparator()
        pref = edit_menu.Append(wx.ID_ANY, "Preferences")

        # Final menubar
        menubar.Append(file_menu, "&File")
        menubar.Append(edit_menu, "&Edit")
        self.SetMenuBar(menubar)

        # events
        self.Bind(wx.EVT_MENU, self.spanel.onNewSignal, new_signal)
        self.Bind(wx.EVT_MENU, self.onClose, quit_item)
        self.Bind(wx.EVT_MENU, self.onProcQuest, process_quest)
        self.Bind(wx.EVT_MENU, self.onOpenQuestFolder, qfolder)
        self.Bind(wx.EVT_MENU, self.onEditPreferences, pref)

        self.Bind(wx.EVT_MENU, lambda evt: self.onPreproc(evt, 'scale'), scaling)
        self.Bind(wx.EVT_MENU, lambda evt: self.onPreproc(evt, 'phase'), phasing)
        self.Bind(wx.EVT_MENU, lambda evt: self.onPreproc(evt, 'shift'), shifting)
        self.Bind(wx.EVT_MENU, lambda evt: self.onPreproc(evt, 'dampe'), damping)

    # -------------------------------------------------------------------
    def onProcQuest(self, event)->None:
        """
        Launch the window that allow to process quest.
        """
        Qst.ProcessQuestFrame(self)

    # -------------------------------------------------------------------
    def onEndProcess(self, *args)->None:
        """
        Display notifications to let the user know that cQuest has finished.
        """
        NotificationMessage("CQUEST", "QUEST has finished", self).Show(5)
        self.logStatus("QUEST has finished !", 2, 5)
        self.statusbar.resetDefaultMessage()

    # -------------------------------------------------------------------
    def onPreproc(self, event, action):
        """
        Launch a non-modal dialog for pre-processing.

        Args:
            event : wx.Event instance
            action : string in 'scale', 'phase'
        """
        dlg = Proc.PreprocDialogBase.factory(action, self, self.spanel.graph,
                                             self.spanel.signals, self.spanel.param,
                                             title=action)

        self.spanel.onUpdatePlots()
        dlg.Show()

    # -------------------------------------------------------------------
    def onEndPreproc(self, event):
        """
        Refresh graph and parameters when pre processing are finished.
        """
        self.spanel.param.unforcePreprocDisplay()
        self.spanel.param.unregisterObserver(event.dlg)
        if event.dlg.plot.axes is not None:
            event.dlg.plot.remove()
        self.spanel.graph.refresh_legend()
        self.spanel.graph.refresh()

    # -------------------------------------------------------------------
    def onOpenQuestFolder(self, event)->None:
        """
        A QUEST folder named XXX should contain :
            - XXX_Correlation.txt
            - XXX_quest2.txt
            - XXX_quest_back.mrui
            - XXX_quest_orig.mrui
            - XXX_quest_estim.mrui

        For more details about how the display is done, see
        quest.interface.QuestViewerSingleton
        """

        flag = wx.DD_DEFAULT_STYLE|wx.DD_DIR_MUST_EXIST|wx.DD_CHANGE_DIR
        with wx.DirDialog(self, "Choose a directory",
                          style=flag) as dd:
            if dd.ShowModal() == wx.ID_CANCEL:
                return
            dirpath = dd.GetPath()

        prefix = os.path.basename(dirpath)
        long_prefix = os.path.join(dirpath, prefix + '_')

        if (os.path.isfile(long_prefix + 'Correlation.txt') and
                os.path.isfile(long_prefix + 'quest2.txt') and
                os.path.isfile(long_prefix + 'quest_back.mrui') and
                os.path.isfile(long_prefix + 'quest_orig.mrui') and
                os.path.isfile(long_prefix + 'quest_estim.mrui')):
            x, y = self.GetSize()
            Qst.QuestViewerSingleton(dirpath, self, size=(x, y)).Show(True)
        else:
            self.logStatus('Cannot open this folder : missing files or incorrect syntax',
                           4, 5)

    # -------------------------------------------------------------------
    def onClose(self, event)->None:
        """
        Ask a confirm before closing the app.
        """
        dial = wx.MessageDialog(None, 'Are you sure to exit ?', 'Question',
                                wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)
        ret = dial.ShowModal()
        if ret == wx.ID_YES:
            self.Destroy()
        else:
            try:
                event.Veto()
            # Because there's 2 type of event accessing this function ...
            except AttributeError:
                event.Skip()

    # -------------------------------------------------------------------
    def onEditPreferences(self, event)->None:
        """
        Show the preferences editor dialog.
        """
        with PreferencesDialog(wx.GetApp().pref, self.logStatus,
                               self, title="Preferences") as dlg:
            dlg.ShowModal()

    # -------------------------------------------------------------------
    def logStatus(self, msg, lvl=0, timer=-1):
        """
        Show a msg in the status bar.

        Args:
            msg : string, message to display

        Kwargs:
            lvl : 0 <= int < 5, level of log that correspond to a color.
            timer : int, time to display the message in second. If <0, set as
                    a default message.
        """
        self.statusbar.pushMessage(msg, lvl, timer)

# =======================================================================

class StatusBarTime(wx.StatusBar):
    """
    A status bar with time management for each message.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        super().__init__(*args, **kwargs)
        self.current_msg = []

        # A blank initial msg without timer
        # syntax = txt, lvl, timer, id
        msg_init = "", 0, None, None

        self.current_msg.append(msg_init)
        self.refresh()

        # An handler for all the timers
        self.Bind(wx.EVT_TIMER, self.onEndTimer)

    # -------------------------------------------------------------------
    def pushMessage(self, msg, lvl=0, time=-1)->None:
        """
        Push a msg in the status bar.

        Args:
            msg : string, message to display

        Kwargs:
            lvl : 0 <= int < 5, level of log that correspond to a color.
            timer : int, time to display the message in second. If <0, set as
                    a default message.
        """
        self.SetStatusText(msg)
        self.SetBackgroundColour(LOG_LEVEL[lvl])

        if time > 0:
            timer = wx.Timer(self)
            id_ = timer.GetId()
            msg = msg, lvl, timer, id_
            self.current_msg.append(msg)
            timer.StartOnce(time*1e3)
        else:
            msg = msg, color, None, None
            self.current_msg.append(msg)

    # -------------------------------------------------------------------
    def onEndTimer(self, event)->None:
        """
        When the timer is finish, delete the message that has launch the event.
        """
        msg = self.current_msg
        msg[:] = [m for m in msg if event.GetId() != m[3]]
        self.refresh()

    # -------------------------------------------------------------------
    def setDefaultMessage(self, msg, lvl=0)->None:
        """
        Set the default message.

        Args:
            msg : string, message to display

        Kwargs:
            lvl : 0 <= int < 5, level of log that correspond to a color.
        """
        self.current_msg[0] = msg, lvl, None, None
        self.refresh()

    # -------------------------------------------------------------------
    def resetDefaultMessage(self)->None:
        """
        Make the default message empty.
        """
        self.current_msg[0] = ("", 0, None, None)
        self.refresh()

    # -------------------------------------------------------------------
    def refresh(self)->None:
        """
        Redraw the status bar.
        """
        msg = self.current_msg[-1]
        self.SetStatusText(msg[0])
        self.SetBackgroundColour(LOG_LEVEL[msg[1]])

    # -------------------------------------------------------------------
    def popMessage(self) -> None:
        """
        Delete the last message stored.
        """
        self.current_msg.pop()
        self.refresh()

# =======================================================================

if __name__ == "__main__":
    pass
