#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 30 15:13:07 2017
Read a Dicom experiment
@author: yannlefur
"""

import numpy
import math
import pydicom
import struct

# =============================================================================

def load(fname):
    data, meta = readDicomSpec(fname)
    # we only handle voxel, so we store that 4D array in a 2D array
    shape = data.shape
    data.shape = (shape[1]*shape[2]*shape[3], shape[0])

    # add keys that will be used by mrsViewer
    meta['size'] = data.shape[1]
    meta['nbSignal'] = data.shape[0]
    meta['step'] = 1 / meta['SW']
    meta['transFreq'] = meta['SF'] * 1e6
    meta['B0'] = 0
    meta['ppmref'] = 0

    return data, meta

def save(fname):
    raise NotImplementedError("save not implemented for DICOM")

# =============================================================================

# -----------------------------------------------------------------------------
def readDicomSpec(fname):
    datasets = pydicom.read_file(fname)
    privHeader = readSiemensPrivHeader(fname)

    fidNpts = int(privHeader["DataPointColumns"][0])
    csiShape = [fidNpts, int(privHeader["Columns"][0]), int(privHeader["Rows"][0]), int(privHeader["NumberOfFrames"][0])]
    csiAcqShape = [fidNpts, int(privHeader["SpectroscopyAcquisitionPhaseColumns"][0]), \
            int(privHeader["SpectroscopyAcquisitionPhaseRows"][0]), \
            int(privHeader["SpectroscopyAcquisitionOut-of-planePhaseSteps"][0])]
    NAv = int(float(privHeader["NumberOfAverages"][0]))
    TE = float(privHeader["EchoTime"][0])
    SF = float(privHeader["ImagingFrequency"][0])
    Nucleus = privHeader["ImagedNucleus"][0]
    SW = 1e9/float(privHeader["RealDwellTime"][0])
    voxPos = numpy.array(privHeader["VoiPosition"], dtype = float)
    VoiThickness = float(privHeader["VoiThickness"][0])
    VoiPhaseFoV = float(privHeader["VoiPhaseFoV"][0])
    VoiReadoutFoV = float(privHeader["VoiReadoutFoV"][0])
    voxMatrix = numpy.array(privHeader["VoiOrientation"], dtype = float)
    VoiInPlaneRotation  = privHeader["VoiInPlaneRotation"]
    patPos  = numpy.array(privHeader["ImagePositionPatient"], dtype = float)
    imagOrient  = privHeader["ImageOrientationPatient"]
    sliceLocation = privHeader["SliceLocation"]
    sliceThickness = float(privHeader["SliceThickness"][0])

    pixelSpacing = numpy.array(privHeader["PixelSpacing"], dtype = float)
    Fov = pixelSpacing * [int(privHeader["Columns"][0]), int(privHeader["Rows"][0])]
    voxSize = numpy.asarray([VoiReadoutFoV, VoiPhaseFoV, VoiThickness])

    # on calcul la matrice de rotation
    row_cosine = numpy.array(imagOrient[:3], dtype = float)             # afin de determiner la direction correspondant 
    column_cosine = numpy.array(imagOrient[3:], dtype = float)          # aux slices, et ainsi déterminer NSli 
    slice_cosine = numpy.cross(row_cosine, column_cosine)       #(nombre de sliPos différents, et d'orientation différentes)

    patOrient = numpy.identity(3, dtype=numpy.float32)

    patOrient[:3, 0] = row_cosine
    patOrient[:3, 1] = column_cosine
    patOrient[:3, 2] = slice_cosine

    dataSize = csiShape[0]*csiShape[1]*csiShape[2]*csiShape[3]
    data = datasets[0x7fe1, 0x1010]
    fidFormat = "="+str(2*dataSize)+"f"
    rawData = struct.unpack(fidFormat, data.value)
    rawData = numpy.asarray(rawData, dtype = numpy.float64)
    rawData = rawData[..., ::2] - rawData[..., 1::2] * 1.j
    spatSel = "No"  # "PRESS"
    rawData = numpy.reshape(rawData, [csiShape[0], csiShape[1], csiShape[2], csiShape[3]], order = 'F')
    csiNDim = [x > 1 for x in csiAcqShape].count(True)
    if csiNDim == 3 :
        rawData = numpy.fft.fftshift(rawData, axes = [1,2])

        if spatSel == "No" : 
            rawData = numpy.flip(rawData,1)
        if spatSel == "PRESS" :
            rawData = numpy.flip(rawData,2)
        rawData = numpy.fft.ifft2(rawData, axes = (1, 2))

        # Shift 1/2 voxel : 
        indexX = numpy.roll(numpy.arange(-csiShape[1]/2, csiShape[1]/2, dtype = int), int((csiShape[1]+1)/2))
        indexY = numpy.roll(numpy.arange(-csiShape[2]/2, csiShape[2]/2, dtype = int), int((csiShape[2]+1)/2))
        if spatSel == "PRESS" : 
            shiftVectX = numpy.exp(-1.j*numpy.pi*indexX/(csiShape[1]-1))
            shiftVectY = numpy.exp(-1.j*numpy.pi*indexY/(csiShape[2]-1))
        if spatSel == "No" : 
            shiftVectX = numpy.exp(-1.j*numpy.pi*indexX/(csiShape[1]-1))
            shiftVectY = numpy.exp(-1.j*numpy.pi*indexY/(csiShape[2]-1))
        shiftVect = (numpy.outer(shiftVectX, shiftVectY))#

        for i in range(csiShape[0]) :
            for k in range(csiShape[3]) :
                rawData[i, :, :, k] = numpy.squeeze(rawData[i, :, :, k]) * shiftVect
        csiO = rawData[0,:,:,0]

        csiMap = numpy.zeros([256, 256], dtype = numpy.complex64)
        csiMap[0:csiShape[1],0:csiShape[2]] = numpy.fft.ifftshift(csiO)
        csiMap = numpy.roll(csiMap, [-int(csiShape[1]/2), -int(csiShape[2]/2)], axis = [0,1])
        csiMap = numpy.fft.fft2(csiMap)

    oCoil = {"NCoil" : 1}

    oAverage = calcKSpaceSampling (csiAcqShape, NAv)

    hdr = {"SW" : SW, "SF" : SF, "Nucleus" : Nucleus, "TE" : TE , "csiShape" : csiShape, "csiAcqShape" : csiAcqShape, \
            "voxPos" : voxPos, "voxSize" : voxSize, "voxMatrix" : voxMatrix, "oCoil" : oCoil, \
            "oAverage" : oAverage, "fftFlag" : True, "pixelSpacing" : pixelSpacing, "Fov" : Fov, \
            "sliThick" : sliceThickness, "NAv" : NAv, \
            "patPos" : patPos, "patOrient" : patOrient, "tShift" : 0, "format" : "DCM"}

    return rawData, hdr



"""
Created on Mon Oct 30 15:13:07 2017
Read the siemens private header of a spectroscopic experiment
@author: yannlefur
"""


# -----------------------------------------------------------------------------
def readSiemensPrivHeader(fname):
    datasets = pydicom.read_file(fname)

    privHeader = {}
    PH = datasets[0x29, 0x1110]
    Pos = 16
    for elem in range(PH[0]-2):
        Tag = PH[Pos:Pos+63].decode('utf-8','backslashreplace')
        Tag = Tag.split("\x00")
        Tag = Tag[0]
        Pos = Pos + 64
        Vm = int.from_bytes(PH[Pos:Pos + 3], byteorder="little")
        Pos = Pos + 4
        Vr = PH[Pos:Pos + 3].decode().rstrip('\x00')
        Pos = Pos + 4
        Pos = Pos + 4
        NbItems = int.from_bytes(PH[Pos:Pos + 3], byteorder="little")
        Pos = Pos + 4
        Pos = Pos + 4

        if Vm == 0:
            Vm = NbItems
        if NbItems >= 1:                
            tmpData = []
            for z in range(Vm):
                IntItems = int.from_bytes(PH[Pos+12:Pos+12+4], byteorder="little")
                Pos = Pos + 16
                UseFieldWidth = math.ceil(IntItems/4.)*4
                tmpData.append(PH[Pos:Pos + UseFieldWidth].decode().rstrip('\x00'))
                Pos =Pos + UseFieldWidth
            junk_len = 16 * (NbItems - Vm)
            if NbItems < 1:
                junk_len = 4
            elif junk_len < 16  and junk_len != 0:
                junk_len = 16

            Pos = Pos + junk_len
            privHeader[Tag] = tmpData

    return(privHeader)

# -----------------------------------------------------------------------------
def calcKSpaceSampling(csiAcqShape, NAv) :
    csiDim = [x > 1 for x in csiAcqShape].count(True)

    if csiDim == 1 : #------------------Monovoxel--------------
        oAverage = {"avMode": "None", "avSum": NAv,\
                "avList": [1 for i in range(NAv)]}

    elif csiDim == 2 : #  -----------------CSI 1D--------------------
        oAverage = {}

    elif csiDim == 3 :  #  -----------------CSI 2D--------------------
        acqList = numpy.zeros([csiAcqShape[1],csiAcqShape[2]], dtype = numpy.int32)
        colMax = int((csiAcqShape[1]-1)/2)
        rowMax = int((csiAcqShape[2]-1)/2)
        for colNum in range(-colMax, colMax+1) :
            for rowNum in range(-rowMax, rowMax+1) :                
                r = math.sqrt((colNum/(csiAcqShape[1]/2))**2+(rowNum/(csiAcqShape[1]/2))**2)                
                if r <= 1 :

                    acqList[colMax + colNum, rowMax + rowNum] = \
                            (0.5 + (NAv-1) * (0.5 + 0.5 * math.cos(math.pi*r)) + 1)
        oAverage = {"avMode": "Weighted_av", "avSum": acqList.sum(), \
                "avList": acqList}

        return(oAverage)


# -----------------------------------------------------------------------------
if __name__ == '__main__':
    pass
