# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Load and save function for MRUI MRS files.
"""

import struct
import numpy as np

# -------------------------------------------------------------------
def load(filename) -> (np.ndarray, dict):
    # read big endian double type
    data = np.fromfile(filename, dtype='>d')
    meta = _readHeader(data[:64])

    nb_pts = meta['size']
    nbSignal = meta['nbSignal']

    sig = data[64:64+2*nb_pts*nbSignal]
    sig = sig[0::2] + 1j*sig[1::2]
    sig.shape = (nbSignal, nb_pts)

    return sig, meta

# -------------------------------------------------------------------
def _readHeader(header) -> dict:
    info = {}
    info['size'] = int(header[1])
    info['step'] = header[2] / 1000
    info['transFreq'] = header[5]
    info['B0'] = header[6]
    info['ppmref'] = header[9]
    info['nbSignal'] = int(header[13])
    return info

# -------------------------------------------------------------------
def save(sig, filename) -> None:
    """
    Input :
        - sig : a Signal instance to save as a mrui file
        - filename : where to save sig
    """
    with open(filename, 'wb') as dfile:
        _saveHeader(dfile, sig.meta)

        list_data = sig.data.flatten()
        real = (r for r in np.real(list_data))
        imag = (i for i in np.imag(list_data))
        final_list = []
        while True:
            try:
                final_list.append(next(real))
                final_list.append(next(imag))
            except StopIteration:
                break

        for elem in final_list:
            dfile.write(struct.pack('>d', elem))

# -------------------------------------------------------------------
def _saveHeader(dfile, meta) -> None:
    try:
        signal = 0
        nop = meta['size']
        stepTime = meta['step'] * 1000
        beginTime = 0
        ph0 = 0
        transmFreq = meta['transFreq']
        B0 = meta['B0']
        nucleus = 0
        hzref = 0
        ppmref = meta['ppmref']
        echoes = 0
        apodizeView = 0
        zeroFillingView = 0
        SignalsNb = meta['nbSignal']
    except KeyError:
        raise SyntaxError("Meta data are missing for saving this signal in JMRUI.")

    dfile.write(struct.pack('>d', signal))
    dfile.write(struct.pack('>d', nop))
    dfile.write(struct.pack('>d', stepTime))
    dfile.write(struct.pack('>d', beginTime))
    dfile.write(struct.pack('>d', ph0))
    dfile.write(struct.pack('>d', transmFreq))
    dfile.write(struct.pack('>d', B0))
    dfile.write(struct.pack('>d', nucleus))
    dfile.write(struct.pack('>d', hzref))
    dfile.write(struct.pack('>d', ppmref))
    dfile.write(struct.pack('>d', echoes))
    dfile.write(struct.pack('>d', apodizeView))
    dfile.write(struct.pack('>d', zeroFillingView))
    dfile.write(struct.pack('>d', SignalsNb))
    dfile.write(struct.pack('>d', echoes))
    dfile.write(struct.pack('>d', echoes))
    dfile.write(struct.pack('>d', echoes))
    for k in range(47):
        dfile.write(struct.pack('>d', 0))

if __name__ == "__main__":
    pass
