# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Simple widgets used in mrsignal.interface module.
"""

from utils.constants import SHOWCASE_COLOR, WHITE
from utils import Subject

import wx
from matplotlib.lines import Line2D

# =======================================================================

class SignalDescription(wx.Panel):
    """
    Display a box with some info about the signal.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        super(SignalDescription, self).__init__(*args, **kwargs)
        self.initGui()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        self.sizer = wx.StaticBoxSizer(wx.VERTICAL, self, "Signal information")
        self.size = wx.StaticText(self)
        self.trans_freq = wx.StaticText(self)
        self.nb_signal = wx.StaticText(self)
        self.sw = wx.StaticText(self)
        self.reset()

        self.sizer.Add(self.size, 1, wx.ALL, 5)
        self.sizer.Add(self.nb_signal, 1, wx.ALL, 5)
        self.sizer.Add(self.trans_freq, 1, wx.ALL, 5)
        self.sizer.Add(self.sw, 1, wx.ALL, 5)
        self.SetSizer(self.sizer)

    # -------------------------------------------------------------------
    def loadMeta(self, meta, name=None)->None:
        """
        Args :
            - meta : a dictionary from a Signal()
        Kwargs :
            - name : set the box title to `name`.
        """

        try:
            freq = round(meta['transFreq']/1e6, 4)
        except TypeError:
            freq = None

        if 'SW' in meta:
            sw = meta['SW']
        elif meta['step']:
            sw = 1/meta['step']
        else:
            sw = None

        if name:
            self.sizer.GetStaticBox().SetLabel(' - {} - '.format(name))

        self.size.SetLabel("Size :" + str(meta['size']))
        self.nb_signal.SetLabel("Number of signals : " + str(meta['nbSignal']))
        self.trans_freq.SetLabel("Transmission freq. : {} MHz".format(freq))
        self.sw.SetLabel("Spectral width : {}".format(sw))

        self.Layout()

    # -------------------------------------------------------------------
    def reset(self) -> None:
        """
        Make empty the text infos
        """
        self.sizer.GetStaticBox().SetLabel('Signal infos')
        self.size.SetLabel("Size :")
        self.trans_freq.SetLabel("Transmission frequency:")
        self.nb_signal.SetLabel("Number of signals :")
        self.sw.SetLabel("Spectral width :")

        self.Layout()



# =======================================================================

class DisplayParam(wx.Panel, Subject):
    """
    GUI interface to choose how to display a signal (1/2/3D, how many points,
    ...)
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        super(DisplayParam, self).__init__(*args, **kwargs)
        Subject.__init__(self)

        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        # elements
        self.imag_option = wx.RadioBox(self,
                                       label="Complex numbers display management",
                                       choices=["Real", "Imaginary", "Module"],
                                       style=wx.RA_HORIZONTAL)

        self.is_1d = wx.RadioButton(self, label="1D Display", style=wx.RB_GROUP)
        d1box = wx.Panel(self, style=wx.BORDER_SIMPLE)
        self.signal_type = wx.RadioBox(d1box, label="Type of signal",
                                       choices=["Natural", "Fourier"],
                                       style=wx.RA_HORIZONTAL)
        self.xunit = wx.RadioBox(d1box, label="X axe unit",
                                 choices=["Hz", "PPM"],
                                 style=wx.RA_HORIZONTAL)
        point_txt = wx.StaticText(d1box, label="Index of the current signal :")
        self.idx_sig = wx.SpinCtrl(d1box, min=0, max=0)

        self.is_2d = wx.RadioButton(self, label="2D Display")
        d2box = wx.Panel(self, style=wx.BORDER_SIMPLE)
        coef_text_2d = wx.StaticText(d2box, label="Contour coefficient :")
        self.lvl_coef = wx.Slider(d2box, value=10, minValue=1,
                                  maxValue=50)
        level_text_2d = wx.StaticText(d2box, label="Number of levels :")
        self.lvl_nb = wx.SpinCtrl(d2box, max=20, value='10')

        self.is_dyn = wx.RadioButton(self, label="3D display")
        dynabox = wx.Panel(self, style=wx.BORDER_SIMPLE)
        range_text_dyn = wx.StaticText(dynabox, label="Display signals from")
        self.min_range_dyn = wx.SpinCtrl(dynabox, max=0)
        to_text_dyn = wx.StaticText(dynabox, label="to")
        self.max_range_dyn = wx.SpinCtrl(dynabox, max=0)
        step_text_dyn = wx.StaticText(dynabox, label="by step")
        self.step_dyn = wx.SpinCtrl(dynabox, min=1, max=1, value='1')

        refresh = wx.Button(self, label="Apply")
        refresh.SetBackgroundColour(SHOWCASE_COLOR)

        # event
        self.is_1d.Bind(wx.EVT_RADIOBUTTON, self.on1dDisplayType)
        self.signal_type.Bind(wx.EVT_RADIOBOX, self.onChangeSignalType)
        self.xunit.Bind(wx.EVT_RADIOBOX, self.notify)
        self.idx_sig.Bind(wx.EVT_SPINCTRL,
                          lambda evt: self.notify(keep_view=True,
                                                  newIndex=evt.GetPosition()))

        self.is_2d.Bind(wx.EVT_RADIOBUTTON, self.on2dDisplayType)
        self.lvl_coef.Bind(wx.EVT_SCROLL_CHANGED,
                           lambda evt: self.notify(keep_view=True))
        self.lvl_nb.Bind(wx.EVT_SPINCTRL,
                         lambda evt: self.notify(keep_view=True))

        self.is_dyn.Bind(wx.EVT_RADIOBUTTON, self.on3dDisplayType)
        self.min_range_dyn.Bind(wx.EVT_SPINCTRL, self.onChangeMinDyn)
        self.max_range_dyn.Bind(wx.EVT_SPINCTRL, self.onChangeMaxDyn)
        self.step_dyn.Bind(wx.EVT_SPINCTRL,
                           lambda evt: self.notify(keep_view=True))

        refresh.Bind(wx.EVT_BUTTON, self.notify)
        self.imag_option.Bind(wx.EVT_RADIOBOX, self.notify)

        # sizers
        self.d1boxs = wx.BoxSizer(wx.VERTICAL)
        self.d1boxs.Add(self.signal_type, 0, wx.ALL, 2)
        self.d1boxs.Add(self.xunit, 0, wx.ALL, 2)
        self.d1boxs.Add(point_txt, 0, wx.ALL, 2)
        self.d1boxs.Add(self.idx_sig, 0, wx.ALL, 2)
        self.d1boxs.Hide(1)
        d1box.SetSizer(self.d1boxs)
        d1box.SetBackgroundColour(WHITE)

        d2boxs = wx.BoxSizer(wx.VERTICAL)
        d2boxs.Add(coef_text_2d, 0, wx.ALL, 2)
        d2boxs.Add(self.lvl_coef, 0, wx.EXPAND|wx.ALL, 2)
        d2boxs.Add(level_text_2d, 0, wx.ALL, 2)
        d2boxs.Add(self.lvl_nb, 0, wx.ALL, 2)
        d2box.SetSizer(d2boxs)
        d2box.SetBackgroundColour(WHITE)

        self.dynbox = wx.BoxSizer(wx.VERTICAL)
        self.dynbox.Add(range_text_dyn, 0, wx.ALL, 2)
        self.dynbox.Add(self.min_range_dyn, 0, wx.ALL, 2)
        self.dynbox.Add(to_text_dyn, 0, wx.ALL, 2)
        self.dynbox.Add(self.max_range_dyn, 0, wx.ALL, 2)
        self.dynbox.Add(step_text_dyn, 0, wx.ALL, 2)
        self.dynbox.Add(self.step_dyn, 0, wx.ALL, 2)
        dynabox.SetSizer(self.dynbox)
        dynabox.SetBackgroundColour(WHITE)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.imag_option, 0, wx.ALL, 2)
        self.sizer.Add(self.is_1d, 0, wx.ALL, 2)
        self.sizer.Add(d1box, 1, wx.ALL | wx.EXPAND, 2)
        self.sizer.Add(self.is_2d, 0, wx.ALL, 2)
        self.sizer.Add(d2box, 1, wx.ALL | wx.EXPAND, 2)
        self.sizer.Add(self.is_dyn, 0, wx.ALL, 2)
        self.sizer.Add(dynabox, 1, wx.ALL | wx.EXPAND, 2)
        self.sizer.Add(refresh, 0, wx.ALL, 4)

        self.sizer.Hide(4)
        self.sizer.Hide(6)

        self.SetSizer(self.sizer)

    # -------------------------------------------------------------------
    def on1dDisplayType(self, event)->None:
        """
        Show the 1d optons panel, hide the 2d and 3d one.
        """
        self.sizer.Show(2)
        self.sizer.Hide(4)
        self.sizer.Hide(6)
        self.Layout()

    # -------------------------------------------------------------------
    def on2dDisplayType(self, event)->None:
        """
        Show the 2d optons panel, hide the 1d and 3d one.
        """
        self.sizer.Hide(2)
        self.sizer.Show(4)
        self.sizer.Hide(6)
        self.Layout()

    # -------------------------------------------------------------------
    def on3dDisplayType(self, event)->None:
        """
        Show the 3d optons panel, hide the 1d and 2d one.
        """
        self.sizer.Hide(2)
        self.sizer.Hide(4)
        self.sizer.Show(6)
        self.Layout()

    # -------------------------------------------------------------------
    def setMaxSig(self, maxi)->None:
        """
        Set max for all widgets that are related to a Signal().meta['nbSignal'].

        Args:
            maxi : the max to set. 
        """
        self.idx_sig.SetMax(maxi-1)
        self.min_range_dyn.SetMax(maxi-1)
        self.max_range_dyn.SetMax(maxi-1)

    # -------------------------------------------------------------------
    def onChangeSignalType(self, event) -> None:
        """
        Show x axes options if 'Frequency' domain is selected.
        Hide if 'Time' domain is selected.
        """
        if self.signal_type.GetSelection() == 1:
            self.d1boxs.Show(1)
        else:
            self.d1boxs.Hide(1)
        self.Layout()
        self.notify()

    # -------------------------------------------------------------------
    def onChangeMinDyn(self, event) -> None:
        """
        Update step and max in the 3D panel.
        Notify its Observers.
        """
        self.max_range_dyn.SetMin(event.GetPosition())
        self.step_dyn.SetMax(self.max_range_dyn.GetValue() - self.min_range_dyn.GetValue() + 1)
        self.notify(keep_view=True)

    # -------------------------------------------------------------------
    def onChangeMaxDyn(self, event) -> None:
        """
        Update step in the 3D panel.
        Notify its Observers.
        """
        self.step_dyn.SetMax(self.max_range_dyn.GetValue() - self.min_range_dyn.GetValue() + 1)
        self.notify(keep_view=True)

    # -------------------------------------------------------------------
    def resetMaxMin(self) -> None:
        """
        Reset max and min for all the wx.SpinCtrl of this class to 0 or 1.
        """
        self.idx_sig.SetMax(0)
        self.min_range_dyn.SetMax(0)
        self.max_range_dyn.SetMax(0)
        self.step_dyn.SetMax(1)

    # -------------------------------------------------------------------
    def resetValue(self, maxi)->None:
        """
        Reset value for all the wx.SpinCtrl of this class to 0 or 1.
        """
        self.idx_sig.SetValue(0)
        self.min_range_dyn.SetValue(0)
        self.max_range_dyn.SetValue(0)
        self.step_dyn.SetValue(1)

    # -------------------------------------------------------------------
    def forcePreprocDisplay(self):
        """
        Force display to be in 1d and frequency domain.
        """
        self.is_1d.SetValue(True)
        self.is_1d.Disable()
        self.is_2d.Disable()
        self.is_dyn.Disable()
        self.on1dDisplayType(None)

        self.signal_type.SetSelection(1)
        self.d1boxs.Show(1)
        self.signal_type.Disable()

        self.Layout()

    # -------------------------------------------------------------------
    def unforcePreprocDisplay(self):
        """
        Unlock 1/2/3d and domain choice.
        """
        self.is_1d.Enable()
        self.is_2d.Enable()
        self.is_dyn.Enable()
        self.signal_type.Enable()


# =======================================================================

if __name__ == '__main__':
    pass
