# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Ready-to-use widgets to deal with MR signals
plus simple model that represent stored signal.
"""

import os.path
import numpy as np

import matplotlib
matplotlib.use('WXAgg')

from mpl_toolkits.mplot3d import axes3d
from matplotlib.collections import LineCollection
import wx

from . import guitools as Gui
from .voxel import Signal
from utils import *

# =======================================================================

class SignalAndPlot:
    """
    Simple model for the viewer that represent a file stored in the app.
    Store a Signal() instance, a matplotlib class instance which represent the
    displayed signal, the index of the currently displayed signal if 1d, and the
    name of this file.
    """

    # -------------------------------------------------------------------
    def __init__(self, sig, plot=None, index=0, name=None) -> None:
        self.signal = sig
        self.plot = plot
        self.index = index
        if name is None:
            self.name = os.path.splitext(os.path.basename(sig.path))[0]
        else:
            self.name = name

    # -------------------------------------------------------------------
    def delPlot(self, set_none=True) -> bool:
        """
        Remove the plot from its canvas. Return True if there was a plot
        to delete, False otherwise.
        """
        try:
            self.plot.remove()
        except Exception:
            return False

        if set_none:
            self.plot = None

        return True

    # -------------------------------------------------------------------
    def isPlotted(self) -> bool:
        """
        Return True if plor is not None and it belong to a matplotlib axes
        instance.
        True can be return even if they are not visible (see
        matplotlib.Artist.get/set_visible() )
        """
        if self.plot is None or self.plot.axes is None:
            return False
        else:
            return True

# ===========================
class SimpleSignalModel(list):
    """
    A list that encapsulate SignalAndPlot.
    """
    # -------------------------------------------------------------------
    def __init__(self):
        super().__init__()

    # -------------------------------------------------------------------
    def append(self, sig, plot=None, index=0, name=None):
        """
        Overriden
        """
        super().append(SignalAndPlot(sig, plot, index, name))

    # -------------------------------------------------------------------
    def pop(self, index):
        """
        Overriden. Remove the plot before.
        """
        self[index].delPlot()
        super().pop(index)

    # -------------------------------------------------------------------
    def __delitem__(self, index):
        """
        Overriden. Remove the plot before.
        """
        self[index].delPlot()
        super().__delitem__(index)

# =======================================================================

class SignalPanel(wx.Panel, Observer):
    """
    The main panel that allow to select a signal in a list and display it in
    several ways. This is in some ways the core of the main application.

    *IMPORTANT* : a very important assertion in this class is
    that the model and widgets list must have their data in the *same order* !
    """

    # -------------------------------------------------------------------
    def __init__(self, logStatus, *args, **kwargs)->None:
        super().__init__(*args, **kwargs)
        self.initGui()
        self.logStatus = logStatus
        self.signals = SimpleSignalModel()

        self.param.registerObserver(self)

        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        # elements
        self.param = Gui.DisplayParam(self)
        self.graph = CanvasPanel(self)

        sig_panel = wx.Panel(self)
        self.list_signal = PimpedListCtrl(sig_panel, ['Path'], "Signals", False)
        self.sig_descr = Gui.SignalDescription(sig_panel)

        # sizer
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(self.list_signal, 1, wx.EXPAND)
        vbox.Add(self.sig_descr, 0, wx.EXPAND | wx.RIGHT | wx.BOTTOM, 15)
        sig_panel.SetSizerAndFit(vbox)

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(self.param, 1, wx.ALL | wx.EXPAND)
        self.sizer.Add(self.graph, 4, wx.ALL | wx.EXPAND, 4)
        self.sizer.Add(sig_panel, 1, wx.BOTTOM | wx.EXPAND)

        self.SetSizer(self.sizer)

        # events
        self.list_signal.Bind(EVT_PMPD_LIST_EVENT, self._listEvtHandler)
        self.list_signal.Bind(wx.EVT_LIST_ITEM_SELECTED, self.onSelected)
        self.list_signal.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.onDeselected)
        self.list_signal.Bind(wx.EVT_LIST_ITEM_FOCUSED, self.onChangeFocus)

    # -------------------------------------------------------------------
    def _listEvtHandler(self, event):
        """
        Handler for events generated by the 'plus' button.
        """
        if event.action == 'add':
            self.onNewSignal()
        elif event.action == 'del':
            self.onDltSignal()
        event.Skip()

    # -------------------------------------------------------------------
    def onNewSignal(self, *args, pathnames=None)->None:
        """
        Open a file dialog and add a file to the model and the widgets.
        """
        if pathnames==None:
            pathnames = self.openFileDialog(SIGNAL_WILDCARDS)
        for path in pathnames:
            index = self.list_signal.list.FindItem(0, path)

            try:
                if index != wx.NOT_FOUND:
                    self.logStatus(path + " : This file has already been loaded.", 3, 5)
                else:
                    sig = Signal(path)
                    self.list_signal.list.Append([path])
                    self.signals.append(sig)

            except NotImplementedError as error: # raise by Signal if couldn't read
                self.logStatus(error.args[0], 4, 5)

    # -------------------------------------------------------------------
    def onDltSignal(self)->None:
        """
        Delete the current selectioned files in the model and the widgets.
        """
        sel = self.list_signal.GetSelections()
        for i in reversed(sel):
            self.list_signal.Delete(i)
            del self.signals[i]

        self.graph.refresh_legend()
        self.graph.canvas.draw()

    # -------------------------------------------------------------------
    def openFileDialog(self, extension)->list:
        """
        Open a file fialog designed to open files.

        Args:
            - extension : string, the wildcards that will be used.
        Return : 
            - a list of selected paths.
        """
        pathname = None
        with wx.FileDialog(self, "Open a signal file", wildcard=extension,
                           style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST |
                           wx.FD_CHANGE_DIR | wx.FD_MULTIPLE) as filedialog:
            if filedialog.ShowModal() == wx.ID_CANCEL:
                return []

            pathnames = filedialog.GetPaths()
            return pathnames

    # -------------------------------------------------------------------
    def onChangeFocus(self, event) -> None:
        """
        When change the focus, display the meta datas.
        Different from selection.
        """
        index = event.GetIndex()
        data = self.signals[index]
        self.sig_descr.loadMeta(data.signal.meta, data.name)

    # -------------------------------------------------------------------
    def onSelected(self, event) -> None:
        """
        Draw the selected file according to the current display parameters.
        """
        wx.BeginBusyCursor()

        index = event.GetIndex()
        data = self.signals[index]
        self.param.setMaxSig(data.signal.meta["nbSignal"])

        imag = self.param.imag_option.GetSelection()

        # 1D
        if self.param.is_1d.GetValue():
            x_temp = (self.param.signal_type.GetSelection() == 0)
            x_freq = (self.param.xunit.GetSelection() == 0)

            self.draw1dSignal(data, x_temp, x_freq, imag, False)

        # 2D
        elif self.param.is_2d.GetValue():
            coef = self.param.lvl_coef.GetValue() / 10
            nb = self.param.lvl_nb.GetValue()
            try:
                self.draw2dSignal(data.signal, coef, nb, imag, False)
            except Exception as err:
                print('error {}'.format(err))
                msg = "Something went wrong when drawing a 2d representation." \
                      "Verify that the current signal is at least 2x2."
                self.logStatus(msg, 4, 4)
                wx.EndBusyCursor()
                return
        # 3D
        else:
            first = self.param.min_range_dyn.GetValue()
            last = self.param.max_range_dyn.GetValue()
            step = self.param.step_dyn.GetValue()
            imag = self.param.imag_option.GetSelection()

            self.drawDynSignal(data.signal, first, last, step, imag, False)

        wx.EndBusyCursor()


    # -------------------------------------------------------------------
    def onDeselected(self, event) -> None:
        """
        Remove the plot of the deselected file.
        """
        index = event.GetIndex()
        if self.signals[index].delPlot():
            self.graph.refresh_legend()
            self.graph.refresh()

    # -------------------------------------------------------------------
    def draw1dSignal(self, data, x_temp, x_freq, imag, camera)->None:
        """
        Manage camera and display the datas with plot.

        Args:
            - data : SignalAndPlot instance to display
            - x_temp : bool, True if time domain, False otherwise.
            - x_freq : bool, True if frequency domain measured with Hz.
            - imag : int in [0, 1, 2], how complex numbers are displayed.
                     0 for real part, 1 for imaginary and 2 for module.
            - camera : None, or a 2-values tuple representing bounds for
                       matplotlib graph.
        """
        # Manage camera
        was_2d = self.graph.axes.get_label() == '2d'
        was_1d = self.graph.axes.get_label() == '1d'
        if was_2d:
            self.graph.axes.clear()
            self.graph.axes.set_label('1d')
        elif not was_1d: # <-> was 3d
            self.graph.figure.delaxes(self.graph.axes)
            self.graph.axes = self.graph.figure.add_subplot(111, label='1d')

        # plot
        idx_m = data.index
        data.plot, = self.graph.draw1dSignal(data.signal, x_temp, x_freq, imag, idx_m)
        data.plot.set_label(data.name)
        self.graph.refresh_legend()

        if not was_1d:
            self.graph.toolbar.update()

        if camera:
            self.graph.axes.set_xlim(*camera[0])
            self.graph.axes.set_ylim(*camera[1])
            self.graph.canvas.draw()
        else:
            self.graph.refresh()

    # -------------------------------------------------------------------
    def draw2dSignal(self, signal, coef, lvl, imag, camera)->None:
        """
        Manage camera and display the datas with contour.

        Args:
            - signal : Signal instance to diplay
            - coef : number > 0, the bigger it is, the more details are displayed. 
            - lvl : int, the number of contour to display
            - imag : int in [0, 1, 2], how complex numbers are displayed.
                     0 for real part, 1 for imaginary and 2 for module.
            - camera : None, or a 2-values tuple representing bounds for
                       matplotlib graph.
        """
        # Manage camera
        was_2d = (self.graph.axes.get_label() == '2d')
        was_1d = (self.graph.axes.get_label() == '1d')
        if was_2d:
            self.graph.axes.clear()
        elif was_1d:
            self.graph.axes.clear()
            self.graph.axes.set_label('2d')
        else:
            self.graph.figure.delaxes(self.graph.axes)
            self.graph.axes = self.graph.figure.add_subplot(111, label='2d')

        # utils
        last = signal.meta['nbSignal']
        size = signal.meta['size']
        df2 = np.fft.fftshift(np.fft.fft2(signal.data))

        # not sure of what this line is doing but it's better
        # see https://github.com/jjhelmus/nmrglue/wiki/Reading-Bruker-processed-files
        cl = df2.std() * coef * 1.2 ** np.arange(lvl)

        df2.shape = (last, size)
        if imag == 0:
            df2 = df2.real
        elif imag == 1:
            df2 = df2.imag
        else:
            df2 = np.abs(df2)

        # Actual display
        y, x = np.mgrid[0:last, 0:size]
        contour = self.graph.axes.contour(x, y, df2, cl)

        if not was_2d:
            self.graph.toolbar.update()
        if camera:
            self.graph.axes.set_xlim(*camera[0])
            self.graph.axes.set_ylim(*camera[1])

        self.graph.canvas.draw()
        self.graph.Layout()

        return contour

    # -------------------------------------------------------------------
    def drawDynSignal(self, signal, first, last, step, imag, camera)->None:
        """
        Manage camera and display the datas with contour.

        Args:
            - signal : Signal instance to display
            - first : int, index of the first signal to display from `signal`
            - last : int, index of the last signal to display from `signal`
            - step : int, index display step
            - imag : int in [0, 1, 2], how complex numbers are displayed.
                     0 for real part, 1 for imaginary and 2 for module.
            - camera : None, or a 2-values tuple representing bounds for
                       matplotlib graph.
        """
        # Manage camera
        was_3d = (self.graph.axes.get_label() == '3d')
        if was_3d:
            self.graph.axes.clear()
        else:
            self.graph.figure.delaxes(self.graph.axes)
            self.graph.axes = self.graph.figure.add_subplot(111,
                    projection='3d', label='3d')
            camera = None

        # Data
        xs = signal.meta['size']
        zs = np.arange(first, last+1, step)
        x = np.arange(0, xs, 1)
        verts = []
        minz = 0
        maxz = 0

        if imag == 0:
            signal = signal.data_fourier.real
        elif imag == 1:
            signal = signal.data_fourier.imag
        else:
            signal = np.abs(signal.data_fourier)

        for z in zs:
            y = signal[z]
            verts.append(list(zip(x, y)))
            minz = min(y.min(), minz)
            maxz = max(y.max(), maxz)

        # Actual display
        black = (0, 0, 0, 1)
        coll = LineCollection(verts, colors=black, linewidths=0.5)
        self.graph.axes.add_collection3d(coll, zs=zs, zdir='y')

        self.graph.axes.set_xlabel('X')
        self.graph.axes.set_xlim3d(0, xs)
        self.graph.axes.set_ylabel('Y')
        self.graph.axes.set_ylim3d(first, last)
        self.graph.axes.set_zlabel('Z')
        self.graph.axes.set_zlim3d(minz, maxz)

        if not was_3d:
            self.graph.toolbar.update()
        if camera:
            self.graph.axes.view_init(*camera)

        self.graph.canvas.draw()
        self.graph.Layout()

        return coll

    # -------------------------------------------------------------------
    def onUpdatePlots(self, keep_view=False, *args, **kwags) -> None:
        """
        Refresh all the current plotted files according to the current display
        parameters.

        Kwargs :
            - keep_view : bool, if True keep the former bounds of the graphs.
        """
        # unwanted case
        if len(self.signals) < 1:
            self.logStatus("No signal loaded", 4, 5)
            self.sig_descr.reset()
            self.param.resetMaxMin()
            return
        sel = self.list_signal.GetSelections()
        if len(sel) == 0:
            return

        # manage camera
        camera = None
        if keep_view:
            try:
                camera = (self.graph.axes.elev,
                          self.graph.axes.azim)
            except AttributeError:
                camera = (self.graph.axes.get_xlim(),
                          self.graph.axes.get_ylim())

        # Display
        imag = self.param.imag_option.GetSelection()
        # 1D 
        if self.param.is_1d.GetValue():
            for i in sel:
                data = self.signals[i]
                if data.isPlotted():
                    data.delPlot(set_none=False)
                    x_temp = (self.param.signal_type.GetSelection() == 0)
                    x_freq = (self.param.xunit.GetSelection() == 0)

                    self.draw1dSignal(data, x_temp, x_freq, imag, camera)

        # 2D
        elif self.param.is_2d.GetValue():
            coef = self.param.lvl_coef.GetValue() / 10
            nb = self.param.lvl_nb.GetValue()

            i = self.list_signal.GetSelection()
            data = self.signals[i]

            try:
                self.draw2dSignal(data.signal, coef, nb, imag, camera)
            except Exception as err:
                print('error {}'.format(err))
                msg = "Something went wrong when drawing a 2d representation." \
                      "Verify that the current signal is at least 2x2."
                self.logStatus(msg, 4, 4)
                wx.EndBusyCursor()
                return
        # 3D
        else:
            first = self.param.min_range_dyn.GetValue()
            last = self.param.max_range_dyn.GetValue()
            step = self.param.step_dyn.GetValue()
            imag = self.param.imag_option.GetSelection()

            i = self.list_signal.GetSelection()
            data = self.signals[i]

            self.drawDynSignal(data.signal, first, last, step, imag, camera)

    # -------------------------------------------------------------------
    def update(self, *args, **kwargs):
        """
        Override of utils.observer.Observer.update
        """
        if 'newIndex' in kwargs:
            index = kwargs['newIndex']
            i = self.list_signal.GetSelection()
            self.signals[i].index = index

        self.onUpdatePlots(**kwargs)

# =======================================================================

if __name__ == "__main__":
    pass
