# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
PROC MODULE

All functions for pre processing are implemented here.
"""

import numpy as np
from numpy import fft

# =======================================================================

# PHASING #

# -------------------------------------------------------------------
def getPosModuleMax(signal, min_=0, max_=None) -> int:
    """
    Get the index of the maximum modul within signal.

    Args:
        - signal : 1D array of complex value
    Kwargs :
        - min_ : the min bounds for where to search
        - max_ : the min bounds for where to search. If None, set to len(signal) 

    Return :
        index of where the modul is max
    """
    if max_ is None:
        max_ = len(signal) - 1
    return np.argmax(np.abs(signal[min_:max_]))

# -------------------------------------------------------------------
def getPhase(cplx) -> complex:
    """
    Return phase for a complex number.
    """
    return np.angle(cplx)

# -------------------------------------------------------------------
def phase_coef(phase) -> complex:
    """
    Return the coef used when rephasing.
    """
    return np.exp(- 1j * phase)

# -------------------------------------------------------------------
def rephase(signal, phase) -> list:
    """
    Return the rephased signal.

    Args :
        signal : numpy.ndarray ro rephase
        phase : number, calculated with getPhase()
    """
    return signal * phase_coef(phase)

# -------------------------------------------------------------------
def rephase_auto(signal, min_=0, max_=None) -> list:
    """
    If you don't want to get intermediary values, this is a compil of the
    precedent functions. Do not modify inputs.

    Args :
        signal : 1D array of complex value
    Kwargs:
        - min_ : the min bounds for where to search
        - max_ : the min bounds for where to search. If None, set to len(signal) 

    Return :
        the rephased array.
    """
    if max_ is None:
        max_ = len(signal)
    i = getPosModuleMax(signal, min_, max_)
    phase = getPhase(signal[i])
    return signal * phase

# -------------------------------------------------------------------
def phase_signal(phase, signal, index=0) -> None:
    """
    Phase a mr file.

    Args :
        - phase : phase value
        - signal : Signal() instance
        - index : the index of the signal to phase. Rephase all if None.
    """
    coef = phase_coef(phase)

    if index is None:
        signal.data *= coef
        signal.data_fourier *= coef
    else:
        signal.data[index] *= coef
        signal.data_fourier[index] *= coef

# =======================================================================

# SHIFTING #

def shift_signal(shift, signal, index=0) -> None:
    """
    Args:
        shift : shift, expressed in Hz for the fourier datas
        signal : Signal() instance
    Kwargs : 
        index : the index of the signal to shift. Shift all if None.
    """
    t = np.arange(signal.meta['size']) * signal.meta['step']
    if index is None:
        signal.data *= np.exp(1j*2*np.pi*shift*t)
        signal.data_fourier = fft.fftshift(fft.fft(signal.data))
    else:
        signal.data[index] *= np.exp(1j*2*np.pi*shift*t)
        signal.data_fourier[index] = fft.fftshift(fft.fft(signal.data[index]))

# =======================================================================

# SCALING #

def scale_signal(scale, signal, index=0) -> None:
    """
    Args:
        scale : scale, percentage of the original datas
        signal : Signal() instance
    Kwargs : 
        index : the index of the signal to shift. Shift all if None.
    """
    if index is None:
        signal.data *= scale
        signal.data_fourier *= scale
    else:
        signal.data[index] *= scale
        signal.data_fourier[index] *= scale

# =======================================================================

# DAMPING #

# -------------------------------------------------------------------
def getIndexesFWHM(signal, min_, max_) -> tuple:
    """
    get the 2 indexes needed for the fwhm.

    Args:
        signal : numpy 1D array
        min_ : the minimal index where to look
        max_ : the maximal index where to look
    Return :
        (x1, x2) tuple : x coordinate for the fwhm. x1 < x2
    """
    if min_ > max_:
        min_, max_ = max_, min_

    # Get max and mid
    ind_max = np.argmax(signal[min_:max_]) + min_
    val_fwhm = signal[ind_max] / 2

    # get x1 (left part)
    x1 = ind_max
    while x1 > min_ and signal[x1-1] > val_fwhm:
        x1 -= 1
    delta1 = abs(val_fwhm - signal[x1])
    delta2 = abs(val_fwhm - signal[x1-1])
    if delta2 < delta1:
        x1 -= 1

    # get x2 (right part)
    x2 = ind_max
    while x2 < max_ and signal[x2+1] > val_fwhm:
        x2 += 1
    delta1 = abs(val_fwhm - signal[x2])
    delta2 = abs(val_fwhm - signal[x2+1])
    if delta2 < delta1:
        x2 += 1

    return (x1, x2)

# -------------------------------------------------------------------
def dampe(signal, delta, ind=0) -> np.ndarray:
    """
    Dampe a signal.

    Args :
        signal : Signal() instance
        delta : float, fwhm delta Hz between 2 signals
    Kwargs :
        ind : int
    Return :
        Temporal signal damped.
    """
    t = np.arange(signal.meta['size']) * signal.meta['step']
    with np.errstate(all='raise'): # catch overflow from exp
        coef = np.exp(- delta * t)

    new_temp = signal.data[ind] * coef
    return new_temp

# -------------------------------------------------------------------
def dampe_signal(delta, signal, index=0) -> None:
    t = np.arange(signal.meta['size']) * signal.meta['step']
    with np.errstate(all='raise'): # catch overflow from exp
        coef = np.exp(- delta * t)

    if index is None:
        signal.data *= coef
        signal.data_fourier = fft.fftshift(fft.fft(signal.data))
    else:
        signal.data[index] *= coef
        signal.data_fourier[index] = fft.fftshift(fft.fft(signal.data[index]))

# =======================================================================

if __name__ == '__main__':
    pass
