# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Widgets used for preprocessing signals.
"""

import numpy as np
from matplotlib.lines import Line2D
from matplotlib.widgets import SpanSelector

import wx
from wx.lib import newevent

from utils import Observer
from . import proc

from os.path import dirname

# =======================================================================

EndPreprocEvent, EVT_END_PREPROC = newevent.NewEvent()

def manageComplex(array, sel=0) -> np.ndarray:
    if sel==0:
        return array.real
    elif sel==1:
        return array.imag
    else:
        return np.abs(array)

# =======================================================================

class PreprocDialogBase(wx.Dialog, Observer):
    """
    The parent for all the preprocessing dialogs. This class should not be
    instantiated. 

    Observe the display paramters panel.
    *IMPORTANT:* an important assertion is that this class has been added to
    DisplayParameters observers *AFTER* the SignalPanel, so the model should
    already have been updated before PreprocDialogBase.update is call.
    """
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, model, param, current=None, **kwargs) -> None:
        wx.Dialog.__init__(self, parent, style=wx.CAPTION|wx.CLOSE_BOX, **kwargs)
        Observer.__init__(self)

        param.forcePreprocDisplay()
        param.registerObserver(self)

        self.graph = graph
        self.plot = Line2D([], [], color='black')
        self.graph.axes.add_line(self.plot)

        self.model = model
        self.current = current
        self.param = param

        self.__initGuiBase()
        self.Layout()

    # -------------------------------------------------------------------
    def __initGuiBase(self):
        # elements
        self.choice = wx.Choice(self)
        for elm in self.model:
            self.choice.Append(elm.name, elm)

        self.all_dim = wx.CheckBox(self, label="Apply along all dimensions")
        save = wx.Button(self, label="Save")
        save_as = wx.Button(self, label="Save as")

        # sizers
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(save, 0, wx.ALL, 10)
        hbox.Add(save_as, 0, wx.ALL, 10)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.choice, 0, wx.ALL, 10)
        self.sizer.Add(wx.StaticLine(self), 1, wx.EXPAND|wx.ALL, 0)
        self.sizer.Add(self.all_dim, 0, wx.ALL, 10)
        self.sizer.Add(hbox, 0)
        self.SetSizerAndFit(self.sizer)

        # events
        self.Bind(wx.EVT_CLOSE, self.onClose)
        self.choice.Bind(wx.EVT_CHOICE, self.onSetChoice)
        save.Bind(wx.EVT_BUTTON, self.onSave)
        save_as.Bind(wx.EVT_BUTTON, self.onSaveAs)

    # -------------------------------------------------------------------
    def _insertProcPanel(self, window):
        """
        Use this function to add the specific pre processign panel to the
        widget.

        Args :
            window : the window (usually a penl) to add.
        """
        flags = [0, wx.EXPAND|wx.ALL, 10]
        self.sizer.Insert(2, window, *flags)
        self.sizer.Fit(self)

    # -------------------------------------------------------------------
    def onSetChoice(self, event) -> None:
        """
        Set current model and refresh graph.
        """
        sel = event.GetSelection()
        if sel < 0:
            self.current = None
            return

        self.current = sel
        self.refreshFromModelData()
        self.graph.refresh()

    # -------------------------------------------------------------------
    def refreshFromModelData(self) -> None:
        """
        Refresh current datas from the model, and then update the plot.
        """
        if self.current is None:
            return

        data = self.model[self.current]
        imag = self.param.imag_option.GetSelection()
        bound = self.getBound(data.signal)

        x = np.linspace(bound[0], bound[1], data.signal.meta['size'])
        y = data.signal.data_fourier[data.index]
        y = manageComplex(y, imag)

        self.plot.set_xdata(x)
        self.plot.set_ydata(y)

    # -------------------------------------------------------------------
    def update(self, *args, **kwargs) -> None:
        """
        Refresh the model and plots.
        """
        if self.current is None:
            return

        self.refreshFromModelData()
        self.applyPreprocOnPlot()
        self.graph.refresh()
        
    # -------------------------------------------------------------------
    def onSave(self, event) -> None:
        """
        Save current pre processing in the file. This write over the opened
        file.
        """
        if self.current is None:
            return

        self.applyPreprocOnSignal()
        self.model[self.current].signal.save(file_type='mrui')
        self.GetParent().logStatus('Pre-processing applied', 2, 4)
        
    # -------------------------------------------------------------------
    def onSaveAs(self, event) -> None:
        """
        Save modified file in a new file, selected with a file dialog.
        """
        if self.current is None:
            return

        sig = self.model[self.current].signal
        #copy old data
        if sig.data.flags['C_CONTIGUOUS']:
            old_data = sig.data.copy()
        else:
            old_data = sig.data.copy(order='F')
        if sig.data_fourier.flags['C_CONTIGUOUS']:
            old_data_fourier = sig.data_fourier.copy()
        else:
            old_data_fourier = sig.data_fourier.copy(order='F')

        #get where to save path
        default_dir = dirname(sig.path)
        with wx.FileDialog(self, 'Save processed signals', sig.path,
                style=wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT) as fd:
            if fd.ShowModal() == wx.ID_CANCEL:
                return

            pathname = fd.GetPath()
            #apply and save
            self.applyPreprocOnSignal()
            sig.save(pathname, file_type='mrui')

        #restore old data
        sig.data = old_data
        sig.data_fourier = old_data_fourier
        #let the user know
        self.GetParent().logStatus('Pre-processing applied', 2, 4)

    # -------------------------------------------------------------------
    def getBound(self, signal) -> tuple:
        if self.param.xunit.GetSelection() == 0:
            return signal.getHzRange()
        else:
            return signal.getPpmRange()

    # -------------------------------------------------------------------
    def refreshViewerPlots(self):
        self.GetParent().spanel.onUpdatePlots(keep_view=True)

    # -------------------------------------------------------------------
    def onClose(self, event) -> None:
        """
        Post an event to the parent to say that pre processing are finished,
        then close the dialog.
        """
        wx.PostEvent(self.GetParent(), EndPreprocEvent(dlg=self))
        event.Skip()

    # -------------------------------------------------------------------
    def applyPreprocOnPlot(self):
        """
        Update the plot so he display the signal after pre processing.
        """
        raise NotImplementedError('This should be override by childrens')

    # -------------------------------------------------------------------
    def reset(self):
        """
        Reset the pre processed plot and all the values stored.
        """
        raise NotImplementedError('This should be override by childrens')

    # -------------------------------------------------------------------
    def applyPreprocOnSignal(self):
        """
        Apply the pre processing on the current signal stored in the model.
        """
        raise NotImplementedError('This should be override by childrens')

    # -------------------------------------------------------------------
    @staticmethod
    def factory(name, *args, **kwargs):
        """
        Factory method to create pre processing dialogs.

        Args : 
            name : string, define which type of dialog to create. name in
            ['phase', 'scale']

        All other arguments are transmitted to the constructor of the dialogs.
        """
        if name == 'phase':
            return PhasingDialog(*args, **kwargs)
        elif name == 'scale':
            return ScalingDialog(*args, **kwargs)
        elif name == 'shift':
            return ShiftingDialog(*args, **kwargs)
        elif name == 'dampe':
            return DampingDialog(*args, **kwargs)
        else:
            raise NotImplementedError(name + ' : preprocessing unknows')

# =======================================================================

class DampingDialog(PreprocDialogBase):
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, model, param, current=None, **kwargs) -> None:
        super().__init__(parent, graph, model, param, current, **kwargs)
        self.span = SpanSelector(self.graph.axes, self.onSpan, 'horizontal', useblit=True)
        self.peak_selection = None
        self.current_dval = None
        self.reference_dval = None

        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self):
        self.plot.set_label("Damped")
        self.graph.refresh_legend()

        pnl = wx.Panel(self)
        # elements
        self.current_mol_txt = wx.StaticText(pnl)
        self.current_mol_fwhm = wx.StaticText(pnl)
        current_mol_btn = wx.Button(pnl, label='Select peak')

        self.reference_choice = wx.Choice(pnl)
        for i in range(self.choice.GetCount()):
            string = self.choice.GetString(i)
            d = self.choice.GetClientData(i)
            self.reference_choice.Append(string, d)

        self.reference_fwhm = wx.StaticText(pnl)
        reference_btn = wx.Button(pnl, label='Select Peak')

        # sizers
        txt_flag = [0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5]
        cur_box = wx.BoxSizer(wx.VERTICAL)
        line1 = wx.BoxSizer(wx.HORIZONTAL)
        line1.Add(wx.StaticText(pnl, label="Studied signal : "), *txt_flag)
        line1.Add(self.current_mol_txt, *txt_flag)
        line2 = wx.BoxSizer(wx.HORIZONTAL)
        line2.Add(wx.StaticText(pnl, label='Damping : '), *txt_flag)
        line2.Add(self.current_mol_fwhm, *txt_flag)
        self.line3 = wx.BoxSizer(wx.HORIZONTAL)
        self.line3.Add(current_mol_btn, 0, wx.ALL, 5)
        self.line3.Add(wx.StaticText(pnl, label="Span on graph"), *txt_flag)
        self.line3.Hide(1)
        cur_box.Add(line1, 0, wx.ALL|wx.EXPAND, 5)
        cur_box.Add(line2, 0, wx.ALL|wx.EXPAND, 5)
        cur_box.Add(self.line3, 0, wx.ALL, 5)

        ref_box = wx.BoxSizer(wx.VERTICAL)
        line1b = wx.BoxSizer(wx.HORIZONTAL)
        line1b.Add(wx.StaticText(pnl, label="Reference : "), *txt_flag)
        line1b.Add(self.reference_choice, 0, wx.ALL, 5)
        line2b = wx.BoxSizer(wx.HORIZONTAL)
        line2b.Add(wx.StaticText(pnl, label='Damping : '), *txt_flag)
        line2b.Add(self.reference_fwhm, *txt_flag)
        self.line3b = wx.BoxSizer(wx.HORIZONTAL)
        self.line3b.Add(reference_btn, 0, wx.ALL, 5)
        self.line3b.Add(wx.StaticText(pnl, label="Span on graph"), *txt_flag)
        self.line3b.Hide(1)
        ref_box.Add(line1b, 0, wx.ALL|wx.EXPAND, 5)
        ref_box.Add(line2b, 0, wx.ALL|wx.EXPAND, 5)
        ref_box.Add(self.line3b, 0, wx.ALL, 5)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(cur_box, 1, wx.ALL|wx.EXPAND)
        hbox.Add(wx.StaticLine(pnl, style=wx.LI_VERTICAL), 0,
                  wx.BOTTOM|wx.EXPAND, 0)
        hbox.Add(ref_box, 1, wx.ALL|wx.EXPAND)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(hbox, 1, wx.ALL|wx.EXPAND)
        pnl.SetSizerAndFit(sizer)

        # events
        current_mol_btn.Bind(wx.EVT_BUTTON, lambda evt: self.onSelectPeak(evt, 'current'))
        reference_btn.Bind(wx.EVT_BUTTON, lambda evt: self.onSelectPeak(evt, 'reference'))

        self._insertProcPanel(pnl)

    # -------------------------------------------------------------------
    def onSelectPeak(self, event, src) -> None:
        """
        Args:
            - event : fired by wx
            - src in ['current', reference']
        """
        self.peak_selection = src
        self.span.set_visible(True)
        if src == 'current':
            self.line3.Show(1)
            self.line3b.Hide(1)
        else:
            self.line3b.Show(1)
            self.line3.Hide(1)
        self.Layout()

    # -------------------------------------------------------------------
    def onSpan(self, xmin, xmax):
        self.span.set_visible(False)
        self.line3.Hide(1)
        self.line3b.Hide(1)
        sel = self.peak_selection
        self.peak_selection = None
        self.Layout()

        if sel == 'current':
            self._onSpanCurrent(xmin, xmax)
        else:
            self._onSpanReference(xmin, xmax)

        if self.current_dval is not None and \
           self.reference_dval is not None:
            self.applyPreprocOnPlot()
            self.graph.refresh()

    # -------------------------------------------------------------------
    def _onSpanCurrent(self, xmin, xmax) -> None:
        # calculing values
        if self.current is None:
            return

        data = self.model[self.current]
        bound = self.getBound(data.signal)
        meta = data.signal.meta
        imin = int((xmin - bound[0]) / (bound[1] - bound[0]) * meta['size'])
        imax = int((xmax - bound[0]) / (bound[1] - bound[0]) * meta['size'])
        if imin == imax:
            imax += 1

        try:
            fmin, fmax = proc.getIndexesFWHM(data.signal.data_fourier[data.index], imin, imax)
        except (AssertionError, RecursionError) as err:
            msg = "An error ocurred when trying to get the FWHM." + \
                  "Please select an area where there's no unwanted high peak"
            wx.MessageDialog(self, msg).ShowModal()
            print(err)
            return


        hz_range = data.signal.getHzRange()

        hz_min = fmin*(hz_range[1] - hz_range[0])/meta['size']
        hz_max = fmax*(hz_range[1] - hz_range[0])/meta['size']
        damping = (hz_max - hz_min) / np.pi

        # updating UI
        self.current_dval = damping
        self.current_mol_fwhm.SetLabel(str(damping))

    # -------------------------------------------------------------------
    def _onSpanReference(self, xmin, xmax) -> None:
        # calculing values
        if self.reference_choice.GetSelection() < 0:
            return

        data = self.reference_choice.GetClientData(self.reference_choice.GetSelection())
        bound = self.getBound(data.signal)
        meta = data.signal.meta
        imin = int((xmin - bound[0]) / (bound[1] - bound[0]) * meta['size'])
        imax = int((xmax - bound[0]) / (bound[1] - bound[0]) * meta['size'])
        if imin == imax:
            imax += 1

        try:
            fmin, fmax = proc.getIndexesFWHM(data.signal.data_fourier[data.index], imin, imax)
        except (AssertionError, RecursionError) as err:
            msg = "An error ocurred when trying to get the FWHM.\n" + \
                  "Please select an area where there's no unwanted high peak"
            wx.MessageDialog(self, msg).ShowModal()
            print(err)
            return


        hz_range = data.signal.getHzRange()

        hz_min = fmin*(hz_range[1] - hz_range[0])/meta['size']
        hz_max = fmax*(hz_range[1] - hz_range[0])/meta['size']
        damping = (hz_max - hz_min) / np.pi

        # updating UI
        self.reference_dval = damping
        self.reference_fwhm.SetLabel(str(damping))

    # -------------------------------------------------------------------
    def onSetChoice(self, event) -> None:
        super().onSetChoice(event)

        if self.current is None:
            return
        self.current_mol_txt.SetLabel(self.choice.GetString(self.current))

    # -------------------------------------------------------------------
    def applyPreprocOnPlot(self):
        if self.current is None:
            return

        data = self.model[self.current]
        delta = self.current_dval - self.reference_dval
        ind = data.index
        try:
            new_freq = np.fft.fftshift(np.fft.fft(proc.dampe(data.signal,
                                                                   delta, ind)))
        except FloatingPointError:
            wx.MessageBox( 'over/under flow when trying to make exp function', 'Error')
            return

        new_freq = manageComplex(new_freq)
        self.plot.set_ydata(new_freq)

    # -------------------------------------------------------------------
    def applyPreprocOnSignal(self):
        if self.current is None:
            return

        data = self.model[self.current]
        index = data.index
        if self.all_dim.IsChecked():
            index = None

        delta = self.current_dval - self.reference_dval
        proc.dampe_signal(delta, data.signal, index)
        self.refreshViewerPlots()

# =======================================================================

class PhasingDialog(PreprocDialogBase):
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, model, param, current=None, **kwargs) -> None:
        super().__init__(parent, graph, model, param, current, **kwargs)
        self.phase_value = 0
        self.span = SpanSelector(self.graph.axes, self.onSpan, 'horizontal', useblit=True)

        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self):
        self.plot.set_label("Phased")
        self.graph.refresh_legend()

        pnl = wx.Panel(self)
        self.start_span = wx.StaticText(pnl, label='From : ')
        self.end_span = wx.StaticText(pnl, label='to :   ppm')
        self.phase = wx.StaticText(pnl, label='Phase :   rad')
        clear_phasing = wx.Button(pnl, label="Reset")

        # sizers
        txt_flag = [0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 10]
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.start_span, *txt_flag)
        sizer.Add(self.end_span, *txt_flag)
        sizer.Add(self.phase, *txt_flag)
        sizer.Add(clear_phasing, 0, wx.ALL, 10)
        pnl.SetSizerAndFit(sizer)

        self._insertProcPanel(pnl)

    # -------------------------------------------------------------------
    def onSpan(self, xmin, xmax):
        """
        Called when the user span over the graph. See `SpanSelector
        <https://matplotlib.org/gallery/widgets/span_selector.html>`_ for more
        informations.
        """
        if self.current is None:
            return

        data = self.model[self.current]
        # calculing values
        x_axe = self.plot.get_xdata()
        meta = data.signal.meta
        imin = int((xmin - x_axe[0]) / (x_axe[-1] - x_axe[0]) * meta['size'])
        imax = int((xmax - x_axe[0]) / (x_axe[-1] - x_axe[0]) * meta['size'])
        if imin == imax:
            imax += 1
        ind = proc.getPosModuleMax(data.signal.data_fourier[data.index], imin, imax)
        self.phase_value = proc.getPhase(data.signal.data_fourier[data.index][ind])
        self.applyPreprocOnPlot()

        # updating UI
        self.phase.SetLabel('Phase : ' + str(round(self.phase_value, 4)) + ' rad')
        self.start_span.SetLabel('From : ' + str(round(xmin, 4)))
        self.end_span.SetLabel('to : ' + str(round(xmax, 4)) + ' ppm')
        self.graph.refresh()

    # -------------------------------------------------------------------
    def applyPreprocOnPlot(self):
        data = self.model[self.current]
        imag = self.param.imag_option.GetSelection()

        trans_data = proc.rephase(data.signal.data_fourier[data.index], self.phase_value)
        trans_data = manageComplex(trans_data, imag)
        self.plot.set_ydata(trans_data)

    # -------------------------------------------------------------------
    def applyPreprocOnSignal(self):
        if self.current is None:
            return

        data = self.model[self.current]
        index = data.index
        if self.all_dim.IsChecked():
            index = None
        proc.phase_signal(self.phase_value, data.signal, index)
        self.refreshViewerPlots()

# =======================================================================

class ShiftingDialog(PreprocDialogBase):
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, model, param, current=None, **kwargs) -> None:
        super().__init__(parent, graph, model, param, current, **kwargs)
        self.hz_value = 0
        self.shift_val = 0

        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self):
        self.plot.set_label("Shifted")
        self.graph.refresh_legend()

        pnl = wx.Panel(self)
        # elements
        self.shift = wx.Slider(pnl, maxValue=1)
        self.shift_hz = wx.StaticText(pnl, label=' Hz')
        self.shift_ppm = wx.StaticText(pnl, label=' ppm')

        # sizers
        txt_flag = [0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 10]

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.shift, 0, wx.EXPAND|wx.RIGHT, 5)
        sizer.Add(self.shift_hz, *txt_flag)
        sizer.Add(self.shift_ppm, *txt_flag)
        pnl.SetSizerAndFit(sizer)

        self._insertProcPanel(pnl)

        # events
        self.shift.Bind(wx.EVT_SCROLL_THUMBTRACK, self.onChangeShift)

    # -------------------------------------------------------------------
    def onChangeShift(self, event) -> None:
        if self.current is None:
            return

        data = self.model[self.current]
        if data:
            pos = event.GetPosition()
            self.shift_val = pos
            meta = data.signal.meta
            hz_range = data.signal.getHzRange()
            ppm_range = data.signal.getPpmRange()

            self.hz_value = - pos*(hz_range[1] - hz_range[0])/meta['size']
            ppm_value = pos*(ppm_range[1] - ppm_range[0])/meta['size']
            self.shift_hz.SetLabel(str(self.hz_value) + ' Hz')
            self.shift_ppm.SetLabel(str(ppm_value) + ' ppm')

            self.applyPreprocOnPlot()

    # -------------------------------------------------------------------
    def onSetChoice(self, event) -> None:
        super().onSetChoice(event)
        self.shift.SetMax(self.model[self.current].signal.meta['size'])

    # -------------------------------------------------------------------
    def applyPreprocOnPlot(self):
        if self.current == None:
            return

        data = self.model[self.current]
        array = manageComplex(data.signal.data_fourier[data.index])
        self.plot.set_ydata(np.roll(array, -self.shift_val))
        self.graph.canvas.draw_idle()

    # -------------------------------------------------------------------
    def applyPreprocOnSignal(self):
        if self.current is None:
            return

        data = self.model[self.current]
        index = data.index
        if self.all_dim.IsChecked():
            index = None
        proc.shift_signal(self.hz_value, data.signal, index)
        self.refreshViewerPlots()

# =======================================================================

class ScalingDialog(PreprocDialogBase):
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, model, param, current=None, **kwargs) -> None:
        super().__init__(parent, graph, model, param, current, **kwargs)
        self.scale_val = 1
        self.divisor = 10

        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self):
        self.plot.set_label("Scale")
        self.graph.refresh_legend()

        pnl = wx.Panel(self)
        # elements
        self.scale = wx.Slider(pnl, value=10, minValue=1, maxValue=10)
        self.min_txt = wx.StaticText(pnl, label='min:1e-1')
        self.max_txt = wx.StaticText(pnl, label='max:1e0')
        back_bit = wx.ArtProvider.GetBitmap(wx.ART_GO_BACK, wx.ART_BUTTON)
        next_bit = wx.ArtProvider.GetBitmap(wx.ART_GO_FORWARD, wx.ART_BUTTON)
        back_btn = wx.BitmapButton(pnl, bitmap=back_bit)
        next_btn = wx.BitmapButton(pnl, bitmap=next_bit)

        # sizers
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(back_btn, 0, wx.ALL, 5)
        hbox.Add(self.scale, 1, wx.EXPAND|wx.RIGHT, 5)
        hbox.Add(next_btn, 0, wx.ALL, 5)
        hbbox = wx.BoxSizer(wx.HORIZONTAL)
        hbbox.Add(self.min_txt, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
        hbbox.AddStretchSpacer(prop=1)
        hbbox.Add(self.max_txt, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(hbox, 0, wx.EXPAND|wx.RIGHT, 0)
        sizer.Add(hbbox, 0, wx.EXPAND|wx.RIGHT, 0)
        pnl.SetSizerAndFit(sizer)

        self._insertProcPanel(pnl)

        # events
        back_btn.Bind(wx.EVT_BUTTON, self.onBackRange)
        next_btn.Bind(wx.EVT_BUTTON, self.onNextRange)
        self.scale.Bind(wx.EVT_SCROLL_THUMBTRACK, self.onChangeScale)

    # -------------------------------------------------------------------
    def onChangeScale(self, event) -> None:
        if self.current is None:
            return

        pos = event.GetPosition()
        self.scale_val = pos/self.divisor
        self.applyPreprocOnPlot()

    # -------------------------------------------------------------------
    def onBackRange(self, event) -> None:
        self.divisor *= 10
        expo = int(np.log10(self.divisor))
        self.min_txt.SetLabel('min: 1e{}'.format(-expo))
        self.max_txt.SetLabel('max: 1e{}'.format(-expo+1))
        self.scale_val = self.scale.GetValue()/self.divisor
        self.applyPreprocOnPlot()
        self.Layout()

    # -------------------------------------------------------------------
    def onNextRange(self, event) -> None:
        self.divisor /= 10
        expo = int(np.log10(self.divisor))
        self.min_txt.SetLabel('min: 1e{}'.format(-expo))
        self.max_txt.SetLabel('max: 1e{}'.format(-expo+1))
        self.scale_val = self.scale.GetValue()/self.divisor
        self.applyPreprocOnPlot()
        self.Layout()

    # -------------------------------------------------------------------
    def applyPreprocOnPlot(self):
        if self.current == None:
            return
        data = self.model[self.current]
        self.plot.set_ydata(data.plot.get_ydata() * self.scale_val)
        self.graph.refresh()

    # -------------------------------------------------------------------
    def applyPreprocOnSignal(self):
        if self.current is None:
            return

        data = self.model[self.current]
        index = data.index
        if self.all_dim.IsChecked():
            index = None
        proc.scale_signal(self.scale_val, data.signal, index)
        self.refreshViewerPlots()

# =======================================================================

if __name__ == '__main__':
    pass
