# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
MR file class representation.

This class support only file describing a voxel. If the dimension of the datas
is higher than 2, this will store all the datas in a 2d array like a voxel.
"""

import numpy as np
import os.path

from .fileio import *

# =======================================================================
class Signal:
    """
    Store signals data, fourier-transformed data and meta-data for a MRS.
    Meta-data are rperesented by a dictionary with at least keys in :
        - size : dimension, number of points for 1 signal
        - step : step time used during acquisition
        - transFreq : transmission frequency during acqu.
        - B0 : wave reference
        - ppmref : ppm references for display
        - nbSignal : number of signals stored in this file
    """

    # -------------------------------------------------------------------
    def __init__(self, pathname=None, file_type=None)->None:
        """
        Keep in memory the path of the current signal.

        Kwargs :
            pathname : string, if not None, ty to load the file at this path.
            file_type : string, the file type to load.
        """
        self.data = []
        self.data_fourier = []
        self.meta = {}
        self.path = ""
        if pathname is not None:
            self.load(pathname, file_type)

    # -------------------------------------------------------------------
    def load(self, file_name, file_type=None, change_path=True)->None:
        """
        Load a file in the current Signal instance.

        Args:
            - file_name : The complete path to the file.
        Kwargs:
            - file_type : Precise how to read this file. If None, autodetection
                          is implemented with regarding to the extension of the
                          file, or the name for Bruker files. See _detect_extension().
                          file_type in ['mrui', 'h5', 'bruker', 'dcm', 'ima'].
            - change_path : bool, if True change the current instance path
                            reference.
        """
        if not file_type:
            file_type = self._detect_extension(file_name)

        if file_type == "mrui":
            data, meta = mrui.load(file_name)
        elif file_type == "h5":
            data, meta = hdf5.load(file_name)
        elif file_type == "bruker":
            data, meta = bruker.load(file_name)
        elif file_type == "dcm":
            data, meta = dicom.load(file_name)
        elif file_type == "ima":
            data, meta = dicom.load(file_name)
        else:
            raise NotImplementedError("{} : {} files are not supported.".format(file_name,
                                                                                file_type))

        self.data, self.meta = data, meta
        self.data_fourier = np.fft.fftshift(np.fft.fft(data))
        if change_path:
            self.path = file_name

    # -------------------------------------------------------------------
    def save(self, file_name=None, file_type=None) -> str:
        """
        Save the current instance of Signal in a file. 

        Kwargs :
            file_name : string, the file name *without its extension*.
            file_type : strin, how to save this file. In ['mrui'].
        Raise :
            NotImplementedError
        Return :
            string : the path created.
        """
        if file_name is None:
            file_name = self.path
        if file_type is None:
            file_type = self._detect_extension(file_name)
        file_name = os.path.splitext(file_name)[0]

        final_path = ""
        if file_type == "mrui":
            final_path = file_name + '.' + file_type
            mrui.save(self, final_path)
        else:
            raise NotImplementedError("{} files are not supported for saving.".format(file_type))

        return final_path

    # -------------------------------------------------------------------
    @staticmethod
    def _detect_extension(file_name) -> str:
        extension = os.path.splitext(file_name)[1]
        if (extension == '' and os.path.basename(file_name) in ['fid', 'ser']):
            extension = ".bruker"
        return extension[1:]

    # -------------------------------------------------------------------
    def getHzRange(self) -> tuple:
        Fe = 1/self.meta['step']
        return (-Fe/2, Fe/2)

    # -------------------------------------------------------------------
    def getPpmRange(self) -> tuple:
        Fe = 1/self.meta['step']
        car = self.meta['transFreq']
        bound = Fe*1e6/(2*car)
        return (-bound, bound)

# =======================================================================

if __name__ == '__main__':
    pass
