# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
#
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
MRS VIEWER

This software is a viewer for MR signal, and allow to process cQUEST
and some pre processing. See `here <https://www.creatis.insa-lyon.fr/site7/fr/mrsviewer>`_
for more details.
"""

import os
import os.path
import sys
import traceback
import logging
from argparse import ArgumentParser

import wx

from mainframe import MainFrame
from utils.preferences import AppPreferences, isCQuestPathValid
from quest.interface import ProcessQuestFrame

# =======================================================================

def ExceptionHooker(etype, value, trace)->None:
    """
    Handler for all unhandled exceptions. All this exceptions close the
    application. Stock details into ./errors.log text file.
    """

    tmp = traceback.format_exception(etype, value, trace)
    exception = "".join(tmp)

    sys.stderr.write(exception)
    logging.debug(exception + '\r\n##############################\r\n')

    wx.MessageBox("An unhandled error has occured.\r\n\r\n {} \r\n\r\n Please contact the dev\
 to send him/her logs, and the context of when it append. The app will now\
 close. \r\n".format(value.__str__()))
    wx.GetApp().Destroy()

    wx.Exit()

# =======================================================================

class MRSviewer(wx.App):
    """
    The mrsViewer core application.
    Store the preferences and manage unhandled errors with ExceptionHooker
    """

    # -------------------------------------------------------------------
    def OnInit(self)->bool:
        """
        Called when creating the instance.
        """
        logging.basicConfig(filename='errors.log', level=logging.DEBUG)
        sys.excepthook = ExceptionHooker
        self.cwd = os.getcwd()

        # Create a preferences folder
        if os.name != "posix":
            from win32com.shell import shellcon, shell
            homedir = "{}\\".format(shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0))
        else:
            homedir = "{}/".format(os.path.expanduser('~'))
        pref_path = "{}.{}".format(homedir, "mrsViewer")
        print(pref_path)
        if not os.path.isdir(pref_path):
            os.mkdir(pref_path)
        self.pref = AppPreferences(pref_path)

        return True

# =======================================================================

def main(args):
    """
    Main function.Support arguments. Try ``python3 mrsviewer.py -h`` to get
    help.
    """
    app = MRSviewer()

    if 'qpath' not in args:
        frame = MainFrame(None, title="MR signal Viewer")
        app.SetTopWindow(frame)
        frame.spanel.onNewSignal(pathnames=args.files)
    else:
        if not isCQuestPathValid(args.qpath):
            wx.MessageBox("CQuest folder do not contain necessary elements. Abort.",
                          'Warning', style=wx.OK|wx.ICON_WARNING)
            app.Destroy()
            return

        app.pref.quest_soft_path = args.qpath
        app.pref.is_quest_local = True

        frame = ProcessQuestFrame(None)
        frame.addParserFiles(args)
        app.SetTopWindow(frame)

    app.MainLoop()
    app.Destroy()

# =======================================================================

if __name__ == "__main__":
    parser = ArgumentParser()
    subparsers = parser.add_subparsers()

#    parser.add_argument("-n",  dest='name',      default='',help="net name")
#    parser.add_argument("-bs", dest='batchSize', type=int, default=100,help="batch size")
#    parser.add_argument("-label", dest='label', default='label',
#        help="label type used to train the network ('label' as default for amplitudes)")
#
#    parser.add_argument("oroot", help="root name of output proto files")
#    parser.add_argument("droot", help="root name of data files (_train_list.txt _testn_list.txt)")
#
#    parser.add_argument("layers", help="layers as key strings",nargs='+')

    # general options
    parser.add_argument("-f", dest="files", default=[], nargs='+',
                        help="Input files.")

    # cquest options
    quest_parser = subparsers.add_parser('cquest', help='Options used to fill infos' + \
                                                        'for the cQuest window')
    quest_parser.add_argument('qpath', help="The path to the cQuest folder.")
    quest_parser.add_argument('-mf', '--metafiles', default=[], nargs='+',
                              help="Input metabolite files.")
    quest_parser.add_argument('-Mf', '--macrofiles', default=[], nargs='+',
                              help="Input macro molecules files.")
    quest_parser.add_argument("-f", dest="files", default=[], nargs='+',
                              help="Files to process.")


    args = parser.parse_args()
    main(args)
