# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
#
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Widgets used in interface module.
"""

import os.path
from numpy import linspace, roll

import matplotlib
matplotlib.use('WXAgg')
matplotlib.rcParams['lines.linewidth'] = 0.8
import matplotlib.patches as mpatches
from matplotlib.widgets import SpanSelector

import wx
import wx.lib.scrolledpanel as scrolled

from utils import *
from mrsignal import proc
from mrsignal.voxel import Signal

# =======================================================================

class ConstraintPanel(wx.Panel):
    """
    Panel for display current relations constaints and add them.
    """
    # -------------------------------------------------------------------
    def __init__(self, parent, model_meta=[], model_macro=[])->None:
        super().__init__(parent)
        self.meta = model_meta
        self.macro = model_macro
        self.initGui()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        # The list where constraints will be added
        columns = ['Ref,subject', 'Relation', 'Type', 'Value']
        self.meta_constr = PimpedListCtrl(self, columns,
                "Metabolites constraints", False)
        self.macro_constr = PimpedListCtrl(self, columns,
                "Macromolecules constraints", False)

        # Sizers
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.meta_constr, 1, wx.EXPAND|wx.ALL, 10)
        sizer.Add(self.macro_constr, 1, wx.EXPAND|wx.ALL, 10)
        self.SetSizerAndFit(sizer)

        # events
        self.meta_constr.Bind(EVT_PMPD_LIST_EVENT,
                lambda evt: self._listEvtHandler(evt, 'meta'))
        self.macro_constr.Bind(EVT_PMPD_LIST_EVENT,
                lambda evt: self._listEvtHandler(evt, 'macro'))

    # -------------------------------------------------------------------
    def _listEvtHandler(self, event, target):
        """
        Events handler for events produced by relations lists widgets.
        """
        if event.action == 'add':
            self.onNewConstr(target)
        elif event.action == 'del':
            if target == 'meta':
                sel = self.meta_constr.GetSelections()
                for i in reversed(sel):
                    self.meta_constr.Delete(i)
            elif target == 'macro':
                sel = self.macro_constr.GetSelections()
                for i in reversed(sel):
                    self.macro_constr.Delete(i)

    # -------------------------------------------------------------------
    def onNewConstr(self, target)->None:
        """
        Display a dialog which allow to make constraints for elements
        belonging to the list.

        Args :
            - target : string in ['meta', 'macro']
        """
        if target == 'meta':
            molecules = self.meta
            b = True
        else:
            molecules = self.macro
            b = False

        with ConstraintDialog(self, molecules) as dlg:
            if dlg.ShowModal() == wx.ID_OK:
                ref = dlg.reference.GetString(dlg.reference.GetSelection())
                nature = dlg.nature.GetString(dlg.nature.GetSelection())
                ntype = dlg.type.GetString(dlg.type.GetSelection())
                val = str(dlg.val.GetValue()) if ntype=='soft' else "None"

                lst = self.meta_constr if b else self.macro_constr
                for i in range(dlg.apply_on.list.GetCount()):
                    couple = ref + ',' + dlg.apply_on.list.GetString(i)
                    lst.list.Append([couple, nature, ntype, val])

    # -------------------------------------------------------------------
    def __delMol(self, mol, lst, constr) -> None:
        """
        General function to delete a macromolecules or a metabolite.

        Args :
            mol : str
            lst : self.meta OR self.macro
            constr : self.meta_constr OR self.macro_constr
        """
        try:
            ind = lst.index(mol)
        except ValueError:
            return
        else:
            del lst[ind]
            count = constr.list.GetItemCount()
            for row in range(count-1, -1, -1):
                names = constr.list.GetItem(row, 0).GetText()
                names = names.split(',')
                if names[0] == mol or names[1] == mol:
                    constr.list.DeleteItem(row)

    # -------------------------------------------------------------------
    def addMeta(self, meta) -> None:
        """
        Args:
            meta : string
        """
        self.meta.append(meta)

    def delMeta(self, meta) -> None:
        """
        Args:
            meta : string
        """
        self.__delMol(meta, self.meta, self.meta_constr)

    # -------------------------------------------------------------------
    def addMacro(self, macro) -> None:
        """
        Args:
            macro : string
        """
        self.macro.append(macro)

    def delMacro(self, macro) -> None:
        """
        Args:
            macro : string
        """
        self.__delMol(macro, self.macro, self.macro_constr)

    # -------------------------------------------------------------------
    def extractConstraints(self) -> list:
        """
        Create a relations list from the GUI that can fit into a QuestParameters
        instance.

        Return :
            the relations list
        """
        constr_meta = []
        count = self.meta_constr.list.GetItemCount()
        for row in range(count):
            c = []
            item = self.meta_constr.list.GetItem(row, 0).GetText()
            item = item.split(',')
            c.append(item[0])
            c.append(item[1])
            effect = self.meta_constr.list.GetItem(row, 1).GetText()
            etype = self.meta_constr.list.GetItem(row, 2).GetText()
            if etype == "soft":
                val = self.meta_constr.list.GetItem(row, 3).GetText()
                item = effect + "_soft"
                c.append(item)
                c.append(val)
            else:
                if effect == "damping":
                    item = "extra_damping"
                elif effect == "frequency":
                    item = "frequency_shift"
                else:
                    item = "amplitude"
                c.append(item)
            constr_meta.append(c)

        constr_macro = []
        count = self.macro_constr.list.GetItemCount()
        for row in range(count):
            c = []
            item = self.macro_constr.list.GetItem(row, 0).GetText()
            item = item.split(',')
            c.append(item[0])
            c.append(item[1])
            effect = self.macro_constr.list.GetItem(row, 1).GetText()
            etype = self.macro_constr.list.GetItem(row, 2).GetText()
            if etype == "soft":
                val = self.macro_constr.list.GetItem(row, 3).GetText()
                item = effect + "_soft"
                c.append(item)
                c.append(val)
            else:
                if effect == "damping":
                    item = "extra_damping"
                elif effect == "frequency":
                    item = "frequency_shift"
                else:
                    item = "amplitude"
                c.append(item)
            constr_macro.append(c)

        return constr_meta + constr_macro

    # -------------------------------------------------------------------
    def loadConstraints(self, constr)->None:
        """
        Load constraints into the GUI from a list which respect the
        QuestParameters format list.
        """
        self.meta_constr.list.DeleteAllItems()
        self.macro_constr.list.DeleteAllItems()

        # Constraints relative to what we store
        for c in constr:
            if c[0] in self.meta:
                lst = self.meta_constr.list
            elif c[0] in self.macro:
                lst = self.macro_constr.list
            else:
                continue

            couple = ",".join(c[0:2])
            if "damping" in c[2]:
                effect = "damping"
            elif "frequency" in c[2]:
                effect = "frequency"
            elif "amplitude" in c[2]:
                effect = "amplitude"
            else:
                continue

            if "soft" in c[2]:
                etype = "soft"
                val = c[3]
            else:
                etype = "hard"
                val = "None"
            lst.Append([couple, effect, etype, val])

# =======================================================================

class ConstraintDialog(wx.Dialog):
    """
    The panel that allow to add constraints to a wxListCtrl with 4 columns.
    """

    # -------------------------------------------------------------------
    def __init__(self, parent, molecules)->None:
        super().__init__(parent, style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
        self.SetTitle("Add Constraint")

        self.molecules = molecules
        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        # elements
        self.reference = wx.Choice(self, choices=self.molecules)
        self.apply_on = ChoiceListBox(self, "Elements")
        self.nature = wx.Choice(self,
                choices=["damping", "frequency", "amplitude"])
        self.nature.SetSelection(0)
        self.type = wx.Choice(self, choices=["soft", "hard"])
        self.type.SetSelection(1)
        self.val_txt = wx.StaticText(self, label="Constraint value : ( )")
        self.val = wx.SpinCtrl(self)
        ok = wx.Button(self, wx.ID_OK, "Ok")
        cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        # events
        self.nature.Bind(wx.EVT_CHOICE, self.onChoseType)
        self.type.Bind(wx.EVT_CHOICE, self.onChoseType)
        self.reference.Bind(wx.EVT_CHOICE, self.onChoseRef)

        # sizers
        text_args = [0, wx.ALIGN_CENTER_VERTICAL, 5]
        args = [0, wx.ALL, 5]
        sizer1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer1.Add(wx.StaticText(self, label="Reference : "), *text_args)
        sizer1.Add(self.reference)

        sizer2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer2.Add(wx.StaticText(self, label="Constr. on : "), *text_args)
        sizer2.Add(self.nature)

        sizer3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer3.Add(wx.StaticText(self, label="Type : "), *text_args)
        sizer3.Add(self.type)

        sizer4 = wx.BoxSizer(wx.HORIZONTAL)
        sizer4.Add(self.val_txt, *text_args)
        sizer4.Add(self.val)

        sizer5 = wx.BoxSizer(wx.HORIZONTAL)
        sizer5.AddStretchSpacer(prop=1)
        sizer5.Add(cancel, *args)
        sizer5.Add(ok, *args)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(sizer1, *args)
        self.sizer.Add(wx.StaticText(self, label="Apply on : "), *text_args)
        self.sizer.Add(self.apply_on, 1, wx.ALL|wx.EXPAND, 5)
        self.sizer.Add(sizer2, *args)
        self.sizer.Add(sizer3, *args)
        self.sizer.Add(sizer4, *args)
        self.sizer.Add(sizer5, *args)
        self.SetSizerAndFit(self.sizer)
        self.showValue(False)

    # -------------------------------------------------------------------
    def showValue(self, b=True) -> None:
        """
        Show the widgets for the optional relation value.

        Kwargs:
            b : bool, show if True hide otherwise
        """
        if b:
            self.sizer.Show(5)
        else:
            self.sizer.Hide(5)

    # -------------------------------------------------------------------
    def onChoseType(self, event)->None:
        """
        If type is soft there's an extra parameter to input, either in Hz or %.
        Called when type is changed.
        """

        if self.type.GetString(self.type.GetSelection()) == "soft":
            if self.nature.GetString(self.nature.GetSelection()) == "amplitude":
                self.showValue()
                self.val.SetRange(-1e6, 1e6)
                self.setValueUnit('%')
            elif self.nature.GetString(self.nature.GetSelection()) in \
                    ("damping", "frequency"):
                self.showValue()
                self.val.SetRange(-1e9, 1e9)
                self.setValueUnit('Hz')
        else:
            self.showValue(False)
        self.Layout()

    # -------------------------------------------------------------------
    def setValueUnit(self, m) -> None:
        """
        Set the unit of the optional value text.
        """
        self.val_txt.SetLabel("Constr. value ({}): ".format(m))
        self.Layout()

    # -------------------------------------------------------------------
    def onChoseRef(self, event) -> None:
        """
        When chossing reference.
        """
        ref = self.molecules.index(event.GetString())
        self.setApplyOn(ref)

    # -------------------------------------------------------------------
    def setApplyOn(self, index=None) -> None:
        """
        Change 'apply on' choices, according to the current reference.
        """
        if index == None:
            index = self.reference.GetSelection()
        if index == wx.NOT_FOUND:
            return
        self.apply_on.setChoices(self.molecules[:index] + \
                                 self.molecules[index+1:])

# =======================================================================

class ToolsPanel(wx.Panel):
    """
    The panel which contain all the pre processing UI tools.
    """
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, model)->None:
        super().__init__(parent)
        self.graph = graph
        self.model = model

        self.initGui()
        self.Layout()
        self.Update()

    # -------------------------------------------------------------------
    def initGui(self) -> None:
        # elements
        option_panel = wx.Panel(self)
        self.apply_on = wx.Choice(option_panel)
        self.index = wx.SpinCtrl(option_panel, max=0)

        self.nb = wx.Notebook(self)
        self.phase_panel = PhasingPanel(self.nb, self.graph)
        self.shift_panel = ShiftingPanel(self.nb, self.graph,
                                        self.GetGrandParent().rel_constr)
        self.scale_panel = ScalingPanel(self.nb, self.graph,
                                        self.GetGrandParent().rel_constr)
        self.dampe_panel = DampingPanel(self.nb, self.graph,
                                        self.GetGrandParent().rel_constr)
        self.nb.AddPage(self.phase_panel, "Phasing")
        self.nb.AddPage(self.shift_panel, "Shifting")
        self.nb.AddPage(self.scale_panel, "Scaling")
        self.nb.AddPage(self.dampe_panel, "Damping")

        apply_panel = wx.Panel(self)
        self.apply_cur_sig = wx.CheckBox(apply_panel, label="current signal")
        self.apply_cur_sig.SetValue(wx.CHK_CHECKED)
        self.apply_all_dim = wx.CheckBox(apply_panel, label="along all dimensions")
        self.apply_all_sig = wx.CheckBox(apply_panel, label="all signals")
        self.apply_all_met = wx.CheckBox(apply_panel, label="all metabolites")
        self.apply_all_mac = wx.CheckBox(apply_panel, label="all macromolecules")
        apply = wx.Button(apply_panel, label="Apply")

        # sizers
        args = [0, wx.ALL, 5]
        txt_flag = [0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 10]

        option_sizer = wx.BoxSizer(wx.HORIZONTAL)
        option_sizer.Add(wx.StaticText(option_panel, label="Studied signal : "), *txt_flag)
        option_sizer.Add(self.apply_on, *args)
        option_sizer.AddStretchSpacer(prop=1)
        option_sizer.Add(wx.StaticText(option_panel, label="Index : "), *txt_flag)
        option_sizer.Add(self.index, *args)
        option_panel.SetSizer(option_sizer)

        gbox = wx.GridSizer(4, 3, 5, 5)
        gbox.Add(wx.StaticText(apply_panel, label ="Apply on :"), *txt_flag)
        gbox.Add(self.apply_cur_sig, *args)
        gbox.Add(self.apply_all_dim, *args)
        gbox.AddMany([(0,0), (self.apply_all_sig, *args), (0,0)])
        gbox.AddMany([(0,0), (self.apply_all_met, *args), (0,0)])
        gbox.AddMany([(0,0), (self.apply_all_mac, *args), (apply, *args)])
        apply_panel.SetSizerAndFit(gbox)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(option_panel, 0, wx.EXPAND|wx.ALL, 5)
        sizer.Add(wx.StaticLine(self), 0, wx.EXPAND|wx.ALL, 2)
        sizer.Add(self.nb, 1, wx.EXPAND|wx.ALL, 5)
        sizer.Add(apply_panel, 0, wx.ALL, 5)
        self.SetSizerAndFit(sizer)

        # events
        self.apply_on.Bind(wx.EVT_CHOICE, self.onChangeChoice)
        self.nb.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self._nbHandler)
        self.index.Bind(wx.EVT_SPINCTRL, self.onChangeIndex)
        apply.Bind(wx.EVT_BUTTON, self.onApply)

    # -------------------------------------------------------------------
    def _nbHandler(self, event):
        """
        Handler for notebook page selection.
        """
        old = event.GetOldSelection()
        cur = event.GetSelection()
        # bcz weird errors when closing the frame
        if old >= 0 and cur >= 0:
            for i in range(self.nb.GetPageCount()):
                if cur == i:
                    self.nb.GetPage(i).showPlot()
                else:
                    self.nb.GetPage(i).showPlot(False)

    # -------------------------------------------------------------------
    def onChangeIndex(self, event) -> None:
        """
        Update the plotted signal index signal with the current loaded file.
        """
        data = self._getCurrentData()
        pos = event.GetPosition()
        data.index = pos
        data.plot.set_ydata(data.signal.data_fourier[pos].real)

        for i in range(self.nb.GetPageCount()):
            self.nb.GetPage(i).refreshFromModelPlot()

        self.graph.refresh()

    # -------------------------------------------------------------------
    def onChangeChoice(self, event) -> None:
        """
        When change the rference.
        """
        data = self._getCurrentData()
        self.index.SetMax(data.signal.meta['nbSignal'] - 1)
        self.index.SetValue(str(data.index))
        sel = self.nb.GetSelection()
        if data:
            for i in range(self.nb.GetPageCount()):
                self.nb.GetPage(i).setModel(data)
                self.nb.GetPage(i).showPlot(i==sel)

    # -------------------------------------------------------------------
    def appendChoice(self, data) -> None:
        """
        Append a reference choice.

        Args:
            - data : 'QuestSignalModel' instance.
        """
        self.apply_on.Append(data.name, data)
        self.dampe_panel.reference_choice.Append(data.name, data)

    # -------------------------------------------------------------------
    def deleteChoice(self, data) -> None:
        """
        Args:
            - data : 'QuestSignalModel' instance.
        """
        for i in range(self.apply_on.GetCount()):
            if data is self.apply_on.GetClientData(i):
                if i == self.apply_on.GetSelection():
                    self._showAllToolsPlot(False)
                self.apply_on.Delete(i)
                break

        for i in range(self.dampe_panel.reference_choice.GetCount()):
            if data is self.dampe_panel.reference_choice.GetClientData(i):
                self.dampe_panel.reference_choice.Delete(i)
                break

    # -------------------------------------------------------------------
    def _showAllToolsPlot(self, show=True) -> None:
        """
        Set visible all the tools plot.
        """
        for i in range(self.nb.GetPageCount()):
            self.nb.GetPage(i).showPlot(show)
        self.graph.canvas.draw_idle()

    # -------------------------------------------------------------------
    def _showCurrentToolPlot(self, show=True) -> None:
        """
        Set visible only the current selected tools plot.
        """
        i = self.nb.GetSelection()
        self.nb.GetPage(i).showPlot(show)
        self.graph.canvas.draw_idle()

    # -------------------------------------------------------------------
    def refreshChoices(self) -> None:
        for i in range(self.apply_on.GetCount()):
            data = self.apply_on.GetClientData(i)
            self.apply_on.SetString(i, data.name)

    # -------------------------------------------------------------------
    def onApply(self, event) -> None:
        """
        Apply the current preprocessing.
        Called by 'Apply' button.
        """
        current = self.apply_cur_sig.IsChecked()
        alldims = self.apply_all_dim.IsChecked()
        allsigs = self.apply_all_sig.IsChecked()
        allmets = self.apply_all_met.IsChecked()
        allmacs = self.apply_all_mac.IsChecked()
        if self.nb.GetSelection() == 0:
            self.__onApplyPhasing(current, alldims, allsigs, allmets, allmacs)
        elif self.nb.GetSelection() == 1:
            self.__onApplyShifting(current, alldims, allsigs, allmets, allmacs)
        elif self.nb.GetSelection() == 2:
            self.__onApplyScaling(current, alldims, allsigs, allmets, allmacs)
        elif self.nb.GetSelection() == 3:
            self.__onApplyDamping(current, alldims, allsigs, allmets, allmacs)

        # refresh all graphs
        data = self._getCurrentData()
        data.plot.set_ydata(data.signal.data_fourier[data.index])
        for i in range(self.nb.GetPageCount()):
            self.nb.GetPage(i).refreshFromModelData()
        self.graph.refresh()

    # -------------------------------------------------------------------
    def __onApplyPhasing(self, current, alldims, allsigs, allmets, allmacs) -> None:
        """
        Args:
            current : bool
            alldims : bool
            allsigs : bool
            allmets : bool
            allmacs : bool
        All this args just represent checkboxes states.
        """
        phase = self.phase_panel.phase_value
        index = self.index.GetValue()
        cur_sig = self._getCurrentData().signal
        if alldims:
            index = None

        #apply
        if current:
            proc.phase_signal(phase, cur_sig, index)
        if allsigs:
            for elm in self.model.getSigList():
                if current and elm.signal is not cur_sig:
                    proc.phase_signal(phase, elm.signal, elm.index)
        if allmets:
            for elm in self.model.getMetaList():
                if current and elm.signal is not cur_sig:
                    proc.phase_signal(phase, elm.signal, elm.index)
        if allmacs:
            for elm in self.model.getMacroList():
                if current and elm.signal is not cur_sig:
                    proc.phase_signal(phase, elm.signal, elm.index)

    # -------------------------------------------------------------------
    def __onApplyShifting(self, current, alldims, allsigs, allmets, allmacs) -> None:
        """
        Args:
            current : bool
            alldims : bool
            allsigs : bool
            allmets : bool
            allmacs : bool
        All this args just represent checkboxes states.
        """
        shift = self.shift_panel.hz_value
        index = self.index.GetValue()
        cur_sig = self._getCurrentData().signal
        if alldims:
            index = None

        #apply
        if current:
            proc.shift_signal(shift, cur_sig, index)
        if allsigs:
            for elm in self.model.getSigList():
                if current and elm.signal is not cur_sig:
                    proc.shift_signal(shift, elm.signal, elm.index)
        if allmets:
            for elm in self.model.getMetaList():
                if current and elm.signal is not cur_sig:
                    proc.shift_signal(shift, elm.signal, elm.index)
        if allmacs:
            for elm in self.model.getMacroList():
                if current and elm.signal is not cur_sig:
                    proc.shift_signal(shift, elm.signal, elm.index)

    # -------------------------------------------------------------------
    def __onApplyScaling(self, current, alldims, allsigs, allmets, allmacs) -> None:
        """
        Args:
            current : bool
            alldims : bool
            allsigs : bool
            allmets : bool
            allmacs : bool
        All this args just represent checkboxes states.
        """
        scale = self.scale_panel.scale_val
        index = self.index.GetValue()
        cur_sig = self._getCurrentData().signal
        if alldims:
            index = None

        #apply
        if current:
            proc.scale_signal(scale, cur_sig, index)
        if allsigs:
            for elm in self.model.getSigList():
                if current and elm.signal is not cur_sig:
                    proc.scale_signal(scale, elm.signal, elm.index)
        if allmets:
            for elm in self.model.getMetaList():
                if current and elm.signal is not cur_sig:
                    proc.scale_signal(scale, elm.signal, elm.index)
        if allmacs:
            for elm in self.model.getMacroList():
                if current and elm.signal is not cur_sig:
                    proc.scale_signal(scale, elm.signal, elm.index)

    # -------------------------------------------------------------------
    def __onApplyDamping(self, current, alldims, allsigs, allmets, allmacs) -> None:
        """
        Args:
            current : bool
            alldims : bool
            allsigs : bool
            allmets : bool
            allmacs : bool
        All this args just represent checkboxes states.
        """
        dampe = self.dampe_panel.current_dval - self.dampe_panel.reference_dval
        index = self.index.GetValue()
        cur_sig = self._getCurrentData().signal
        if alldims:
            index = None

        #apply
        if current:
            proc.dampe_signal(dampe, cur_sig, index)
        if allsigs:
            for elm in self.model.getSigList():
                if current and elm.signal is not cur_sig:
                    proc.dampe_signal(dampe, elm.signal, elm.index)
        if allmets:
            for elm in self.model.getMetaList():
                if current and elm.signal is not cur_sig:
                    proc.dampe_signal(dampe, elm.signal, elm.index)
        if allmacs:
            for elm in self.model.getMacroList():
                if current and elm.signal is not cur_sig:
                    proc.dampe_signal(dampe, elm.signal, elm.index)

    # -------------------------------------------------------------------
    def _getCurrentData(self):
        """
        Get data associated to the current choice.
        """
        ind = self.apply_on.GetSelection()
        if ind >= 0:
            return self.apply_on.GetClientData(ind)
        else:
            return None

# =======================================================================

class PhasingPanel(wx.Panel):
    """
    Panel used in quest frame for phasing.
    """
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, *args, **kwargs) -> None:
        super().__init__(parent)
        self.graph = graph
        self.orig_data = None
        self.transformed_plot = matplotlib.lines.Line2D([],[], color='black')
        self.phase_value = 0
        self.span = SpanSelector(self.graph.axes, self.onSpan, 'horizontal', useblit=True)
        self.span.set_visible(False) # delete this line if this is the first panel

        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self) -> None:
        # widgets
        self.start_span = wx.StaticText(self, label='From : ')
        self.end_span = wx.StaticText(self, label='to :   ppm')
        self.phase = wx.StaticText(self, label='Phase :   rad')
        clear_phasing = wx.Button(self, label="Reset")

        # sizers
        txt_flag = [0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 10]
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.start_span, *txt_flag)
        sizer.Add(self.end_span, *txt_flag)
        sizer.Add(self.phase, *txt_flag)
        sizer.Add(clear_phasing, 0, wx.ALL, 10)
        self.SetSizerAndFit(sizer)

        # events
        clear_phasing.Bind(wx.EVT_BUTTON, self.onClearPhasing)

    # -------------------------------------------------------------------
    def setModel(self, model) -> None:
        self.orig_data = model
        self.refreshFromModelPlot()
        self.transformed_plot.set_label("Phased")
        self.graph.refresh_legend()
        self.graph.refresh()
        
    # -------------------------------------------------------------------
    def refreshFromModelPlot(self) -> None:
        self.transformed_plot.set_xdata(self.orig_data.plot.get_xdata())
        self.transformed_plot.set_ydata(self.orig_data.plot.get_ydata())

    # -------------------------------------------------------------------
    def refreshFromModelData(self) -> None:
        sig = self.orig_data.signal
        meta = sig.meta
        bound = sig.getPpmRange()
        x = np.linspace(bound[0], bound[1], meta['size'])
        y = self.orig_data.signal.data_fourier[self.orig_data.index]

        self.transformed_plot.set_xdata(x)
        self.transformed_plot.set_ydata(y)

    # -------------------------------------------------------------------
    def onClearPhasing(self, event) -> None:
        self.start_span.SetLabel('From : ')
        self.end_span.SetLabel('to :   ppm')
        self.phase.SetLabel('Phase :   rad')
        self.phase_value = 0

        if event:
            self.refreshFromModelPlot()
            self.graph.refresh()

    # -------------------------------------------------------------------
    def showPlot(self, b=True) -> None:
        if b and self.transformed_plot.axes is None:
            self.graph.axes.add_line(self.transformed_plot)
        elif not b and self.transformed_plot.axes is not None:
            self.transformed_plot.remove()
        self.span.set_visible(b)
        self.graph.refresh_legend()
        self.graph.refresh()

    # -------------------------------------------------------------------
    def onSpan(self, xmin, xmax):
        if not self.IsShown():
            return
        data = self.orig_data
        if data is None:
            return
        # calculing values
        x_axe = self.transformed_plot.get_xdata()
        meta = data.signal.meta
        imin = int((xmin - x_axe[0]) / (x_axe[-1] - x_axe[0]) * meta['size'])
        imax = int((xmax - x_axe[0]) / (x_axe[-1] - x_axe[0]) * meta['size'])
        if imin == imax:
            imax += 1
        ind = proc.getPosModuleMax(data.signal.data_fourier[data.index], imin, imax)
        self.phase_value = proc.getPhase(data.signal.data_fourier[data.index][ind])
        self.phaseCurrentPlot()

        # updating UI
        self.phase.SetLabel('Phase : ' + str(round(self.phase_value, 4)) + ' rad')
        self.start_span.SetLabel('From : ' + str(round(xmin, 4)))
        self.end_span.SetLabel('to : ' + str(round(xmax, 4)) + ' ppm')

    # -------------------------------------------------------------------
    def phaseCurrentPlot(self):
        # plotting
        data = self.orig_data
        trans_data = proc.rephase(data.signal.data_fourier[data.index], self.phase_value)
        self.transformed_plot.set_ydata(trans_data)
        self.graph.refresh()

# =======================================================================

class ShiftingPanel(wx.Panel):
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, rel_panel, *args, **kwargs) -> None:
        super().__init__(parent)
        self.graph = graph
        self.orig_data = None
        self.transformed_plot = matplotlib.lines.Line2D([],[], color='black')
        self.rel_panel = rel_panel
        self.hz_value = 0

        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self) -> None:
        # widgets
        self.shift = wx.Slider(self, maxValue=1)
        self.shift_hz = wx.StaticText(self, label=' Hz')
        self.shift_ppm = wx.StaticText(self, label=' ppm')
        constr = wx.Button(self, label="Create relations")

        # sizers
        txt_flag = [0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 10]

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.shift, 0, wx.EXPAND|wx.RIGHT, 5)
        sizer.Add(self.shift_hz, *txt_flag)
        sizer.Add(self.shift_ppm, *txt_flag)
        sizer.Add(constr, 0, wx.ALL, 10)
        self.SetSizerAndFit(sizer)

        # events
        self.shift.Bind(wx.EVT_SCROLL_THUMBTRACK, self.onChangeShift)
        constr.Bind(wx.EVT_BUTTON, self.onCreateRel)

    # -------------------------------------------------------------------
    def setModel(self, model) -> None:
        self.orig_data = model
        self.refreshFromModelPlot()
        self.transformed_plot.set_label("Shifted")
        self.graph.refresh_legend()
        self.graph.refresh()
        self.shift.SetMax(model.signal.meta['size'])

    # -------------------------------------------------------------------
    def refreshFromModelPlot(self) -> None:
        self.transformed_plot.set_xdata(self.orig_data.plot.get_xdata())
        self.transformed_plot.set_ydata(self.orig_data.plot.get_ydata())

    # -------------------------------------------------------------------
    def refreshFromModelData(self) -> None:
        sig = self.orig_data.signal
        meta = sig.meta
        bound = sig.getPpmRange()
        x = np.linspace(bound[0], bound[1], meta['size'])
        y = self.orig_data.signal.data_fourier[self.orig_data.index]
        self.transformed_plot.set_xdata(x)
        self.transformed_plot.set_ydata(y)

    # -------------------------------------------------------------------
    def onCreateRel(self, event) -> None:
        if self.orig_data is None:
            return

        if self.orig_data.name in self.rel_panel.meta:
            molecules = self.rel_panel.meta
            b = True
        elif self.orig_data.name in self.rel_panel.macro:
            molecules = self.rel_panel.macro
            b = False
        else:
            msg = 'The reference should be a metabolite or a macro-molecule.'
            wx.MessageDialog(self, msg).ShowModal()
            return

        with ConstraintDialog(self, molecules) as dlg:
            sel = dlg.reference.FindString(self.orig_data.name)
            dlg.reference.SetSelection(sel)
            dlg.setApplyOn()
            dlg.nature.SetSelection(1)
            dlg.type.SetSelection(0)
            dlg.val.SetMin(-1e9)
            dlg.val.SetMax(1e9)
            dlg.val.SetValue(str(self.hz_value))
            dlg.sizer.Show(5)
            dlg.setValueUnit('Hz')
            if dlg.ShowModal() == wx.ID_OK:
                ref = dlg.reference.GetString(dlg.reference.GetSelection())
                nature = dlg.nature.GetString(dlg.nature.GetSelection())
                ntype = dlg.type.GetString(dlg.type.GetSelection())
                val = str(dlg.val.GetValue()) if ntype=='soft' else "None"

                lst = self.rel_panel.meta_constr if b else self.rel_panel.macro_constr
                for i in range(dlg.apply_on.list.GetCount()):
                    couple = ref + ',' + dlg.apply_on.list.GetString(i)
                    lst.list.Append([couple, nature, ntype, val])

    # -------------------------------------------------------------------
    def onChangeShift(self, event) -> None:
        data = self.orig_data
        if data:
            pos = event.GetPosition()
            self.transformed_plot.set_ydata(roll(data.signal.data_fourier[data.index].real, -pos))
            self.graph.canvas.draw()

            meta = data.signal.meta
            hz_range = data.signal.getHzRange()
            ppm_range = data.signal.getPpmRange()

            self.hz_value = - pos*(hz_range[1] - hz_range[0])/meta['size']
            ppm_value = pos*(ppm_range[1] - ppm_range[0])/meta['size']
            self.shift_hz.SetLabel(str(self.hz_value) + ' Hz')
            self.shift_ppm.SetLabel(str(ppm_value) + ' ppm')

    # -------------------------------------------------------------------
    def showPlot(self, b=True) -> None:
        if b and self.transformed_plot.axes is None:
            self.graph.axes.add_line(self.transformed_plot)
        elif not b and self.transformed_plot.axes is not None:
            self.transformed_plot.remove()
        self.graph.refresh_legend()
        self.graph.refresh()

# =======================================================================

class ScalingPanel(wx.Panel):
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, rel_panel, *args, **kwargs) -> None:
        super().__init__(parent)
        self.graph = graph
        self.orig_data = None
        self.transformed_plot = matplotlib.lines.Line2D([],[], color='black')
        self.scale_val = 1
        self.divisor = 10
        self.rel_panel = rel_panel

        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self) -> None:
        # widgets
        self.scale = wx.Slider(self, value=10, minValue=1, maxValue=10)
        self.min_txt = wx.StaticText(self, label='min:1e-1')
        self.max_txt = wx.StaticText(self, label='max:1e0')
        back_bit = wx.ArtProvider.GetBitmap(wx.ART_GO_BACK, wx.ART_BUTTON)
        next_bit = wx.ArtProvider.GetBitmap(wx.ART_GO_FORWARD, wx.ART_BUTTON)
        back_btn = wx.BitmapButton(self, bitmap=back_bit)
        next_btn = wx.BitmapButton(self, bitmap=next_bit)

        constr = wx.Button(self, label="Create relation")

        # sizers
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(back_btn, 0, wx.ALL, 5)
        hbox.Add(self.scale, 1, wx.EXPAND|wx.RIGHT, 5)
        hbox.Add(next_btn, 0, wx.ALL, 5)
        hbbox = wx.BoxSizer(wx.HORIZONTAL)
        hbbox.Add(self.min_txt, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
        hbbox.AddStretchSpacer(prop=1)
        hbbox.Add(self.max_txt, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(hbox, 0, wx.EXPAND|wx.RIGHT, 0)
        sizer.Add(hbbox, 0, wx.EXPAND|wx.RIGHT, 0)
        sizer.Add(constr, 0, wx.ALL, 10)
        self.SetSizerAndFit(sizer)

        # events
        back_btn.Bind(wx.EVT_BUTTON, self.onBackRange)
        next_btn.Bind(wx.EVT_BUTTON, self.onNextRange)
        self.scale.Bind(wx.EVT_SCROLL_THUMBTRACK, self.onChangeScale)
        constr.Bind(wx.EVT_BUTTON, self.onCreateRel)

    # -------------------------------------------------------------------
    def onChangeScale(self, event) -> None:
        data = self.orig_data
        if data:
            pos = event.GetPosition()
            self.scale_val = pos/self.divisor
            self.transformed_plot.set_ydata(data.signal.data_fourier[data.index].real \
                    * self.scale_val)
            self.graph.canvas.draw()

    # -------------------------------------------------------------------
    def setModel(self, model) -> None:
        self.orig_data = model
        self.refreshFromModelPlot()
        self.transformed_plot.set_label("Scaled")
        self.refresh_graph()
        self.graph.refresh_legend()
        self.graph.canvas.draw()

    # -------------------------------------------------------------------
    def refreshFromModelPlot(self) -> None:
        self.transformed_plot.set_xdata(self.orig_data.plot.get_xdata())
        self.transformed_plot.set_ydata(self.orig_data.plot.get_ydata())

    # -------------------------------------------------------------------
    def refreshFromModelData(self) -> None:
        sig = self.orig_data.signal
        meta = sig.meta
        bound = sig.getPpmRange()
        x = np.linspace(bound[0], bound[1], meta['size'])
        y = self.orig_data.signal.data_fourier[self.orig_data.index]
        self.transformed_plot.set_xdata(x)
        self.transformed_plot.set_ydata(y)

    # -------------------------------------------------------------------
    def refresh_graph(self) -> None:
        if self.orig_data:
            self.transformed_plot.set_ydata(self.orig_data.signal.data_fourier[0].real \
                    * self.scale_val)
            self.graph.refresh()

    # -------------------------------------------------------------------
    def onBackRange(self, event) -> None:
        self.divisor *= 10
        expo = int(np.log10(self.divisor))
        self.min_txt.SetLabel('min: 1e{}'.format(-expo))
        self.max_txt.SetLabel('max: 1e{}'.format(-expo+1))
        self.scale_val = self.scale.GetValue()/self.divisor
        self.refresh_graph()
        self.Layout()

    # -------------------------------------------------------------------
    def onNextRange(self, event) -> None:
        self.divisor /= 10
        expo = int(np.log10(self.divisor))
        self.min_txt.SetLabel('min: 1e{}'.format(-expo))
        self.max_txt.SetLabel('max: 1e{}'.format(-expo+1))
        self.scale_val = self.scale.GetValue()/self.divisor
        self.refresh_graph()
        self.Layout()

    # -------------------------------------------------------------------
    def onCreateRel(self, event) -> None:
        if self.orig_data is None:
            return

        if self.orig_data.name in self.rel_panel.meta:
            molecules = self.rel_panel.meta
            b = True
        elif self.orig_data.name in self.rel_panel.macro:
            molecules = self.rel_panel.macro
            b = False
        else:
            msg = 'The reference should be a metabolite or a macro-molecule.'
            wx.MessageDialog(self, msg).ShowModal()
            return

        with ConstraintDialog(self, molecules) as dlg:
            sel = dlg.reference.FindString(self.orig_data.name)
            dlg.reference.SetSelection(sel)
            dlg.setApplyOn()
            dlg.nature.SetSelection(2)
            dlg.type.SetSelection(0)
            dlg.val.SetMin(-1e9)
            dlg.val.SetMax(1e9)
            dlg.val.SetValue(str(self.scale_val*100))
            dlg.sizer.Show(5)
            dlg.setValueUnit('%')
            if dlg.ShowModal() == wx.ID_OK:
                ref = dlg.reference.GetString(dlg.reference.GetSelection())
                nature = dlg.nature.GetString(dlg.nature.GetSelection())
                ntype = dlg.type.GetString(dlg.type.GetSelection())
                val = str(dlg.val.GetValue()) if ntype=='soft' else "None"

                lst = self.rel_panel.meta_constr if b else self.rel_panel.macro_constr
                for i in range(dlg.apply_on.list.GetCount()):
                    couple = ref + ',' + dlg.apply_on.list.GetString(i)
                    lst.list.Append([couple, nature, ntype, val])

    # -------------------------------------------------------------------
    def showPlot(self, b=True) -> None:
        if b and self.transformed_plot.axes is None:
            self.graph.axes.add_line(self.transformed_plot)
        elif not b and self.transformed_plot.axes is not None:
            self.transformed_plot.remove()
        self.graph.refresh_legend()
        self.graph.refresh()

# =======================================================================

class DampingPanel(wx.Panel):
    # -------------------------------------------------------------------
    def __init__(self, parent, graph, rel_panel, *args, **kwargs) -> None:
        super().__init__(parent)
        self.graph = graph
        self.orig_data = None
        self.transformed_plot = matplotlib.lines.Line2D([],[], color='black')
        self.span = SpanSelector(self.graph.axes, self.onSpan, 'horizontal', useblit=True)
        self.peak_selection = None
        self.current_dval = None
        self.reference_dval = None
        self.rel_panel = rel_panel

        self.span.set_visible(False)
        self.initGui()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self) -> None:
        # widgets
        self.current_mol_txt = wx.StaticText(self)
        self.current_mol_fwhm = wx.StaticText(self)
        current_mol_btn = wx.Button(self, label='Select peak')

        self.reference_choice = wx.Choice(self)
        self.reference_fwhm = wx.StaticText(self)
        reference_btn = wx.Button(self, label='Select Peak')

        constr = wx.Button(self, label="Create relations")

        # sizers
        txt_flag = [0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5]
        cur_box = wx.BoxSizer(wx.VERTICAL)
        line1 = wx.BoxSizer(wx.HORIZONTAL)
        line1.Add(wx.StaticText(self, label="Studied signal : "), *txt_flag)
        line1.Add(self.current_mol_txt, *txt_flag)
        line2 = wx.BoxSizer(wx.HORIZONTAL)
        line2.Add(wx.StaticText(self, label='Damping : '), *txt_flag)
        line2.Add(self.current_mol_fwhm, *txt_flag)
        self.line3 = wx.BoxSizer(wx.HORIZONTAL)
        self.line3.Add(current_mol_btn, 0, wx.ALL, 5)
        self.line3.Add(wx.StaticText(self, label="Span on graph"), *txt_flag)
        self.line3.Hide(1)
        cur_box.Add(line1, 0, wx.ALL|wx.EXPAND, 5)
        cur_box.Add(line2, 0, wx.ALL|wx.EXPAND, 5)
        cur_box.Add(self.line3, 0, wx.ALL, 5)

        ref_box = wx.BoxSizer(wx.VERTICAL)
        line1b = wx.BoxSizer(wx.HORIZONTAL)
        line1b.Add(wx.StaticText(self, label="Reference : "), *txt_flag)
        line1b.Add(self.reference_choice, 0, wx.ALL, 5)
        line2b = wx.BoxSizer(wx.HORIZONTAL)
        line2b.Add(wx.StaticText(self, label='Damping : '), *txt_flag)
        line2b.Add(self.reference_fwhm, *txt_flag)
        self.line3b = wx.BoxSizer(wx.HORIZONTAL)
        self.line3b.Add(reference_btn, 0, wx.ALL, 5)
        self.line3b.Add(wx.StaticText(self, label="Span on graph"), *txt_flag)
        self.line3b.Hide(1)
        ref_box.Add(line1b, 0, wx.ALL|wx.EXPAND, 5)
        ref_box.Add(line2b, 0, wx.ALL|wx.EXPAND, 5)
        ref_box.Add(self.line3b, 0, wx.ALL, 5)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(cur_box, 1, wx.ALL|wx.EXPAND)
        hbox.Add(wx.StaticLine(self, style=wx.LI_VERTICAL), 0,
                  wx.BOTTOM|wx.EXPAND, 0)
        hbox.Add(ref_box, 1, wx.ALL|wx.EXPAND)

        btnbox = wx.BoxSizer(wx.HORIZONTAL)
        btnbox.Add(constr, 0, wx.ALL, 8)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(hbox, 1, wx.ALL|wx.EXPAND)
        sizer.Add(btnbox)
        self.SetSizer(sizer)

        # events
        current_mol_btn.Bind(wx.EVT_BUTTON, lambda evt: self.onSelectPeak(evt, 'current'))
        reference_btn.Bind(wx.EVT_BUTTON, lambda evt: self.onSelectPeak(evt, 'reference'))
        constr.Bind(wx.EVT_BUTTON, self.onCreateRel)

    # -------------------------------------------------------------------
    def onSelectPeak(self, event, src) -> None:
        self.peak_selection = src
        self.span.set_visible(True)
        if src == 'current':
            self.line3.Show(1)
            self.line3b.Hide(1)
        else:
            self.line3b.Show(1)
            self.line3.Hide(1)
        self.Layout()

    # -------------------------------------------------------------------
    def onSpan(self, xmin, xmax):
        if not self.IsShown():
            return
        self.span.set_visible(False)
        self.line3.Hide(1)
        self.line3b.Hide(1)
        data = self.orig_data
        sel = self.peak_selection
        self.peak_selection = None
        self.Layout()
        if data is None:
            return

        if sel == 'current':
            self._onSpanCurrent(xmin, xmax)
        else:
            self._onSpanReference(xmin, xmax)

        if self.current_dval is not None and \
           self.reference_dval is not None:
            delta = self.current_dval - self.reference_dval
            ind = self.orig_data.index
            try:
                new_freq = np.fft.fftshift(np.fft.fft(proc.dampe(data.signal,
                                                                 delta, ind)))
            except FloatingPointError:
                wx.MessageBox( 'over/under flow when trying to make exp function', 'Error')
                return

            self.transformed_plot.set_ydata(new_freq)
            self.graph.refresh()

    # -------------------------------------------------------------------
    def _onSpanCurrent(self, xmin, xmax) -> None:
        # calculing values
        x_axe = self.transformed_plot.get_xdata()
        data = self.orig_data
        meta = data.signal.meta
        imin = int((xmin - x_axe[0]) / (x_axe[-1] - x_axe[0]) * meta['size'])
        imax = int((xmax - x_axe[0]) / (x_axe[-1] - x_axe[0]) * meta['size'])
        if imin == imax:
            imax += 1

        try:
            fmin, fmax = proc.getIndexesFWHM(data.signal.data_fourier[data.index], imin, imax)
        except (AssertionError, RecursionError) as err:
            msg = "An error ocurred when trying to get the FWHM." + \
                  "Please select an area where there's no unwanted high peak"
            wx.MessageDialog(self, msg).ShowModal()
            print(err)
            return


        hz_range = data.signal.getHzRange()

        hz_min = fmin*(hz_range[1] - hz_range[0])/meta['size']
        hz_max = fmax*(hz_range[1] - hz_range[0])/meta['size']
        damping = (hz_max - hz_min) / np.pi

        # updating UI
        self.current_dval = damping
        self.current_mol_fwhm.SetLabel(str(damping))

    # -------------------------------------------------------------------
    def _onSpanReference(self, xmin, xmax) -> None:
        # calculing values
        if self.reference_choice.GetSelection() < 0:
            return

        data = self.reference_choice.GetClientData(self.reference_choice.GetSelection())
        x_axe = data.plot.get_xdata()
        meta = data.signal.meta
        imin = int((xmin - x_axe[0]) / (x_axe[-1] - x_axe[0]) * meta['size'])
        imax = int((xmax - x_axe[0]) / (x_axe[-1] - x_axe[0]) * meta['size'])
        if imin == imax:
            imax += 1

        try:
            fmin, fmax = proc.getIndexesFWHM(data.signal.data_fourier[data.index], imin, imax)
        except (AssertionError, RecursionError) as err:
            msg = "An error ocurred when trying to get the FWHM.\n" + \
                  "Please select an area where there's no unwanted high peak"
            wx.MessageDialog(self, msg).ShowModal()
            print(err)
            return


        hz_range = data.signal.getHzRange()

        hz_min = fmin*(hz_range[1] - hz_range[0])/meta['size']
        hz_max = fmax*(hz_range[1] - hz_range[0])/meta['size']
        damping = (hz_max - hz_min) / np.pi

        # updating UI
        self.reference_dval = damping
        self.reference_fwhm.SetLabel(str(damping))

    # -------------------------------------------------------------------
    def onCreateRel(self, event) -> None:
        if self.orig_data is None:
            return

        if self.orig_data.name in self.rel_panel.meta:
            molecules = self.rel_panel.meta
            b = True
        elif self.orig_data.name in self.rel_panel.macro:
            molecules = self.rel_panel.macro
            b = False
        else:
            msg = 'The studied signal should be a metabolite or a macro-molecule.'
            wx.MessageDialog(self, msg).ShowModal()
            return

        with ConstraintDialog(self, molecules) as dlg:
            sel = dlg.reference.FindString(self.orig_data.name)
            dlg.reference.SetSelection(sel)
            dlg.setApplyOn()
            dlg.nature.SetSelection(0)
            dlg.type.SetSelection(0)
            dlg.val.SetMin(-1e9)
            dlg.val.SetMax(1e9)
            dlg.val.SetValue(str(self.current_dval))
            dlg.sizer.Show(5)
            dlg.setValueUnit('Hz')
            if dlg.ShowModal() == wx.ID_OK:
                ref = dlg.reference.GetString(dlg.reference.GetSelection())
                nature = dlg.nature.GetString(dlg.nature.GetSelection())
                ntype = dlg.type.GetString(dlg.type.GetSelection())
                val = str(dlg.val.GetValue()) if ntype=='soft' else "None"

                lst = self.rel_panel.meta_constr if b else self.rel_panel.macro_constr
                for i in range(dlg.apply_on.list.GetCount()):
                    couple = ref + ',' + dlg.apply_on.list.GetString(i)
                    lst.list.Append([couple, nature, ntype, val])

    # -------------------------------------------------------------------
    def setModel(self, model) -> None:
        self.current_mol_txt.SetLabel(model.name)
        self.orig_data = model
        self.refreshFromModelPlot()
        self.transformed_plot.set_label("Damped")
        self.graph.refresh_legend()
        self.graph.refresh()

    # -------------------------------------------------------------------
    def refreshFromModelPlot(self) -> None:
        self.transformed_plot.set_xdata(self.orig_data.plot.get_xdata())
        self.transformed_plot.set_ydata(self.orig_data.plot.get_ydata())

    # -------------------------------------------------------------------
    def refreshFromModelData(self) -> None:
        sig = self.orig_data.signal
        meta = sig.meta
        bound = sig.getPpmRange()
        x = np.linspace(bound[0], bound[1], meta['size'])
        y = self.orig_data.signal.data_fourier[self.orig_data.index]
        self.transformed_plot.set_xdata(x)
        self.transformed_plot.set_ydata(y)

    # -------------------------------------------------------------------
    def showPlot(self, b=True) -> None:
        if b and self.transformed_plot.axes is None:
            self.graph.axes.add_line(self.transformed_plot)
        elif not b and self.transformed_plot.axes is not None:
            self.transformed_plot.remove()
        self.graph.refresh_legend()
        self.graph.refresh()

# =======================================================================

class FittingOptionPanel(scrolled.ScrolledPanel):
    """
    Represent all the fitting options for QUEST.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        super(FittingOptionPanel, self).__init__(*args, **kwargs)
        self.SetupScrolling(False, True)
        self.SetFocus()
        self.initGui()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        self.gsizer = wx.FlexGridSizer(2, gap=(50, 10))

        #---
        param = wx.StaticText(self, label="Lineshape",
                style=wx.ALIGN_RIGHT)
        self.lineshape = wx.Choice(self, choices=["Lorentzian", "Gaussian"])
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.lineshape, 0)])

        #---
        param = wx.StaticText(self, label="Background selection",
                style=wx.ALIGN_RIGHT)
        self.backsel = wx.Choice(self, choices=["None", "In base", "Substract"])
        self.backsel.Bind(wx.EVT_CHOICE, self.onChangeBackground)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.backsel, 0)])

        #---
        param = wx.StaticText(self, label="Background SVD",
                style=wx.ALIGN_RIGHT)
        self.svd = []
        self.svd.append(wx.SpinCtrl(self, min=0, max=1000000))
        self.svd.append(wx.SpinCtrl(self, min=0, max=1000000))
        self.svd.append(wx.SpinCtrl(self, min=0, max=1000000))
        inpt = wx.BoxSizer(wx.VERTICAL)
        tmp = wx.BoxSizer(wx.HORIZONTAL)
        tmp.Add(self.svd[0], 0)
        txt = wx.StaticText(self, label="nb points")
        fnt = wx.Font(10, wx.DECORATIVE, wx.NORMAL, wx.LIGHT)
        txt.SetFont(fnt)
        tmp.Add(txt, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        inpt.Add(tmp, 1, wx.BOTTOM, 2)
        tmp = wx.BoxSizer(wx.HORIZONTAL)
        tmp.Add(self.svd[1], 0)
        txt = wx.StaticText(self, label="nb column")
        txt.SetFont(fnt)
        tmp.Add(txt, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        inpt.Add(tmp, 1, wx.BOTTOM, 2)
        tmp = wx.BoxSizer(wx.HORIZONTAL)
        tmp.Add(self.svd[2], 0)
        txt = wx.StaticText(self, label="nb component")
        txt.SetFont(fnt)
        tmp.Add(txt, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        inpt.Add(tmp, 1, wx.BOTTOM, 2)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (inpt, 0)])

        #---
        param = wx.StaticText(self, label="Background threshold",
                style=wx.ALIGN_RIGHT)
        self.threshold = wx.SpinCtrl(self, min=0, max=1000)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.threshold, 0)])

        #---
        param = wx.StaticText(self, label="Truncation number",
                style=wx.ALIGN_RIGHT)
        self.trunc = wx.SpinCtrl(self, min=0, max=1000000)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.trunc, 0)])

        #---
        param = wx.StaticText(self, label="Iteration number",
                style=wx.ALIGN_RIGHT)
        self.iter = wx.SpinCtrl(self, min=0, max=1000000)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.iter, 0)])

        #---
        param = wx.StaticText(self, label="Phi0 estimation",
                style=wx.ALIGN_RIGHT)
        self.phiest = wx.Choice(self, choices=["Fixed", "Estimated"])
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.phiest, 0)])

        #---
        param = wx.StaticText(self, label="Phi0 initialization (°)",
                style=wx.ALIGN_RIGHT)
        self.phinit = wx.SpinCtrl(self, min=-180, max=180)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.phinit, 0)])

        #---
        param = wx.StaticText(self, label="Extra Damping",
                style=wx.ALIGN_RIGHT)
        self.damping = []
        self.damping.append(wx.SpinCtrl(self, min=-1000000, max=1000000))
        self.damping.append(wx.SpinCtrl(self, min=-1000000, max=1000000))
        inpt = wx.BoxSizer(wx.VERTICAL)
        tmp = wx.BoxSizer(wx.HORIZONTAL)
        tmp.Add(self.damping[0])
        txt = wx.StaticText(self, label="min")
        txt.SetFont(fnt)
        tmp.Add(txt, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        inpt.Add(tmp, 1, wx.BOTTOM, 2)
        tmp = wx.BoxSizer(wx.HORIZONTAL)
        tmp.Add(self.damping[1])
        txt = wx.StaticText(self, label="max")
        txt.SetFont(fnt)
        tmp.Add(txt, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        inpt.Add(tmp, 1, wx.BOTTOM, 2)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (inpt, 0)])

        #---
        param = wx.StaticText(self, label="Frequency Shift",
                style=wx.ALIGN_RIGHT)
        self.fshift = []
        self.fshift.append(wx.SpinCtrl(self, min=-1000000, max=1000000))
        self.fshift.append(wx.SpinCtrl(self, min=-1000000, max=1000000))
        inpt = wx.BoxSizer(wx.VERTICAL)
        txt = wx.StaticText(self, label="min")
        txt.SetFont(fnt)
        tmp = wx.BoxSizer(wx.HORIZONTAL)
        tmp.Add(self.fshift[0])
        tmp.Add(txt, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        inpt.Add(tmp, 1, wx.BOTTOM, 2)
        tmp = wx.BoxSizer(wx.HORIZONTAL)
        tmp.Add(self.fshift[1])
        txt = wx.StaticText(self, label="max")
        txt.SetFont(fnt)
        tmp.Add(txt, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        inpt.Add(tmp, 1, wx.BOTTOM, 2)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (inpt, 0)])

        #---
        param = wx.StaticText(self, label="Last point",
                style=wx.ALIGN_RIGHT)
        self.lastpoint = wx.SpinCtrl(self, min=0, max=10000)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.lastpoint, 0)])

        #---
        param = wx.StaticText(self, label="Number of test",
                style=wx.ALIGN_RIGHT)
        self.nbtest = wx.SpinCtrl(self, min=0, max=1000)
        self.gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.nbtest, 0)])

        #---#

        adv_btn = wx.Button(self, label="Advanced options")
        adv_btn.Bind(wx.EVT_BUTTON, self.onShowAdvanced)

        adv_panel = wx.Panel(self)
        adv_gsizer = wx.FlexGridSizer(2, gap=(10, 10))

        param = wx.StaticText(adv_panel, label="Save quest parameters output",
                style=wx.ALIGN_RIGHT)
        self.savetxt = wx.CheckBox(adv_panel)
        self.savetxt.SetValue(wx.CHK_CHECKED)
        adv_gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.savetxt, 0)])

        param = wx.StaticText(adv_panel, label="HLSVD",
                style=wx.ALIGN_RIGHT)
        self.hlsvd = wx.CheckBox(adv_panel)
        self.hlsvd.SetValue(wx.CHK_CHECKED)
        adv_gsizer.AddMany([(param, 0, wx.ALIGN_CENTER_VERTICAL), (self.hlsvd, 0)])

        adv_panel.SetSizerAndFit(adv_gsizer)

        # Hide backgrounds option by default, see onChangeBackground
        self.gsizer.Hide(4)
        self.gsizer.Hide(5)
        self.gsizer.Hide(6)
        self.gsizer.Hide(7)

        self.globalsizer = wx.BoxSizer(wx.VERTICAL)
        self.globalsizer.Add(self.gsizer, 0, wx.ALL, 8)
        self.globalsizer.Add(adv_btn, 0, wx.ALL, 8)
        self.globalsizer.Add(wx.StaticLine(self, style=wx.LI_HORIZONTAL), 0,
                             wx.EXPAND|wx.RIGHT, 8)
        self.globalsizer.Add(adv_panel, 0, wx.ALL, 8)
        self.globalsizer.Hide(3)
        self.SetSizer(self.globalsizer)

    # -------------------------------------------------------------------
    def onShowAdvanced(self, event):
        """
        Hide or show advanced options, according to the previous state.
        """
        if self.globalsizer.IsShown(3):
            self.globalsizer.Hide(3)
        else:
            self.globalsizer.Show(3)
        self.Layout()
        self.FitInside()

    # -------------------------------------------------------------------
    def onChangeBackground(self, event=None, is_show=None) -> None:
        """
        Show or Hide options for SVD background.

        Kwargs :
            event : if called with a wx.Window.Bind function.
            is_show : bool, if precised this will always be the used value.
        """

        if is_show is not None:
            b = is_show
        elif event is not None:
            b = not bool(event.GetSelection())
        else:
            return

        if b:
            self.gsizer.Hide(4)
            self.gsizer.Hide(5)
            self.gsizer.Hide(6)
            self.gsizer.Hide(7)
        else:
            self.gsizer.Show(4)
            self.gsizer.Show(5)
            self.gsizer.Show(6)
            self.gsizer.Show(7)

        self.FitInside()

    # -------------------------------------------------------------------
    def getData(self)->dict:
        """
        Return a dictionary according to the syntax from QuestParameter class.
        """

        dic = {}
        dic['lineshape'] = [self.lineshape.GetString(self.lineshape.GetSelection())]
        dic['useHLSVD'] = [str(self.hlsvd.IsChecked()).lower()]
        dic['backsel'] = [str(self.backsel.GetSelection())]
        dic['nbTrunc'] = [str(self.trunc.GetValue())]
        dic['nbIteration'] = [str(self.iter.GetValue())]
        dic['estim_phi0'] = [self.phiest.GetString(self.phiest.GetSelection())]
        dic['init_phi0'] = [str(self.phinit.GetValue())]
        dic['ExtraDampinglow/upp'] = []
        for inpt in self.damping:
            dic['ExtraDampinglow/upp'].append(str(inpt.GetValue()))
        dic['FrequencyShiftlow/upp'] = []
        for inpt in self.fshift:
            dic['FrequencyShiftlow/upp'].append(str(inpt.GetValue()))
        dic['backgroundSVD'] = []
        for inpt in self.svd:
            dic['backgroundSVD'].append(str(inpt.GetValue()))
        dic['backgroundThres'] = [str(self.threshold.GetValue())]
        if self.savetxt.IsChecked():
            dic['SaveTXT2'] = ["y"]
        else:
            dic['SaveTXT2'] = ["n"]
        dic['lastPoint'] = [str(self.lastpoint.GetValue())]
        dic['nbTest'] = [str(self.nbtest.GetValue())]

        return dic

    # -------------------------------------------------------------------
    def setUIvalue(self, dic)->None:
        """
        Set GUI value from the 'dic' argument. o

        Args :
            - dic : dictionary that must respect QuestParameters syntax
        """

        self.lineshape.SetSelection(self.lineshape.FindString(dic['lineshape'][0]))
        if dic['useHLSVD'][0] == "true":
            self.hlsvd.SetValue(wx.CHK_CHECKED)
        else:
            self.hlsvd.SetValue(wx.CHK_UNCHECKED)
        self.backsel.SetSelection(int(dic['backsel'][0]))
        self.trunc.SetValue(int(dic['nbTrunc'][0]))
        self.iter.SetValue(int(dic['nbIteration'][0]))
        self.phiest.SetSelection(self.phiest.FindString(dic['estim_phi0'][0]))
        self.phinit.SetValue(int(dic['init_phi0'][0]))
        for i, elm in enumerate(dic['ExtraDampinglow/upp']):
            self.damping[i].SetValue(int(elm))
        for i, elm in enumerate(dic['FrequencyShiftlow/upp']):
            self.fshift[i].SetValue(int(elm))
        for i, elm in enumerate(dic['backgroundSVD']):
            self.svd[i].SetValue(int(elm))
        self.threshold.SetValue(int(dic['backgroundThres'][0]))
        if dic['SaveTXT2'][0] == "y":
            self.savetxt.SetValue(wx.CHK_CHECKED)
        else:
            self.savetxt.SetValue(wx.CHK_UNCHECKED)
        self.lastpoint.SetValue(int(dic['lastPoint'][0]))
        self.nbtest.SetValue(int(dic['nbTest'][0]))

        is_show = not bool(int(dic['backsel'][0]))
        self.onChangeBackground(is_show=is_show)

# =======================================================================

class QuestViewerPanel(wx.Panel):
    """
    The viewer for a QUEST folder. This has 3 tabs for 3 different display :
    time domain, frequency domain and fitting details.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        super(QuestViewerPanel, self).__init__(*args, **kwargs)
        self.initGui()

        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        nb = wx.Notebook(self)

        self.time_panel = QuestCanvasPanel(nb)
        self.freq_panel = QuestCanvasPanel(nb)

        nb.AddPage(self.time_panel, "Time Domain")
        nb.AddPage(self.freq_panel, "Frequency Domain")

        sizer.Add(nb, 2, wx.ALL | wx.EXPAND, 5)

        self.SetSizer(sizer)

    # -------------------------------------------------------------------
    def refreshLegend(self):
        # legend for both time and frequency domain
        legend_patches = []
        legend_patches.append(mpatches.Patch(color='slategray', label='Residue'))
        legend_patches.append(mpatches.Patch(color='seagreen', label='Original Spectrum'))
        legend_patches.append(mpatches.Patch(color='dodgerblue', label='Background'))
        legend_patches.append(mpatches.Patch(color='crimson', label='Estimated Spectrum'))
        self.time_panel.graph.axes.legend(handles=legend_patches) 
        self.freq_panel.graph.axes.legend(handles=legend_patches)

    # -------------------------------------------------------------------
    def loadFolder(self, pathname)->None:
        """
        Args : 
            - pathname : the path for the target folder, with
                          '/foo/bar/folder' syntax, and NOT '/foo/bar/folder/'. This is
                          relevant on *NIX system, see os.path.basename Python
                          documentation for more infos.
        """

        prefix = os.path.basename(pathname)
        long_prefix = os.path.join(pathname, prefix+'_')

        self.back_sig = Signal(long_prefix + 'quest_back.mrui')
        self.estim_sig = Signal(long_prefix + 'quest_estim.mrui')
        self.orig_sig = Signal(long_prefix + 'quest_orig.mrui')

        self.time_panel.index_sig.SetMax(len(self.orig_sig.data) - 1)
        self.freq_panel.index_sig.SetMax(len(self.orig_sig.data) - 1)

    # -------------------------------------------------------------------
    def draw(self, index=0) -> None:
        """
        Draw the 2 tabs.
        """

        self.drawTime(index)
        self.drawFreq(index)

    # -------------------------------------------------------------------
    def drawTime(self, index=0) -> None:
        """
        Draw the Time domain graph
        """

        residu = self.orig_sig.data[index] - self.estim_sig.data[index]
        pnl = self.time_panel
        pnl.graph.clear()

        pnl.plots = []
        pnl.plots.append(pnl.graph.axes.plot(self.orig_sig.data[index],
            color="seagreen", linewidth=0.8)[0])
        pnl.plots.append(pnl.graph.axes.plot(self.back_sig.data[index],
            color="dodgerblue", linewidth=0.9)[0])
        pnl.plots.append(pnl.graph.axes.plot(self.estim_sig.data[index],
            color="crimson", linewidth=1)[0])
        pnl.plots.append(pnl.graph.axes.plot(residu,
            color="slategray", linewidth=0.7)[0])
        self.refreshLegend()

    # -------------------------------------------------------------------
    def drawFreq(self, index=0) -> None:
        """
        Draw the frequency domain graph
        """

        residu = self.orig_sig.data_fourier[index] - self.estim_sig.data_fourier[index]
        pnl = self.freq_panel
        pnl.graph.clear()

        bounds = self.orig_sig.getPpmRange()
        f = linspace(bounds[0], bounds[1], self.orig_sig.meta['size'])

        pnl.plots = []
        pnl.plots.append(pnl.graph.axes.plot(f, self.orig_sig.data_fourier[index],
            color="seagreen", linewidth=0.8)[0])
        pnl.plots.append(pnl.graph.axes.plot(f, self.back_sig.data_fourier[index],
            color="dodgerblue", linewidth=0.9)[0])
        pnl.plots.append(pnl.graph.axes.plot(f, self.estim_sig.data_fourier[index],
            color="crimson", linewidth=1)[0])
        pnl.plots.append(pnl.graph.axes.plot(f, residu,
            color="slategray", linewidth=0.7)[0])
        self.refreshLegend()

# =======================================================================

class WaitQuestDialog(wx.Dialog):
    """
    A diolog which show a gauge in indeterminated mode so the user can know that
    quest is running.
    """
    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs):
        super().__init__(*args, style=wx.CAPTION, **kwargs)
        self.initGui()
        self.Layout()
        self.Center()

    # -------------------------------------------------------------------
    def initGui(self):
        pnl = wx.Panel(self)
        self.waiter = Waiting(pnl)
        self.msg = wx.StaticText(pnl, label="Quest is running ...")
        self.ok = wx.Button(pnl, label='Ok')

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.msg, 0,
                       wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 10)
        self.sizer.Add(self.waiter, 0, wx.ALL|wx.EXPAND, 10)
        self.sizer.Add(self.ok, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)
        self.sizer.Hide(self.ok)
        pnl.SetSizerAndFit(self.sizer)

        self.ok.Bind(wx.EVT_BUTTON, self.onOk)

    # -------------------------------------------------------------------
    def ShowModal(self):
        """
        Override ShowModal so the gauge can start pulsing.
        """
        self.waiter.start()
        super().ShowModal()

    # -------------------------------------------------------------------
    def onEndProcess(self, *args):
        self.waiter.stop()
        self.msg.SetLabel("Quest has finished !")
        self.sizer.Show(self.ok)
        self.Layout()

    # -------------------------------------------------------------------
    def onOk(self, event):
        self.EndModal(wx.ID_OK)

# =======================================================================

class QuestCanvasPanel(wx.Panel):
    """
    A canvas panel that is designed to display both Time domain
    and Frequency domain graph.
    You can decide to draw the original/background/estimation/residu signal
    with CheckBoxes on the side.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        super(QuestCanvasPanel, self).__init__(*args, **kwargs)
        self.plots = []
        self.initGui()

        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        self.graph = CanvasPanel(self)
        vpanel = wx.Panel(self)
        txt_index = wx.StaticText(vpanel, label="Index of the displayed signal :")
        self.index_sig = wx.SpinCtrl(vpanel, min=0, max=0)
        display_opt = wx.StaticBoxSizer(wx.VERTICAL, vpanel, "Display :")
        ref_disp = wx.CheckBox(vpanel, label="Reference")
        ref_disp.SetValue(True)
        est_disp = wx.CheckBox(vpanel, label="Estimation")
        est_disp.SetValue(True)
        back_disp = wx.CheckBox(vpanel, label="Background")
        back_disp.SetValue(True)
        residu_disp = wx.CheckBox(vpanel, label="Residu")
        residu_disp.SetValue(True)

        ref_disp.Bind(wx.EVT_CHECKBOX,
                lambda evt, idx=0: self.onCheckDisplay(evt, idx))
        est_disp.Bind(wx.EVT_CHECKBOX,
                lambda evt, idx=2: self.onCheckDisplay(evt, idx))
        back_disp.Bind(wx.EVT_CHECKBOX,
                lambda evt, idx=1: self.onCheckDisplay(evt, idx))
        residu_disp.Bind(wx.EVT_CHECKBOX,
                lambda evt, idx=3: self.onCheckDisplay(evt, idx))

        display_opt.Add(ref_disp, 0, wx.ALL, 5)
        display_opt.Add(est_disp, 0, wx.ALL, 5)
        display_opt.Add(back_disp, 0, wx.ALL, 5)
        display_opt.Add(residu_disp, 0, wx.ALL, 5)
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(txt_index, 0, wx.ALL, 2)
        vbox.Add(self.index_sig, 0, wx.ALL, 2)
        vbox.Add(display_opt, 0, wx.ALL | wx.EXPAND, 2)
        vpanel.SetSizer(vbox)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(self.graph, 1, wx.EXPAND|wx.ALL, 2)
        hbox.Add(vpanel, 0, wx.ALL, 5)
        self.SetSizer(hbox)
        self.SetBackgroundColour(wx.Colour(230, 230, 230))

    # -------------------------------------------------------------------
    def onCheckDisplay(self, event, idx):
        try:
            if event.IsChecked():
                self.plots[idx].set_visible(True)
            else:
                self.plots[idx].set_visible(False)
            self.graph.canvas.draw()
        except IndexError:
            print("No folder loaded")

# =======================================================================

class ListContextMenu(wx.Menu):
    """
    A context menu used to restore the original signal in quest frame when right
    clik on items.
    """
    def __init__(self, data, draw):
        super().__init__()
        self.data = data
        self.draw = draw

        restore = self.Append(wx.NewId(), "Restore")
        
        self.Bind(wx.EVT_MENU, self.onRestore, restore)

    def onRestore(self, event):
        self.data.signal.load(self.data.origin, change_path=False)
        self.data.plot.set_ydata(self.data.signal.data_fourier[self.data.index].real)
        self.draw()

# =======================================================================

if __name__ == "__main__":
    pass
