# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
The ready-to-use widgets to deal with QUEST.
"""

import os
import os.path
import shutil
import tempfile
import wx

from utils.constants import SHOWCASE_COLOR, BACKGROUND_COLOR, FOREGROUND_COLOR
from .parameters import QuestParam
from . import guitools as Gui
from .qthreading import QuestThread, VipThread
from .model import QuestModel
from utils import *
from mrsignal import voxel


# =======================================================================

class QuestPanel(wx.Panel):
    """
    Used for QUEST parameters file editing.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        super().__init__(*args, **kwargs)
        self.cwd = os.getcwd()
        self.wts_basis = None # will be set by ProcessQuestFrame just after init

        self.model = QuestModel()

        self.initGui()
        self.initEvent()
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        # elements
        self.graph = CanvasPanel(self)

        self.nb = wx.Notebook(self)
        self.page_fitt_constr = Gui.FittingOptionPanel(self.nb)
        self.rel_constr = Gui.ConstraintPanel(self.nb)
        self.tools = Gui.ToolsPanel(self.nb, self.graph, self.model)
        self.nb.AddPage(self.page_fitt_constr, "Fitting parameters")
        self.nb.AddPage(self.rel_constr, "Relations")
        self.nb.AddPage(self.tools, "Tools")

        loaded = wx.Panel(self)
        self.signals = PimpedListCtrl(loaded, ['Path'], "Signals", False)
        self.meta = PimpedListCtrl(loaded, ['Name', 'Path'], "Metabolites")
        self.macro = PimpedListCtrl(loaded, ['Name', 'Path'], "Macro-molecules")

        # sizers
        loaded_sizer = wx.BoxSizer(wx.VERTICAL)
        loaded_sizer.Add(self.signals, 1, wx.ALL|wx.EXPAND, 1)
        loaded_sizer.Add(self.meta, 1, wx.ALL|wx.EXPAND, 1)
        loaded_sizer.Add(self.macro, 1, wx.ALL|wx.EXPAND, 1)
        loaded.SetSizerAndFit(loaded_sizer)
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.graph, 2, wx.EXPAND|wx.ALL, 1)
        sizer.Add(self.nb, 1, wx.EXPAND|wx.ALL, 1)
        sizer.Add(loaded, 1, wx.EXPAND|wx.ALL, 1)

        self.SetSizerAndFit(sizer)

    # -------------------------------------------------------------------
    def initEvent(self) -> None:
        """
        set all the bindings.
        """
        #signals
        self.signals.Bind(EVT_PMPD_LIST_EVENT,
                          lambda evt: self._listEvtHandler(evt, 'sig'))
        self.signals.list.Bind(wx.EVT_LIST_ITEM_SELECTED,
                               lambda evt: self._selectHandler(evt, 'sig'))
        self.signals.list.Bind(wx.EVT_LIST_ITEM_DESELECTED,
                               lambda evt: self._deselectHandler(evt, 'sig'))
        self.signals.list.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK,
                                lambda evt: self.onRightClick(evt, 'sig'))

        #meta
        self.meta.Bind(EVT_PMPD_LIST_EVENT,
                          lambda evt: self._listEvtHandler(evt, 'met'))
        self.meta.list.Bind(wx.EVT_LIST_ITEM_SELECTED,
                               lambda evt: self._selectHandler(evt, 'met'))
        self.meta.list.Bind(wx.EVT_LIST_ITEM_DESELECTED,
                               lambda evt: self._deselectHandler(evt, 'met'))
        self.meta.list.Bind(wx.EVT_LIST_END_LABEL_EDIT,
                               lambda evt: self.onEndEdit(evt, 'met'))
        self.meta.list.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK,
                                lambda evt: self.onRightClick(evt, 'met'))

        #macro
        self.macro.Bind(EVT_PMPD_LIST_EVENT,
                          lambda evt: self._listEvtHandler(evt, 'mac'))
        self.macro.list.Bind(wx.EVT_LIST_ITEM_SELECTED,
                               lambda evt: self._selectHandler(evt, 'mac'))
        self.macro.list.Bind(wx.EVT_LIST_ITEM_DESELECTED,
                               lambda evt: self._deselectHandler(evt, 'mac'))
        self.macro.list.Bind(wx.EVT_LIST_END_LABEL_EDIT,
                               lambda evt: self.onEndEdit(evt, 'mac'))
        self.macro.list.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK,
                                lambda evt: self.onRightClick(evt, 'mac'))

        #notebook
        self.nb.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self._nbHandler)

    # -------------------------------------------------------------------
    def _nbHandler(self, event):
        """
        events handler for the notebook.
        """
        if event.GetOldSelection() == 2:
            self.tools._showAllToolsPlot(False)

        if event.GetSelection() == 2:
            self.tools._showCurrentToolPlot()

    # -------------------------------------------------------------------
    def _listEvtHandler(self, event, src) -> None:
        """
        events handler for the 3 lists of files.

        Args :
            - event : event fired by wx
            - src : string in ['add', 'del', 'edt']
        """
        if event.action == 'add':
            self.addFile(src)
        elif event.action == 'del':
            self.delFile(src)
        elif event.action == 'edt':
            self.editName(src)

    # -------------------------------------------------------------------
    def _selectHandler(self, event, src) -> None:
        """
        events handler for the 3 lists of files when selection.

        Args :
            - event : event fired by wx
            - src : string in ['sig', 'met', 'mac']
        """
        self.graph.axes.autoscale(True)

        if src == 'sig':
            data = self.model.getSignalData(event.GetIndex())
        elif src == 'met':
            data = self.model.getMetaData(event.GetIndex())
        elif src == 'mac':
            data = self.model.getMacroData(event.GetIndex())

        self.tools.appendChoice(data)

        plot, = self.graph.draw1dSignal(data.signal, False, idx_m=data.index)
        name = data.name
        if len(name) > 10:
            name = name[:7] + '...'
        else:
            name[:10]
        plot.set_label(name)
        data.plot = plot
        self.graph.refresh_legend()
        self.graph.refresh()

    # -------------------------------------------------------------------
    def _deselectHandler(self, event, src) -> None:
        """
        events handler for the 3 lists of files when deselection.

        Args :
            - event : event fired by wx
            - src : string in ['sig', 'met', 'mac']
        """
        self.graph.axes.autoscale(True)

        if src == 'sig':
            data = self.model.getSignalData(event.GetIndex())
        elif src == 'met':
            data = self.model.getMetaData(event.GetIndex()) #metabolite data
        elif src == 'mac':
            data = self.model.getMacroData(event.GetIndex())

        self.tools.deleteChoice(data)

        data.plot.remove()
        data.plot = None
        self.graph.refresh_legend()
        self.graph.refresh()

    # -------------------------------------------------------------------
    def onRightClick(self, event, src) -> None:
        """
        events handler for the 3 lists of files when right click.

        Args :
            - event : event fired by wx
            - src : string in ['sig', 'met', 'mac']
        """
        if src == 'sig':
            lst = self.model.getSigList()
        elif src == 'met':
            lst = self.model.getMetaList()
        elif src == 'mac':
            lst = self.model.getMacroList()
        if len(lst) == 0:
            return

        index = event.GetIndex()
        parent = event.GetEventObject()
        data = lst[index]
        pos = parent.GetItemPosition(index)

        parent.PopupMenu(Gui.ListContextMenu(data, self.graph.canvas.draw_idle), pos)

    # -------------------------------------------------------------------
    def addFile(self, src) -> None:
        """
        Open a file dialog to select files and add them with `addPaths`

        Args :
            - src : string in ['sig', 'met', 'mac']
        """
        with wx.FileDialog(self, "Open a signal file", wildcard=SIGNAL_WILDCARDS,
                           style=wx.FD_OPEN|wx.FD_FILE_MUST_EXIST|
                           wx.FD_CHANGE_DIR|wx.FD_MULTIPLE) as filedialog:
            if filedialog.ShowModal() == wx.ID_CANCEL:
                return ""
            pathnames = filedialog.GetPaths()
            self.addPaths(src, pathnames)

    # -------------------------------------------------------------------
    def addPaths(self, src, pathnames, names=None) -> None:
        """
        Add files in the model and the GUI.

        Args :
            - src : string in ['sig', 'met', 'mac']
            - pathnames : list of strings (paths) to add
            - names : if name is not None, list of names to set to each file.
                      Must have the same length of `pathnames` and settled in
                      the same order.
        """
        if src=='sig':
            wts = self.GetParent().where_to_save
            for path in pathnames:
                sig = voxel.Signal(path)
                self.model.addSignal(sig, origin=path)
                new_path = os.path.join(wts, self.model.getSigList()[-1].name)
                new_path = os.path.join(new_path, os.path.basename(path))
                sig.path = new_path
                self.signals.list.Append([new_path])
        else:
            if src == 'met':
                lst = self.meta.list
                model_add = self.model.addMeta
                rel_add = self.rel_constr.addMeta
            else:
                lst = self.macro.list
                model_add = self.model.addMacro
                rel_add = self.rel_constr.addMacro

            if names is not None:
                for path, name in zip(pathnames, names):
                    sig = voxel.Signal(path)
                    new_dest = os.path.join(self.wts_basis, os.path.basename(path))

                    sig.path = new_dest
                    lst.Append([name, new_dest])
                    model_add(sig, origin=path)
                    rel_add(name)
            else:
                for path in pathnames:
                    sig = voxel.Signal(path)
                    new_dest = os.path.join(self.wts_basis, os.path.basename(path))
                    name = os.path.splitext(os.path.basename(new_dest))[0]

                    sig.path = new_dest
                    lst.Append([name, new_dest])
                    model_add(sig, origin=path)
                    rel_add(name)

    # -------------------------------------------------------------------
    def delFile(self, src) -> None:
        """
        Delete the currently selectionned files.

        Args :
            - src : string in ['sig', 'met', 'mac']
        """
        if src == 'sig':
            lst = self.signals
            model_del = self.model.delSignal
            model_data = self.model.getSignalData
            rel_del = None
        elif src == 'met':
            lst = self.meta
            model_del = self.model.delMeta
            model_data = self.model.getMetaData
            rel_del = self.rel_constr.delMeta
        else:
            lst = self.macro
            model_del = self.model.delMacro
            model_data = self.model.getMacroData
            rel_del = self.rel_constr.delMacro

        todlt = lst.GetSelections()
        for i in reversed(todlt):
            self.tools.deleteChoice(model_data(i))
            lst.list.DeleteItem(i)
            if rel_del:
                rel_del(model_data(i).name)

            model_del(i)

        self.graph.refresh_legend()
        self.graph.refresh()

    # -------------------------------------------------------------------
    def editName(self, src) -> None:
        """
        Start an Edit event on the currently focused item.

        Args :
            - src : string in ['met', 'mac']
        """
        if src == 'met':
            lst = self.meta
        elif src == 'mac':
            lst = self.macro

        elm = lst.list.GetFocusedItem()
        if elm >= 0:
            lst.list.EditLabel(elm)

    # -------------------------------------------------------------------
    def onEndEdit(self, event, src) -> None:
        """
        Apply the current edition.

        Args :
            - event: event fired by wx
            - src : string in ['met', 'mac']
        """
        if event.IsEditCancelled():
            return

        if src == 'met':
            get_data = self.model.getMetaData
        elif src == 'mac':
            get_data = self.model.getMacroData

        data = get_data(event.GetIndex())
        if data.plot is None:
            return

        data.plot.set_label(event.GetLabel())
        data.name = event.GetLabel()

        self.tools.refreshChoices()
        self.graph.refresh_legend()
        self.graph.canvas.draw()

    # -------------------------------------------------------------------
    def getParameters(self) -> QuestParam:
        """
        Extract infos from the GUI and the model, and store them into self.qparam,
        which is a QuestParam instance.
        """
        qparam = QuestParam()

        for sig in self.model.getMetaList():
            qparam.met.append([sig.signal.path, sig.name, '1', '0'])
        for sig in self.model.getMacroList():
            qparam.macro.append([sig.signal.path, sig.name, '1', '0'])
        qparam.rel = self.rel_constr.extractConstraints()
        qparam.fit = self.page_fitt_constr.getData()

        return qparam

    # -------------------------------------------------------------------
    def setFromParameters(self, qparam)->None:
        """ 
        Set the GUI and the model values.

        Args :
            - qparam : `QuestParameters` instance used to fill the GUI and model
        """
        # delte all
        for i in range(self.meta.list.GetItemCount()-1, -1, -1):
            data = self.model.getMetaData(i)
            self.meta.list.DeleteItem(i)
            self.rel_constr.delMeta(data)
            self.tools.deleteChoice(data)
            self.model.delMeta(i)
        for i in range(self.macro.list.GetItemCount()-1, -1, -1):
            data = self.model.getMacroData(i)
            self.macro.list.DeleteItem(i)
            self.rel_constr.delMacro(data)
            self.tools.deleteChoice(data)
            self.model.delMacro(i)

        # metabolites
        if len(qparam.met) > 0:
            lst = [(e[0], e[1]) for e in qparam.met]
            pathnames, names = zip(*lst)
            self.addPaths('met', pathnames, names)

        # macro molecules
        if len(qparam.macro) > 0:
            lst = [(e[0], e[1]) for e in qparam.macro]
            pathnames, names = zip(*lst)
            self.addPaths('mac', pathnames, names)

        # other
        self.rel_constr.loadConstraints(qparam.rel)
        self.page_fitt_constr.setUIvalue(qparam.fit)

    # -------------------------------------------------------------------
    def onSaveParameters(self, event)->None:
        """
        Save current parameters to a file. Open a file dialog.
        """
        old = os.getcwd()
        os.chdir(self.cwd)

        with wx.FileDialog(self, "Save quest file",
                           style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT) as filedial:
            if filedial.ShowModal() == wx.ID_CANCEL:
                return
            pathname = filedial.GetPath()
            self.getParameters().save(pathname)

        os.chdir(old)

    # -------------------------------------------------------------------
    def onLoadParameters(self, event)->None:
        """
        Load parameters and all the files precised inside from a quest paramters
        file. If there was already files loaded in mrsviewer, they will be
        unload.
        """
        old = os.getcwd()
        os.chdir(self.cwd)

        with wx.FileDialog(self, "Open quest file",
                           style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as filedial:
            if filedial.ShowModal() == wx.ID_CANCEL:
                return
            pathname = filedial.GetPath()
            qparam = QuestParam()
            qparam.load(pathname)
            self.setFromParameters(qparam)

        os.chdir(old)

# =======================================================================

class ProcessQuestFrame(wx.Frame):
    """
    Sort of wrapper for a list of signals to process and QuestPanel.
    """

    # -------------------------------------------------------------------
    def __init__(self, frame, *args, **kwargs)->None:
        super().__init__(frame, *args, **kwargs)
        # gui
        self.initGui()
        self.initToolbar()
        # datas
        self.where_to_save = None
        self.qparam = QuestParam()

        # we suppose that if frame is None, this frame was launched from command
        # line arguments, else frome the MainFrame instance.
        if frame is not None:
            try:
                self.qparam.load(wx.GetApp().pref.quest_file_path)
                self.qpanel.setFromParameters(self.qparam)
            except FileNotFoundError:
                wx.MessageBox("The default QUEST parameters file was not found.",
                               'Information', wx.OK)
            self.statusbar_frame = frame.statusbar
            self.endproc_frame = frame.onEndProcess
        else:
            self.waiting_dial = Gui.WaitQuestDialog(self)
            self.statusbar_frame = None
            self.endproc_frame = self.waiting_dial.onEndProcess


        self.Show()
        self.initFolders()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        self.SetBackgroundColour(BACKGROUND_COLOR)
        self.SetForegroundColour(FOREGROUND_COLOR)

        self.qpanel = QuestPanel(self)
        self.SetTitle("Process Quest")

        self.Layout()
        screen_size = wx.DisplaySize()
        size = (screen_size[0]*2.5/(3*wx.Display.GetCount()),
                screen_size[1]*2/3)
        self.SetSize(size)
        self.Centre()

    # -------------------------------------------------------------------
    def initToolbar(self) -> None:
        toolbar = self.CreateToolBar(style=wx.TB_DEFAULT_STYLE|wx.TB_HORZ_TEXT)
        load_bitmap = wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_TOOLBAR)
        save_bitmap = wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE_AS, wx.ART_TOOLBAR)
        confirm_bit = wx.ArtProvider.GetBitmap(wx.ART_TICK_MARK, wx.ART_TOOLBAR)

        # elements
        load = toolbar.AddTool(wx.ID_ANY, 'Load Parameters', load_bitmap)
        save = toolbar.AddTool(wx.ID_ANY, 'Save Parameters', save_bitmap)
        toolbar.AddSeparator()
        conf = toolbar.AddTool(wx.ID_ANY, 'Process CQUEST', confirm_bit)
        toolbar.Realize()

        # events
        self.Bind(wx.EVT_TOOL, self.qpanel.onLoadParameters, load)
        self.Bind(wx.EVT_TOOL, self.qpanel.onSaveParameters, save)
        self.Bind(wx.EVT_TOOL, self.onProcess, conf)

    # -------------------------------------------------------------------
    def initFolders(self) -> None:
        """
        Open a filedialog to choose a quest output folder, and create a basis
        folder. If this folder already exist, ask if delete or not. If not
        deleted, abort the process quest action.
        """
        with wx.DirDialog(self, "Choose a QUEST output folder",
                           style=wx.DD_DEFAULT_STYLE) as dd:
            if dd.ShowModal() == wx.ID_CANCEL:
                self.Close()
                return

            basis_path = os.path.join(dd.GetPath(), 'Basis')
            if os.path.exists(basis_path):
                msg = " --- " + basis_path + " ---\r\n" + \
                      "This folder already exist. ALL its content will be DELETED. Proceed ?"
                with wx.MessageDialog(self, msg, 'Warning',
                                      wx.YES_NO|wx.NO_DEFAULT|wx.ICON_EXCLAMATION) as md :
                    if md.ShowModal() == wx.ID_YES:
                        shutil.rmtree(basis_path)
                    else:
                        self.Close()
                        return
            os.mkdir(basis_path)

            self.where_to_save = dd.GetPath()
            self.qpanel.wts_basis = basis_path

    # -------------------------------------------------------------------
    def addParserFiles(self, args):
        """
        Add signals from parser argument.
        """
        sigs = args.files
        metas = args.metafiles
        macros = args.macrofiles
        sigs = [os.path.abspath(p) for p in sigs]
        metas = [os.path.abspath(p) for p in metas]
        macros = [os.path.abspath(p) for p in macros]

        self.qpanel.addPaths('sig', sigs)
        self.qpanel.addPaths('met', metas)
        self.qpanel.addPaths('mac', macros)

    # -------------------------------------------------------------------
    def preventUserOnProcess(self) -> None:
        """
        feedback for the user so he know that quest is running.
        """
        if self.statusbar_frame is not None:
            self.statusbar_frame.setDefaultMessage("QUEST is running ...", 1)
        else:
            self.waiting_dial.ShowModal()

    # -------------------------------------------------------------------
    def onProcess(self, event) -> None:
        """
        Launch quest on vip or locally, according to the prefernces stored in
        the main app.
        """
        self.qparam = self.qpanel.getParameters()
        b = False
        if wx.GetApp().pref.is_quest_local:
            b = self._processLocal()
        else:
            b = self._processVip()
        # signal to user
        if b:
            self.preventUserOnProcess()

        self.Close()

    # -------------------------------------------------------------------
    def _processLocal(self) -> bool:
        """
        Process quest locally. This function launch a thread.
        """
        wts = self.where_to_save

        # do QUEST path really exist ?
        if not os.path.isdir(wx.GetApp().pref.quest_soft_path):
            wx.MessageBox("QUEST path is not valid. Aborted.", 'Warning')
            self.Close()
            return False

        # Fill basis folder
        for meta in self.qpanel.model.getMetaList():
            meta.signal.save(file_type='mrui')
        for macro in self.qpanel.model.getMacroList():
            macro.signal.save(file_type='mrui')

        # get signals to process, create their directory and save them as mrui
        sigs = self.qpanel.model.getSigList()
        files = []
        for elm in sigs:
            new_dir = os.path.join(wts, elm.name)
            assert new_dir == os.path.dirname(elm.signal.path)

            if os.path.exists(new_dir):
                msg = " --- " + new_dir + " ---\r\n" + \
                      "This folder already exist. ALL its content will be DELETED. Proceed ?"
                with wx.MessageDialog(self, msg, 'Warning',
                                      wx.YES_NO|wx.NO_DEFAULT|wx.ICON_EXCLAMATION) as md :
                    if md.ShowModal() == wx.ID_YES:
                        shutil.rmtree(new_dir)
                    else:
                        continue

            os.mkdir(new_dir) 
            elm.signal.save(file_type='mrui')
            files.append(elm.signal.path)

        # param file
        param_path = os.path.join(wts, 'param_quest.txt')
        self.qparam.save(param_path)

        # exec quest async
        if len(files) > 0:
            QuestThread(wx.GetApp().pref.quest_soft_path,
                        self.endproc_frame, param_path,
                        files, wts, wx.GetApp().cwd)
        return True

    # -------------------------------------------------------------------
    def _processVip(self) -> None:
        """
        process quest on VIP. This function launch a thread.
        """
        wts = self.where_to_save

        # get signals to process, create their directory and save them as mrui
        sigs = self.qpanel.model.getSigList()
        files = []
        for elm in sigs:
            new_dir = os.path.join(wts, elm.name)
            assert new_dir == os.path.dirname(elm.signal.path)

            if os.path.exists(new_dir):
                msg = " --- " + new_dir + " ---\r\n" + \
                      "This folder already exist. ALL its content will be DELETED. Proceed ?"
                with wx.MessageDialog(self, msg, 'Warning',
                                      wx.YES_NO|wx.NO_DEFAULT|wx.ICON_EXCLAMATION) as md :
                    if md.ShowModal() == wx.ID_YES:
                        shutil.rmtree(new_dir)
                    else:
                        continue

            os.mkdir(new_dir) 
            elm.signal.save(file_type='mrui')
            files.append(elm.signal.path)

        # Fill basis folder
        for meta in self.qpanel.model.getMetaList():
            meta.signal.save(file_type='mrui')
        for macro in self.qpanel.model.getMacroList():
            macro.signal.save(file_type='mrui')

        # Create the basis zip folder from the quest parameters file
        base_path = tempfile.mkdtemp(prefix='basis_')
        deep_base_path = os.path.join(base_path, 'basis')
        os.mkdir(deep_base_path)
        for m in self.qparam.met:
            name = m[1]+".mrui"
            shutil.copyfile(m[0], os.path.join(deep_base_path, name))
            vip_path = "./basis/" + name
            m[0] = vip_path
        for m in self.qparam.macro:
            name = m[1]+".mrui"
            shutil.copyfile(m[0], os.path.join(deep_base_path, name))
            vip_path = "./basis/" + name
            m[0] = vip_path
        shutil.make_archive(base_path, 'zip', base_path)
        param_path = self.qparam.save_temp()

        # launch thread
        VipThread(self.endproc_frame,
                  wx.GetApp().pref.apikey,
                  param_path, base_path+'.zip', files, wts)

        return True

# =======================================================================

class OpenBasisDialog(wx.Dialog):
    """
    Not used for now. This dialog was designed to allow user to load a basis
    folder when opening a ProcessQuestFrame.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.initGui()

    # -------------------------------------------------------------------
    def initGui(self) -> None:
        # elements
        msg = " --- " + loc + " ---\r\n" + \
              "This basis folder already exist."
        pnl = wx.Panel(self)
        txt = wx.Statictext(pnl, label=msg)
        cancel = wx.Button(pnl, label="Cancel")
        load = wx.Button(pnl, label="Load")
        load.SetHelpText(helptxt)
        replace = wx.Button(pnl, label="Replace")

        # sizers
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        btn_flags = [1, wx.ALL, 8]
        hbox.AddMany([(cancel, *btn_flags),
                      (load, *btn_flags),
                      (replace, *btn_flags)])
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(txt, 1, wx.ALIGN_CENTER_VERTICAL, 10)
        sizer.Add(hbox, 0, wx.CENTER|wx.ALL, 5)

        # events
        msg = 'Will override current files if pre-processing tools are used'
        load.Bind(wx.EVT_ENTER_WINDOW, lambda evt: self.onEnterBtn(evt, msg))
        load.Bind(wx.EVT_BUTTON, self.onLoad)

        msg = 'DELETE the current basis folder and create a new one when' + \
              'loading molecules.'
        replace.Bind(wx.EVT_ENTER_WINDOW, lambda evt: self.onEnterBtn(evt, msg))
        replace.Bind(wx.EVT_BUTTON, self.onReplace)

        cancel.Bind(wx.EVT_BUTTON, self.onCancel)

    # -------------------------------------------------------------------
    def onCancel(self, event) -> None:
        self.EndModal(wx.ID_CANCEL)

    # -------------------------------------------------------------------
    def onLoad(self, event) -> None:
        self.EndModal(wx.ID_FILE)

    # -------------------------------------------------------------------
    def onReplace(self, event) -> None:
        self.EndModal(wx.ID_REPLACE)

    # -------------------------------------------------------------------
    def onEnterBtn(self, event, msg) -> None:
        win = event.GetEventObject()
        wx.TipWindow(win, msg)

# =======================================================================

class QuestViewerSingleton(wx.Frame):
    """
    Basicelly just a frame wrapping a QuestViewerPanel.
    """

    # -------------------------------------------------------------------
    def __init__(self, pathname, *args, **kwargs)->None:
        super().__init__(*args, **kwargs)
        self.SetBackgroundColour(BACKGROUND_COLOR)
        self.SetForegroundColour(FOREGROUND_COLOR)
        self.vpanel = Gui.QuestViewerPanel(self)
        self.vpanel.loadFolder(pathname)
        self.vpanel.draw()

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.vpanel, 1, wx.EXPAND)
        self.SetSizer(sizer)

        self.Layout()

# =======================================================================

if __name__ == '__main__':
    pass
