# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Model for the quest frame that represent a file stored in the app.
"""

from os.path import splitext, basename

# =======================================================================

class QuestSignalModel:
    """
    Represent a loaded file.
    Store a Signal instance, a matplotlib.lines.Line2D instance, the path to the
    original file, its name and the deisplayed signal index.
    """
    def __init__(self, sig=None, plot=None, origin=None, name='', index=0) -> None:
        self.signal = sig # signal instance
        self.plot = plot # matplotlib.lines.Line2D
        self.origin = origin # path to the first opened signal file
        self.name = name # name of the signal
        self.index = index # displayed index

    def getPath(self, index) -> str:
        """
        Return the path of the Signal instance.
        """
        return self.signal.path

# =======================================================================

class QuestModel:
    """
    3 list of QuestSignalModel instances.
    This is not a good idea to deal with them directly. Use pre defined
    functions instead.
    """
    # -------------------------------------------------------------------
    def __init__(self):
        self.__signals = []
        self.__meta = []
        self.__macro = []

    # -------------------------------------------------------------------
    def __add(self, lst, signal, plot, origin, name) -> None:
        if name is None:
            name = splitext(basename(signal.path))[0]
        if origin is None:
            origin = signal.path
        lst.append(QuestSignalModel(signal, plot, origin, name))

    def addSignal(self, signal, origin=None, plot=None) -> None:
        self.__add(self.__signals, signal, plot, origin, None)

    def addMeta(self, signal, origin=None, plot=None, name=None) -> None:
        self.__add(self.__meta, signal, plot, origin, name)

    def addMacro(self, signal, origin=None, plot=None, name=None) -> None:
        self.__add(self.__macro, signal, plot, origin, name)

    # -------------------------------------------------------------------
    def __del(self, lst, index) -> None:
        try:
            lst[index].plot.remove()
        except Exception:
            pass
        del lst[index]

    def delSignal(self, index) -> None:
        self.__del(self.__signals, index)

    def delMeta(self, index) -> None:
        self.__del(self.__meta, index)

    def delMacro(self, index) -> None:
        self.__del(self.__macro, index)

    # -------------------------------------------------------------------
    def __find(self, lst, path) -> int:
        """
        Find a file in lst by looking for similar path.
        """
        for i, sig in enumerate(lst):
            if path == sig.getPath():
                return i

    def findSignal(self, path) -> int:
        return self.__find(self.__signals, path)

    def findMeta(self, path) -> int:
        return self.__find(self.__meta, path)

    def findMacro(self, path) -> int:
        return self.__find(self.__macro, path)

    # -------------------------------------------------------------------
    def __getShown(self, lst, index) -> list:
        if index is None:
            index = [i for i in range(len(lst))]

        res = []
        for i in index:
            plot = lst[i].plot
            if plot is not None and plot.get_visible():
                res.append(i)
        return res

    def getShownSignal(self, index=None) -> list:
        return self.__getShown(self.__signals, index)

    def getShownMeta(self, index=None) -> list:
        return self.__getShown(self.__meta, index)

    def getShownMacro(self, index=None) -> list:
        return self.__getShown(self.__macro, index)

    # -------------------------------------------------------------------
    def getSigList(self) -> list:
        return self.__signals

    def getMetaList(self) -> list:
        return self.__meta

    def getMacroList(self) -> list:
        return self.__macro

    # -------------------------------------------------------------------
    def getSignalData(self, index) -> QuestSignalModel:
        return self.__signals[index]

    def getMetaData(self, index) -> QuestSignalModel:
        return self.__meta[index]

    def getMacroData(self, index) -> QuestSignalModel:
        return self.__macro[index]

    # -------------------------------------------------------------------
    def __getName(self, lst, index) -> list:
        res = []
        try:
            for i in index:
                res.append(lst[index].name)
        except TypeError:
            res.append(lst[index].name)
        return res

    def getSignalName(self, index) -> list:
        return self.__getName(self.__signals, index)

    def getMetaName(self, index) -> list:
        return self.__getName(self.__meta, index)

    def getMacroName(self, index) -> list:
        return self.__getName(self.__macro, index)

    # -------------------------------------------------------------------
    def clearAll(self) -> None:
        self.__signals = []
        self.__meta = []
        self.__macro = []

# =======================================================================

if __name__ == '__main__':
    pass
