# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

import matplotlib
matplotlib.use('WXAgg')
import matplotlib.backends.backend_wxagg as wxagg
from matplotlib.figure import Figure
import numpy as np

import wx
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin
from wx.lib import newevent

# =======================================================================

class CanvasPanel(wx.Panel):
    """
    An easy-to-insert canvas for matplotlib display in a wxPython
    environnement.
    """

    # -------------------------------------------------------------------
    def __init__(self, *args, **kwargs)->None:
        super(CanvasPanel, self).__init__(*args, **kwargs)
        # elements
        self.figure = Figure()
        self.canvas = wxagg.FigureCanvasWxAgg(self, -1, self.figure)
        self.axes = self.figure.add_subplot(111, label='1d')
        self.legend = None
        self.toolbar = wxagg.NavigationToolbar2WxAgg(self.canvas)
        self.status_bar = wx.StatusBar(self, style=wx.STB_SHOW_TIPS | \
                                       wx.STB_ELLIPSIZE_END | wx.FULL_REPAINT_ON_RESIZE)

        # value
        self.status_bar.SetStatusText(("x=0 y=0"))
        self.figure.subplots_adjust(0.08, 0.08, 0.98, 0.98)

        # events
        self.canvas.mpl_connect('motion_notify_event',
                                self.updateStatusBar)
        self.canvas.Bind(wx.EVT_ENTER_WINDOW, self.changeCursor)

        # sizers
        sizer = wx.BoxSizer(wx.VERTICAL)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(self.toolbar, 1, wx.EXPAND)
        hbox.Add(self.status_bar, 1, wx.EXPAND)
        sizer.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
        sizer.Add(hbox, 0, wx.EXPAND)

        self.SetSizer(sizer)
        self.Fit()

    # -------------------------------------------------------------------
    def changeCursor(self, event)->None:
        self.canvas.SetCursor(wxagg.wxc.StockCursor(wx.CURSOR_CROSS))

    # -------------------------------------------------------------------
    def updateStatusBar(self, event)->None:
        if event.inaxes:
            x, y = event.xdata, event.ydata
            self.status_bar.SetStatusText(("x="+str(x)+" y="+str(y)))
        
    # -------------------------------------------------------------------
    def refresh(self) -> None:
        self.axes.autoscale(True)
        self.axes.relim()
        self.axes.autoscale_view(True, True, True)
        self.canvas.draw()

    # -------------------------------------------------------------------
    def refresh_legend(self) -> None:
        if self.legend != None:
            self.legend.remove()
        self.legend = self.figure.legend()

    # -------------------------------------------------------------------
    def clear(self)->None:
        self.axes.clear()
        self.canvas.draw()

    # -------------------------------------------------------------------
    def draw1dSignal(self, sig, x_temp=True, x_freq=False, imag=0, idx_m=0):
        """
        Input :
            sig: Signal() instance
            x_temp: bool
            x_freq: bool
            imag: int
            idx_m: int
        Return :
            maplotlib.line.Line2D
        """
        if x_temp:
            data = sig.data[idx_m]
        elif x_freq:
            data = sig.data_fourier[idx_m]
        else:
            data = sig.data_fourier[idx_m]

        return self.draw1dData(data, sig.meta['size'], sig.meta['step'],
                               sig.meta['transFreq'], x_temp, x_freq, imag)

    # -------------------------------------------------------------------
    def draw1dData(self, data, size, step, transFreq, x_temp=True, x_freq=False, imag=0):
        """
        Input :
            data: complex 1d array to plot
            step, size, transFreq : info relative to data
            x_temp: bool
            x_freq: bool
            imag: int
        Return :
            maplotlib.line.Line2D
        """
        # set axes
        invert_axe = False
        self.axes.set_ylabel("Amplitude (Pa)")
        if x_temp: # time domain
            self.axes.set_xlabel("Time (s)")
        else:
            if x_freq:
                self.axes.set_xlabel("Frequency (Hz)")
            else:
                self.axes.set_xlabel("PPM")
                invert_axe = True

        # draw
        if x_temp:
            try:
                f = np.linspace(0, size*step, size)
            except:
                f = None
        elif x_freq:
            try:
                Fe = 1/step
                f = np.linspace(-Fe/2, Fe/2, size)
            except:
                f = None
        else:
            try:
                Fe = 1/step
                car = transFreq
                f = np.linspace(-Fe*1e6/(2*car), Fe*1e6/(2*car), size)
            except:
                f = None

        # Part of complex number
        if imag == 0:
            data = data.real
        elif imag == 1:
            data = data.imag
        else:
            data = np.abs(data)

        # Actual display
        if f is not None:
            plot = self.axes.plot(f, data)
        else:
            plot = self.axes.plot(data)

        if invert_axe != self.axes.xaxis_inverted():
            self.axes.invert_xaxis()

        return plot

# =======================================================================

class AutoWidthListCtrl(wx.ListCtrl, ListCtrlAutoWidthMixin):
    """
    A wxListCtrl which has auto column resize. It's possible to pimp it more
    by adding more inheritance to this class. See wxPython doc for more
    precision.
    """

    # -------------------------------------------------------------------
    def __init__(self, parent)->None:
        wx.ListCtrl.__init__(self, parent, style=wx.LC_REPORT|wx.LC_HRULES)
        ListCtrlAutoWidthMixin.__init__(self)

# """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

PimpedListEvent, EVT_PMPD_LIST_EVENT = newevent.NewEvent()

# """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

class PimpedListCtrl(wx.Panel):
    # -------------------------------------------------------------------
    def __init__(self, parent, columns=['Column'], label="", editable=True)->None:
        super().__init__(parent)
        self.initGui(label, columns)
        if not editable:
            self.head_sizer.Hide(self.edit_btn)

        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self, label, columns) -> None:
        # elements
        header = wx.Panel(self, style=wx.BORDER_RAISED)
        art = wx.ArtProvider()
        font = wx.Font().Bold()
        self.label = wx.StaticText(header, label=label, 
                                   style=wx.ALIGN_LEFT|wx.ST_ELLIPSIZE_END)
        self.label.SetFont(font)
        self.add_btn = wx.BitmapButton(header,
                                       bitmap=art.GetBitmap(wx.ART_PLUS,
                                                    client=wx.ART_BUTTON) )
        self.del_btn = wx.BitmapButton(header,
                                       bitmap=art.GetBitmap(wx.ART_MINUS,
                                                    client=wx.ART_BUTTON) )
        self.edit_btn = wx.BitmapButton(header,
                                        bitmap=art.GetBitmap(wx.ART_FIND_AND_REPLACE,
                                                    client=wx.ART_BUTTON) )
        header.SetBackgroundColour((242, 242, 242))
        self.list = AutoWidthListCtrl(self)
        for i, name in enumerate(columns):
            self.list.InsertColumn(i, name)
        
        # bindings
        self.add_btn.Bind(wx.EVT_BUTTON, self.onAdd)
        self.del_btn.Bind(wx.EVT_BUTTON, self.onDel)
        self.edit_btn.Bind(wx.EVT_BUTTON, self.onEdit)

        # sizers
        self.head_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.head_sizer.Add(self.label, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, 10)
        self.head_sizer.AddStretchSpacer(prop=1)
        self.head_sizer.Add(self.add_btn, 0)
        self.head_sizer.Add(self.del_btn, 0)
        self.head_sizer.Add(self.edit_btn, 0)
        header.SetSizerAndFit(self.head_sizer)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(header, 0, wx.RIGHT|wx.EXPAND, 0)
        sizer.Add(self.list, 1, wx.ALL|wx.EXPAND, 0)
        self.SetSizerAndFit(sizer)

    # -------------------------------------------------------------------
    def onAdd(self, event):
        evt = PimpedListEvent(action='add')
        wx.PostEvent(self, evt)

    # -------------------------------------------------------------------
    def onDel(self, event):
        evt = PimpedListEvent(action='del')
        wx.PostEvent(self, evt)
            
    # -------------------------------------------------------------------
    def onEdit(self, event):
        evt = PimpedListEvent(action='edt')
        wx.PostEvent(self, evt)

    # -------------------------------------------------------------------
    def GetSelections(self) -> list:
        res = []
        elm = self.list.GetFirstSelected()
        while elm >= 0:
            res.append(elm)
            elm = self.list.GetNextSelected(elm)
        return sorted(res)
    
    # -------------------------------------------------------------------
    def SetSelection(self, index) -> None:
        self.clearSelection()
        try:
            for i in index:
                self.list.Select(i)
        except TypeError:
            self.list.Select(index)
    # -------------------------------------------------------------------
    def clearSelection(self) -> None:
        for i in self.GetSelections():
            self.list.Select(i, on=0)
    # -------------------------------------------------------------------
    def GetSelection(self) -> int:
        return self.list.GetFocusedItem()

    # -------------------------------------------------------------------
    def Delete(self, index) -> None:
        self.list.DeleteItem(index)

# =======================================================================

class ChoiceListBox(wx.Panel):

    # -------------------------------------------------------------------
    def __init__(self, parent, label="", style=wx.LB_EXTENDED, choices=[])->None:
        super().__init__(parent)
        self.initGui(label, style, choices)
        self.Layout()

    # -------------------------------------------------------------------
    def initGui(self, label, style, choices) -> None:
        # elements
        header = wx.Panel(self, style=wx.BORDER_RAISED)
        art = wx.ArtProvider()
        font = wx.Font().Bold()
        self.label = wx.StaticText(header, label=label, 
                                   style=wx.ALIGN_LEFT|wx.ST_ELLIPSIZE_END)
        self.label.SetFont(font)
        self.add_btn = wx.BitmapButton(header,
                                       bitmap=art.GetBitmap(wx.ART_PLUS,
                                                    client=wx.ART_BUTTON) )
        self.del_btn = wx.BitmapButton(header,
                                       bitmap=art.GetBitmap(wx.ART_MINUS,
                                                    client=wx.ART_BUTTON) )
        header.SetBackgroundColour((242, 242, 242))
        self.list = wx.ListBox(self, style=style)
        choice_panel = wx.Panel(self)
        self._list_choice = wx.ListBox(choice_panel, style=wx.LB_MULTIPLE,
                                       choices=choices)
        ok_choice = wx.Button(choice_panel, label="Confirm")
        cancel_choice = wx.Button(choice_panel, label="Cancel")
        
        # bindings
        self.add_btn.Bind(wx.EVT_BUTTON, self.onAdd)
        self.del_btn.Bind(wx.EVT_BUTTON, self.onDel)
        ok_choice.Bind(wx.EVT_BUTTON, self.onConfirmChoice)
        cancel_choice.Bind(wx.EVT_BUTTON, self.onCancelChoice)

        # sizers
        head_sizer = wx.BoxSizer(wx.HORIZONTAL)
        head_sizer.Add(self.label, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, 10)
        head_sizer.AddStretchSpacer(prop=1)
        head_sizer.Add(self.add_btn, 0)
        head_sizer.Add(self.del_btn, 0)
        header.SetSizerAndFit(head_sizer)

        choice_sizer = wx.BoxSizer(wx.VERTICAL)
        btn_sizer = wx.BoxSizer(wx.HORIZONTAL)
        btn_sizer.AddStretchSpacer(prop=1)
        btn_sizer.Add(cancel_choice, 0, wx.ALL, 5)
        btn_sizer.Add(ok_choice, 0, wx.ALL, 5)
        choice_sizer.Add(self._list_choice, 1, wx.ALL|wx.EXPAND, 0)
        choice_sizer.Add(btn_sizer, 0)
        choice_panel.SetSizerAndFit(choice_sizer)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(header, 0, wx.RIGHT|wx.EXPAND, 0)
        self.sizer.Add(self.list, 1, wx.ALL|wx.EXPAND, 0)
        self.sizer.Add(choice_panel, 1, wx.ALL|wx.EXPAND, 0)
        self.sizer.Hide(2)
        self.SetSizerAndFit(self.sizer)

    # -------------------------------------------------------------------
    def onAdd(self, event):
        """
        This is advised to override this function if you want to add
        elements in a different way.
        """
        self.sizer.Hide(1)
        self.sizer.Show(2)
        self.Layout()

    # -------------------------------------------------------------------
    def onDel(self, event):
        if self.list.IsShown():
            todlt = self.list.GetSelections()
            for i in reversed(sorted(todlt)):
                self._list_choice.Append(self.list.GetString(i))
                self.list.Delete(i)

    # -------------------------------------------------------------------
    def setChoices(self, choices):
        for i in range(self.list.GetCount()-1, -1, -1):
            elm = self.list.GetString()
            try:
                ind = choices.index(elm)
            except ValueError:
                self.list.Delete(i)
            else:
                choices.remove(ind)

        self._list_choice.Clear()
        if len(choices) > 0:
            self._list_choice.InsertItems(choices, 0)

    # -------------------------------------------------------------------
    def setLabel(self, label):
        self.label.SetLabel(label)

    # -------------------------------------------------------------------
    def onConfirmChoice(self, event):
        sel = self._list_choice.GetSelections()
        for elm in reversed(sorted(sel)):
            self.list.Append(self._list_choice.GetString(elm))
            self._list_choice.Delete(elm)
        self.sizer.Hide(2)
        self.sizer.Show(1)
        self.Layout()

    # -------------------------------------------------------------------
    def onCancelChoice(self, event):
        self.sizer.Hide(2)
        self.sizer.Show(1)
        self.Layout()

    # -------------------------------------------------------------------
    def clear(self):
        self._list_choice.Clear()
        self.list.Clear()

# =======================================================================

class Waiting(wx.Gauge):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, lambda evt: self.Pulse())

    def start(self):
        self.timer.Start(200)
 
    def stop(self):
        self.timer.Stop()

# =======================================================================

if __name__ == '__main__':
    pass
