# -*-coding:Utf-8 -*

# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS : www.creatis.insa-lyon.fr
# Centre de Recherche en Acquisition et Traitement de l'Image pour la Santé
# 
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENCE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------

"""
Module that allow to interact with the app preferences.
"""

import os.path
import pickle
import wx

from .constants import SHOWCASE_COLOR

# =======================================================================

class AppPreferences:
    """
    Represent the preferences without any widgets. 
    Attribute :
        - location : relative path to the pref file. This variable is not
                    supposed to be edited, but can if needed.
    """


    # -------------------------------------------------------------------
    def __init__(self, path)->None:
        # All the preferences
        self.quest_soft_path = ""
        self.quest_file_path = ""
        self.apikey = ""
        self.is_quest_local = True

        self.location = os.path.join(path, "preferences.viewer")

    # -------------------------------------------------------------------
    def setDefaultConfig(self)->None:
        self.quest_soft_path = os.path.join(os.path.basename(self.location), "CQUEST")
        self.quest_file_path = os.path.join(os.path.basename(self.location), "default.quest")

    # -------------------------------------------------------------------
    def save(self)->None:
        """
        Save a pickle file.
        """
        with open(self.location, 'wb') as pref:
            pickle.dump(self.__dict__, pref)

    # -------------------------------------------------------------------
    def load(self)->None:
        """
        Pickled file.
        """

        with open(self.location, 'rb') as pref:
            try:
                tmp_dic = pickle.load(pref)
            except pickle.UnpicklingError as err: # _pickle.UnpicklingError:
                print("  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! " )
                print("  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! " )
                print("            ERROR WHILE READING PREFERENCE FILE : " + self.location + "\n")
                print("            CORRECT IT OR REMOVE IT"                                      )
                print("  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! " )
                print("  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! " )
                raise
#                err.message=err.message+"error while reading preference file : " + self.location + "\n"
#                err.message=err.message+"correct it or remove it"

            for key in self.__dict__:
                try:
                    self.__dict__[key] = tmp_dic[key]
                except AttributeError:
                    continue

# =======================================================================

class PreferencesDialog(wx.Dialog):
    """
    A dialog for edit app preferences.
    """

    # -------------------------------------------------------------------
    def __init__(self, pref, logStatus, *args, **kwargs)->None:
        """
        Args :
            - pref : AppPreferences variable that will be edited
            - logStatus : callback to display a message in MainFrame
        """
        super(PreferencesDialog, self).__init__(*args, **kwargs)
        self.app_pref = pref
        self.logStatus = logStatus

        self.initGui()
        self.Layout()
        self.Fit()

    # -------------------------------------------------------------------
    def initGui(self)->None:
        # elements
        pnl = wx.Panel(self)
        q_panel = wx.Panel(pnl)
        self.choice_quest = wx.RadioBox(q_panel, label="Quest location :",
                                        choices=["Local", "VIP"])
        qtext = wx.StaticText(q_panel, label="Path to QUEST software folder :")
        self.soft_input = wx.DirPickerCtrl(q_panel,
                                           path=self.app_pref.quest_soft_path)
        viptext = wx.StaticText(q_panel, label="Enter your VIP apikey :")
        self.apikey = wx.TextCtrl(q_panel, value=self.app_pref.apikey)

        ptext = wx.StaticText(pnl, label="Set a default QUEST parameters file :")
        self.param_input = wx.FilePickerCtrl(pnl)
        self.param_input.SetPath(self.app_pref.quest_file_path)

        save_button = wx.Button(pnl, label="Apply")
        save_button.SetBackgroundColour(SHOWCASE_COLOR)

        # event
        save_button.Bind(wx.EVT_BUTTON, self.onSavePref)
        self.choice_quest.Bind(wx.EVT_RADIOBOX, self.onChangeQuest)
        self.soft_input.Bind(wx.EVT_DIRPICKER_CHANGED, self.onBrowseCQuest)

        # sizer
        hbox_save = wx.BoxSizer(wx.HORIZONTAL)
        hbox_save.AddStretchSpacer(prop=1)
        hbox_save.Add(save_button, 0, wx.ALL, 5)

        self.qsizer = wx.BoxSizer(wx.VERTICAL)
        self.qsizer.Add(self.choice_quest, 0, wx.ALL, 5)
        self.qsizer.Add(qtext, 0, wx.ALL, 5)
        self.qsizer.Add(self.soft_input, 0, wx.EXPAND|wx.ALL, 5)
        self.qsizer.Add(viptext, 0, wx.ALL, 5)
        self.qsizer.Add(self.apikey, 0, wx.EXPAND|wx.ALL, 5)
        if self.app_pref.is_quest_local:
            self.qsizer.Hide(3)
            self.qsizer.Hide(4)
        else:
            self.qsizer.Hide(1)
            self.qsizer.Hide(2)
            self.choice_quest.SetSelection(1)
        self.qsizer.Layout()
        q_panel.SetSizerAndFit(self.qsizer)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(q_panel, 0, wx.EXPAND|wx.ALL, 5)
        sizer.Add(ptext, 0, wx.ALL, 5)
        sizer.Add(self.param_input, 0, wx.EXPAND|wx.ALL, 5)
        sizer.Add(hbox_save, 0, wx.EXPAND|wx.ALL, 5)

        pnl.SetSizerAndFit(sizer)

    # -------------------------------------------------------------------
    def onBrowseCQuest(self, event):
        if not isCQuestPathValid(event.GetPath()):
            wx.MessageBox("Folder may not contain necessary elements",
                           'Warning', style=wx.OK|wx.ICON_WARNING)

    # -------------------------------------------------------------------
    def onChangeQuest(self, event)->None:
        """
        Switch between 'local' and 'VIP' preferences display.
        """
        if event.GetInt() == 0:
            self.qsizer.Show(1)
            self.qsizer.Show(2)
            self.qsizer.Hide(3)
            self.qsizer.Hide(4)
        else:
            self.qsizer.Show(3)
            self.qsizer.Show(4)
            self.qsizer.Hide(1)
            self.qsizer.Hide(2)
        self.qsizer.Layout()
        self.Layout()

    # -------------------------------------------------------------------
    def onSavePref(self, event)->None:
        self.app_pref.quest_soft_path = self.soft_input.GetPath()
        self.app_pref.quest_file_path = self.param_input.GetPath()
        self.app_pref.is_quest_local = not bool(self.choice_quest.GetSelection())
        self.app_pref.apikey = self.apikey.GetLineText(0)

        self.app_pref.save()
        self.logStatus("Preferences have been saved", 2, 3)

# =======================================================================

def isCQuestPathValid(path):
    return (os.path.isfile(os.path.join(path, 'cquestCml')) and
            os.path.isfile(os.path.join(path, 'fft.jar')) and
            os.path.isfile(os.path.join(path, 'cQuest_V1_lin.jar')))

# =======================================================================

if __name__ == "__main__":
    pass
